@@ -3,8 +3,8 @@
  *
  *	This file contains basic support for handling addresses.
  *
- * TkRat software and its included text is Copyright 1996,1997,1998
- * by Martin Forssn
+ * TkRat software and its included text is Copyright 1996-2000 by
+ * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
  * COPYRIGHT, included with this distribution.
@@ -18,23 +18,22 @@
  * This struct defines an alias.
  */
 typedef struct {
-    char *book;		/* Address book this alias comes from */
-    char *fullname;	/* Long name of alias (phrase part) */
-    char *content;	/* Content that alias expands to */
-    char *comment;	/* Comment to alias */
+    Tcl_Obj *book;	/* Address book this alias comes from */
+    Tcl_Obj *fullname;	/* Long name of alias (phrase part) */
+    Tcl_Obj *content;	/* Content that alias expands to */
+    Tcl_Obj *comment;	/* Comment to alias */
+    unsigned int flags;	/* Option flags */
     unsigned long mark;	/* Id of last use */
 } AliasInfo;
 
+#define ALIAS_FLAG_ISLIST	(1<<0)
+#define ALIAS_FLAG_NOFULLNAME	(1<<1)
+
 /*
  * This table contains all the aliases.
  */
 Tcl_HashTable aliasTable;
 
-/*
- * This table contains all the users
- */
-Tcl_HashTable userTable;
-
 /*
  * The number of address entities created. This is used to create new
  * unique command names.
@@ -49,34 +48,39 @@ static unsigned long aliasMark = 0;
 /*
  * Internal functions
  */
-static void AddressStrcpy(char *dst, const char *src);
+static int AddressClean(Tcl_Obj *aPtr);
+
+#ifdef MEM_DEBUG
+static char *mem_store;
+#endif /* MEM_DEBUG */
 
 
 /*
  *----------------------------------------------------------------------
  *
- * AddressStrcpy --
+ * AddressClean --
  *
- *      Copy an address list and eliminate all space around te address
- *	entities.
+ *      Clean an address list by removing all whitespace around addresses
  *
  * Results:
- *	None
+ *	The number of addresses contained in the object
  *
  * Side effects:
- *	None
+ *	Modifies the given object
  *
  *
  *----------------------------------------------------------------------
  */
 
-static void
-AddressStrcpy(char *dst, const char *src)
+static int
+AddressClean(Tcl_Obj *aPtr)
 {
-    char *mark = dst;
+    char *mark, *dst, *src, *new;
     int quoted = 0;
-    int skip = 1;
+    int skip = 1, length, num = 1;
 
+    src = Tcl_GetStringFromObj(aPtr, &length);
+    new = dst = mark = (char*)ckalloc(length);
     for (; *src; src++) {
 	if ('\\' == *src) {
 	    *dst++ = *src++;
@@ -89,9 +93,10 @@ AddressStrcpy(char *dst, const char *src)
 	    }
 	} else if (!quoted) {
 	    if (',' == *src) {
+		num++;
 		dst = mark;
 		skip = 1;
-	    } else if (isspace(*src)) {
+	    } else if (isspace((unsigned char)*src)) {
 		if (skip) {
 		    continue;
 		}
@@ -102,7 +107,54 @@ AddressStrcpy(char *dst, const char *src)
 	}
 	*dst++ = *src;
     }
-    *mark = '\0';
+    if (0 == mark-new) {
+	num = 0;
+    }
+    Tcl_SetStringObj(aPtr, new, mark-new);
+    ckfree(new);
+    return num;
+}
+
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * RatCreateAddressCmd --
+ *
+ *      This routine creates an address command by an address given
+ *	as argument
+ *
+ * Results:
+ *	A list of address entity names is appended to the result
+ *
+ * Side effects:
+ *	New address entities are created,
+ *
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+RatCreateAddressCmd(ClientData clientData, Tcl_Interp *interp, int objc,
+	Tcl_Obj *CONST objv[])
+{
+    ADDRESS *adrPtr = NULL;
+    char *s;
+
+    if (objc != 2) {
+	Tcl_AppendResult(interp, "wrong # args: should be \"",
+		Tcl_GetString(objv[0]), " address\"", (char *) NULL);
+	return TCL_ERROR;
+    }
+
+    s = cpystr(Tcl_GetString(objv[1]));
+    rfc822_parse_adrlist(&adrPtr, s, currentHost);
+    ckfree(s);
+    RatEncodeAddresses(interp, adrPtr);
+    RatInitAddresses(interp, adrPtr);
+    mail_free_address(&adrPtr);
+    return TCL_OK;
 }
 
 
@@ -115,7 +167,7 @@ AddressStrcpy(char *dst, const char *src)
  *	list of address entities of it.
  *
  * Results:
- *	A list of address entity names is appended to interp->result.
+ *	A list of address entity names is appended to the result
  *
  * Side effects:
  *	New address entities are created,
@@ -129,15 +181,15 @@ RatInitAddresses(Tcl_Interp *interp, ADDRESS *addressPtr)
 {
     ADDRESS *adrPtr, *newPtr;
     Tcl_DString result;
-    char name[32], *s;
+    char name[32];
 
     Tcl_DStringInit(&result);
     Tcl_DStringGetResult(interp, &result);
     for (adrPtr = addressPtr; adrPtr; adrPtr = adrPtr->next) {
 	newPtr = mail_newaddr();
 	if (adrPtr->personal)	{
-	    s = RatDecodeHeader(interp, adrPtr->personal);
-	    newPtr->personal = cpystr(RatDecodeHeaderFull(interp, s));
+	    newPtr->personal =
+		    cpystr(RatDecodeHeader(interp, adrPtr->personal, 0));
 	}
 	if (adrPtr->adl)	newPtr->adl = cpystr(adrPtr->adl);
 	if (adrPtr->mailbox)	newPtr->mailbox = cpystr(adrPtr->mailbox);
@@ -173,6 +225,8 @@ int
 RatAddress(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
 {
     ADDRESS *adrPtr = (ADDRESS*)clientData;
+    Tcl_Obj *oPtr;
+    int useup;
 
     if (argc < 2) {
 	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
@@ -180,7 +234,12 @@ RatAddress(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
 	return TCL_ERROR;
     }
     if (!strcmp(argv[1], "isMe")) {
-	if (RatAddressIsMe(interp, adrPtr, 1)) {
+	if (3 == argc) {
+	    Tcl_GetBoolean(interp, argv[2], &useup);
+	} else {
+	    useup = 1;
+	}
+	if (RatAddressIsMe(interp, adrPtr, useup)) {
 	    Tcl_SetResult(interp, "1", TCL_STATIC);
 	} else {
 	    Tcl_SetResult(interp, "0", TCL_STATIC);
@@ -212,12 +271,12 @@ RatAddress(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
 		    " set personal name host\"", (char *) NULL);
 	    return TCL_ERROR;
 	}
-	free(adrPtr->mailbox);
+	ckfree(adrPtr->mailbox);
 	if (adrPtr->personal) {
-	    free(adrPtr->personal);
+	    ckfree(adrPtr->personal);
 	}
 	if (adrPtr->host) {
-	    free(adrPtr->host);
+	    ckfree(adrPtr->host);
 	}
 	adrPtr->personal = cpystr(argv[2]);
 	adrPtr->mailbox = cpystr(argv[3]);
@@ -232,13 +291,16 @@ RatAddress(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
 	}
 	if (!strcasecmp(argv[2], "rfc822")) {
 	    if (adrPtr->personal) {
-		char *charset = Tcl_GetVar2(interp, "option", "charset",
-			TCL_GLOBAL_ONLY);
-		char *personal = (char*)RatEncodeHeaderLine(
-	    		(unsigned char*)adrPtr->personal, charset, 0, 1);
-		Tcl_ResetResult(interp);
-		sprintf(interp->result, "%s <%s>", personal,
-			RatAddressMail(adrPtr));
+		char *personal;
+
+		oPtr = Tcl_NewStringObj(adrPtr->personal, -1);
+		personal = RatEncodeHeaderLine(interp, oPtr, 0);
+		Tcl_DecrRefCount(oPtr);
+		oPtr = Tcl_NewObj();
+		Tcl_AppendStringsToObj(oPtr, personal, " <", NULL);
+		Tcl_AppendToObj(oPtr, RatAddressMail(adrPtr), -1);
+		Tcl_AppendToObj(oPtr, ">", 1);
+		Tcl_SetObjResult(interp, oPtr);
 		ckfree(personal);
 	    } else {
 		Tcl_SetResult(interp, RatAddressMail(adrPtr), TCL_VOLATILE);
@@ -288,11 +350,11 @@ void
 RatDeleteAddress(ClientData clientData)
 {
     ADDRESS *adrPtr = (ADDRESS*)clientData;
-    if (adrPtr->personal)	free(adrPtr->personal);
-    if (adrPtr->adl)		free(adrPtr->adl);
-    if (adrPtr->mailbox)	free(adrPtr->mailbox);
-    if (adrPtr->host)		free(adrPtr->host);
-    if (adrPtr->error)		free(adrPtr->error);
+    if (adrPtr->personal)	ckfree(adrPtr->personal);
+    if (adrPtr->adl)		ckfree(adrPtr->adl);
+    if (adrPtr->mailbox)	ckfree(adrPtr->mailbox);
+    if (adrPtr->host)		ckfree(adrPtr->host);
+    if (adrPtr->error)		ckfree(adrPtr->error);
     ckfree(adrPtr);
 }
 
@@ -328,7 +390,9 @@ RatAddressIsMe(Tcl_Interp *interp, ADDRESS *adrPtr, int useUP)
     if (NULL != (from = Tcl_GetVar2(interp,"option","from",TCL_GLOBAL_ONLY))
 	    && *from != '\0') {
 	ADDRESS *a = NULL;
-	rfc822_parse_adrlist(&a, from, currentHost);
+	char *s = cpystr(from);
+	rfc822_parse_adrlist(&a, s, currentHost);
+	ckfree(s);
 	if (a && adrPtr->mailbox && adrPtr->host
 		&& !strcasecmp(a->mailbox, adrPtr->mailbox)
 		&& !strcasecmp(a->host, adrPtr->host)) {
@@ -342,6 +406,8 @@ RatAddressIsMe(Tcl_Interp *interp, ADDRESS *adrPtr, int useUP)
 	if (Tcl_GetCommandInfo(interp, "RatUP_IsMe", &cmdInfo)) {
 	    Tcl_DString cmd;
 	    int isMe;
+	    Tcl_Obj *oPtr;
+
 	    Tcl_DStringInit(&cmd);
 	    Tcl_DStringAppendElement(&cmd, "RatUP_IsMe");
 	    Tcl_DStringAppendElement(&cmd,adrPtr->mailbox?adrPtr->mailbox:"");
@@ -349,7 +415,8 @@ RatAddressIsMe(Tcl_Interp *interp, ADDRESS *adrPtr, int useUP)
 	    Tcl_DStringAppendElement(&cmd,adrPtr->personal?adrPtr->personal:"");
 	    Tcl_DStringAppendElement(&cmd,adrPtr->adl?adrPtr->adl:"");
 	    if (TCL_OK == Tcl_Eval(interp, Tcl_DStringValue(&cmd))
-		    && TCL_OK == Tcl_GetBoolean(interp, interp->result, &isMe)){
+	    	    && (oPtr = Tcl_GetObjResult(interp))
+		    && TCL_OK == Tcl_GetBooleanFromObj(interp, oPtr, &isMe)) {
 		Tcl_DStringFree(&cmd);
 		return isMe;
 	    }
@@ -413,8 +480,9 @@ RatAddressTranslate(Tcl_Interp *interp, ADDRESS *adrPtr)
 {
     Tcl_CmdInfo cmdInfo;
     Tcl_DString cmd;
-    char **argv;
-    int argc;
+    Tcl_Obj *oPtr, *lPtr;
+    char **destPtrPtr = NULL, *s;
+    int argc, i;
 
     if (!Tcl_GetCommandInfo(interp, "RatUP_Translate", &cmdInfo)) {
 	return;
@@ -426,55 +494,31 @@ RatAddressTranslate(Tcl_Interp *interp, ADDRESS *adrPtr)
     Tcl_DStringAppendElement(&cmd,adrPtr->personal?adrPtr->personal:"");
     Tcl_DStringAppendElement(&cmd,adrPtr->adl?adrPtr->adl:"");
     if (TCL_OK != Tcl_Eval(interp, Tcl_DStringValue(&cmd))
-    	    || TCL_OK != Tcl_SplitList(interp, interp->result, &argc, &argv)
+	    || !(lPtr = Tcl_GetObjResult(interp))
+    	    || TCL_OK != Tcl_ListObjLength(interp, lPtr, &argc)
 	    || 4 != argc) {
-	char buf[2048];
-	sprintf(buf, "%s: %s", Tcl_GetVar2(interp, "t", "translate_error",
-		TCL_GLOBAL_ONLY), Tcl_DStringValue(&cmd));
-	RatLog(interp, RAT_ERROR, buf, 0);
+	RatLogF(interp, RAT_ERROR, "translate_error", RATLOG_TIME,
+		Tcl_DStringValue(&cmd));
     } else {
-	if (   (*argv[0]&&(!adrPtr->mailbox||strcmp(argv[0],adrPtr->mailbox)))
-	    || (!*argv[0] && adrPtr->mailbox)) {
-	    if (adrPtr->mailbox) {
-		free(adrPtr->mailbox);
+	for (i=0; i<4; i++) {
+	    switch(i) {
+		case 0: destPtrPtr = &adrPtr->mailbox; break;
+		case 1: destPtrPtr = &adrPtr->host; break;
+		case 2: destPtrPtr = &adrPtr->personal; break;
+		case 3: destPtrPtr = &adrPtr->adl; break;
 	    }
-	    if (*argv[0]) {
-		adrPtr->mailbox = cpystr(argv[0]);
-	    } else {
-		adrPtr->mailbox = NULL;
-	    }
-	}
-	if (   (*argv[1]&&(!adrPtr->host||strcmp(argv[1],adrPtr->host)))
-	    || (!*argv[1] && adrPtr->host)) {
-	    if (adrPtr->host) {
-		free(adrPtr->host);
-	    }
-	    if (*argv[1]) {
-		adrPtr->host = cpystr(argv[1]);
-	    } else {
-		adrPtr->host = cpystr(currentHost);
-	    }
-	}
-	if (   (*argv[2]&&(!adrPtr->personal||strcmp(argv[2],adrPtr->personal)))
-	    || (!*argv[2] && adrPtr->personal)) {
-	    if (adrPtr->personal) {
-		free(adrPtr->personal);
-	    }
-	    if (*argv[2]) {
-		adrPtr->personal = cpystr(argv[2]);
-	    } else {
-		adrPtr->personal = NULL;
-	    }
-	}
-	if (   (*argv[3]&&(!adrPtr->adl||strcmp(argv[3],adrPtr->adl)))
-	    || (!*argv[3] && adrPtr->adl)) {
-	    if (adrPtr->adl) {
-		free(adrPtr->adl);
-	    }
-	    if (*argv[3]) {
-		adrPtr->adl = cpystr(argv[3]);
-	    } else {
-		adrPtr->adl = NULL;
+	    Tcl_ListObjIndex(interp, lPtr, i, &oPtr);
+	    s = Tcl_GetString(oPtr);
+	    if (   (*s && (!(*destPtrPtr) || strcmp(s,*destPtrPtr)))
+		|| (!*s && *destPtrPtr)) {
+		if (*destPtrPtr) {
+		    ckfree(*destPtrPtr);
+		}
+		if (*s) {
+		    *destPtrPtr = cpystr(s);
+		} else {
+		    *destPtrPtr = NULL;
+		}
 	    }
 	}
     }
@@ -484,7 +528,7 @@ RatAddressTranslate(Tcl_Interp *interp, ADDRESS *adrPtr)
 /*
  *----------------------------------------------------------------------
  *
- * RatAlias --
+ * RatAliasCmd --
  *
  *      Implements the RatAlias command as per ../doc/interface
  *
@@ -499,198 +543,252 @@ RatAddressTranslate(Tcl_Interp *interp, ADDRESS *adrPtr)
  */
 
 int
-RatAlias(ClientData dummy, Tcl_Interp *interp, int argc, char *argv[])
+RatAliasCmd(ClientData dummy, Tcl_Interp *interp,int objc,Tcl_Obj *CONST objv[])
 {
-    if (argc < 2) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-		" option ?arg?\"", (char *) NULL);
+    if (objc < 2) {
+	Tcl_AppendResult(interp, "wrong # args: should be \"",
+		Tcl_GetString(objv[0]), " option ?arg?\"", (char *) NULL);
 	return TCL_ERROR;
     }
-    if (!strcmp(argv[1], "add")) {
+    if (!strcmp(Tcl_GetString(objv[1]), "add")) {
 	AliasInfo *aliasPtr;
 	Tcl_HashEntry *entryPtr;
-	char *key, *book, *comment, *cPtr;
-	int new, s;
+	char *key;
+	int new;
 
-	if (argc < 5 || argc > 7) {
-	    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-		    " add book name fullname content comment\"", (char *) NULL);
-	    return TCL_ERROR;
+	aliasPtr = (AliasInfo*)ckalloc(sizeof(AliasInfo));
+	switch (objc) {
+	    case 5:
+		aliasPtr->book = Tcl_NewStringObj("Personal", -1);
+		key = Tcl_GetString(objv[2]);
+		aliasPtr->fullname = objv[3];
+		aliasPtr->content = objv[4];
+		aliasPtr->comment = Tcl_NewObj();
+		aliasPtr->flags = 0;
+		break;
+	    case 6:
+		aliasPtr->book = Tcl_NewStringObj("Personal", -1);
+		key = Tcl_GetString(objv[2]);
+		aliasPtr->fullname = objv[3];
+		aliasPtr->content = objv[4];
+		aliasPtr->comment = Tcl_NewObj();
+		aliasPtr->flags = 0;
+		break;
+	    case 7:
+		aliasPtr->book = objv[2];
+		key = Tcl_GetString(objv[3]);
+		aliasPtr->fullname = objv[4];
+		aliasPtr->content = objv[5];
+		aliasPtr->comment = objv[6];
+		aliasPtr->flags = 0;
+		break;
+	    case 8:
+		aliasPtr->book = objv[2];
+		key = Tcl_GetString(objv[3]);
+		aliasPtr->fullname = objv[4];
+		aliasPtr->content = objv[5];
+		aliasPtr->comment = objv[6];
+		if (!strcmp(Tcl_GetString(objv[7]), "nofullname")) {
+		    aliasPtr->flags = ALIAS_FLAG_NOFULLNAME;
+		} else {
+		    aliasPtr->flags = 0;
+		}
+		break;
+	    default:
+		Tcl_AppendResult(interp, "wrong # args: should be \"",
+			Tcl_GetString(objv[0]),
+			" add book name fullname content comment options\"",
+			(char *) NULL);
+		return TCL_ERROR;
 	}
-	if (5 == argc) {
-	    s = 2;
-	    book = "Personal";
-	    comment = "";
-	} else {
-	    s = 3;
-	    book = argv[2];
-	    comment = argv[6];
-	}
-	if (!strlen(argv[s])) {
+	if (!key || !*key) {
+	    ckfree(aliasPtr);
 	    Tcl_SetResult(interp, "The name can not be an empty string",
 		    TCL_STATIC);
 	    return TCL_OK;
 	}
-	key = argv[s];
-	lcase(key);
-	aliasPtr = (AliasInfo*)ckalloc(sizeof(AliasInfo)+16+strlen(book)+
-		strlen(argv[s+1])+strlen(argv[s+2])+strlen(comment));
-	cPtr = (char*)PADDED((char*)aliasPtr+sizeof(AliasInfo));
-	aliasPtr->book = cPtr;
-	cPtr = PADDED(cPtr + strlen(book) + 1);
-	aliasPtr->fullname = cPtr;
-	cPtr = PADDED(cPtr + strlen(argv[s+1]) + 1);
-	aliasPtr->content = cPtr;
-	cPtr = PADDED(cPtr + strlen(argv[s+2]) + 1);
-	aliasPtr->comment = cPtr;
 	aliasPtr->mark = 0;
-	strcpy(aliasPtr->book, book);
-	strcpy(aliasPtr->fullname, argv[s+1]);
-	AddressStrcpy(aliasPtr->content, argv[s+2]);
-	strcpy(aliasPtr->comment, comment);
+	if (Tcl_IsShared(aliasPtr->content)) {
+	    aliasPtr->content = Tcl_DuplicateObj(aliasPtr->content);
+	}
+	if (1 < AddressClean(aliasPtr->content)) {
+	    aliasPtr->flags |= ALIAS_FLAG_ISLIST;
+	}
+	Tcl_IncrRefCount(aliasPtr->book);
+	Tcl_IncrRefCount(aliasPtr->fullname);
+	Tcl_IncrRefCount(aliasPtr->content);
+	Tcl_IncrRefCount(aliasPtr->comment);
 	entryPtr = Tcl_CreateHashEntry(&aliasTable, key, &new);
 	Tcl_SetHashValue(entryPtr, (ClientData)aliasPtr);
 	return TCL_OK;
 	
-    } else if (!strcmp(argv[1], "delete")) {
+    } else if (!strcmp(Tcl_GetString(objv[1]), "delete")) {
 	Tcl_HashEntry *entryPtr;
-	char key[1024];
+	AliasInfo *aliasPtr;
 	int i;
 
-	for (i=2; i<argc; i++) {
-	    strncpy(key, argv[i], sizeof(key));
-	    lcase(key);
-	    if ((entryPtr = Tcl_FindHashEntry(&aliasTable, key))) {
-		ckfree((AliasInfo*)Tcl_GetHashValue(entryPtr));
+	for (i=2; i<objc; i++) {
+	    if ((entryPtr = Tcl_FindHashEntry(&aliasTable,
+		    Tcl_GetString(objv[i])))) {
+		aliasPtr = (AliasInfo*)Tcl_GetHashValue(entryPtr);
+		Tcl_DecrRefCount(aliasPtr->book);
+		Tcl_DecrRefCount(aliasPtr->fullname);
+		Tcl_DecrRefCount(aliasPtr->content);
+		Tcl_DecrRefCount(aliasPtr->comment);
+		ckfree(aliasPtr);
 		Tcl_DeleteHashEntry(entryPtr);
 	    }
 	}
 	return TCL_OK;
 
-    } else if (!strcmp(argv[1], "get")) {
+    } else if (!strcmp(Tcl_GetString(objv[1]), "get")) {
 	Tcl_HashEntry *entryPtr;
 	AliasInfo *aliasPtr;
+	Tcl_Obj *oPtr;
 
-	if (argc != 3) {
-	    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-		    " get alias\"", (char *) NULL);
+	if (objc != 3) {
+	    Tcl_AppendResult(interp, "wrong # args: should be \"",
+		    Tcl_GetString(objv[0]), " get alias\"", (char *) NULL);
 	    return TCL_ERROR;
 	}
-	if (!(entryPtr = Tcl_FindHashEntry(&aliasTable, argv[2]))) {
+	if (!(entryPtr=Tcl_FindHashEntry(&aliasTable, Tcl_GetString(objv[2])))){
 	    Tcl_SetResult(interp, "Illegal alias", TCL_STATIC);
 	    return TCL_ERROR;
 	}
-	aliasPtr = (AliasInfo*) Tcl_GetHashValue(entryPtr);
-	Tcl_AppendElement(interp, aliasPtr->book);
-	Tcl_AppendElement(interp, aliasPtr->fullname);
-	Tcl_AppendElement(interp, aliasPtr->content);
-	Tcl_AppendElement(interp, aliasPtr->comment);
+	aliasPtr = (AliasInfo*)Tcl_GetHashValue(entryPtr);
+	oPtr = Tcl_NewObj();
+	Tcl_ListObjAppendElement(interp, oPtr, aliasPtr->book);
+	Tcl_ListObjAppendElement(interp, oPtr, aliasPtr->fullname);
+	Tcl_ListObjAppendElement(interp, oPtr, aliasPtr->content);
+	Tcl_ListObjAppendElement(interp, oPtr, aliasPtr->comment);
+	if (aliasPtr->flags & ALIAS_FLAG_NOFULLNAME) {
+	    Tcl_ListObjAppendElement(interp, oPtr,
+		    Tcl_NewStringObj("nofullname", -1));
+	} else {
+	    Tcl_ListObjAppendElement(interp, oPtr, Tcl_NewObj());
+	}
+	Tcl_SetObjResult(interp, oPtr);
 	return TCL_OK;
 
-    } else if (!strcmp(argv[1], "list")) {
+    } else if (!strcmp(Tcl_GetString(objv[1]), "list")) {
 	Tcl_HashEntry *entryPtr;
 	Tcl_HashSearch search;
 	AliasInfo *aliasPtr;
-	Tcl_DString list;
+	Tcl_Obj *oPtr;
 
-	if (argc != 3) {
-	    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-		    " list var\"", (char *) NULL);
+	if (objc != 3) {
+	    Tcl_AppendResult(interp, "wrong # args: should be \"",
+		    Tcl_GetString(objv[0]), " list var\"", (char *) NULL);
 	    return TCL_ERROR;
 	}
 
-	Tcl_DStringInit(&list);
 	for (entryPtr = Tcl_FirstHashEntry(&aliasTable, &search);
 		entryPtr; entryPtr = Tcl_NextHashEntry(&search)) {
 	    aliasPtr = (AliasInfo*) Tcl_GetHashValue(entryPtr);
-	    Tcl_DStringAppendElement(&list, aliasPtr->book);
-	    Tcl_DStringAppendElement(&list, aliasPtr->fullname);
-	    Tcl_DStringAppendElement(&list, aliasPtr->content);
-	    Tcl_DStringAppendElement(&list, aliasPtr->comment);
-	    Tcl_SetVar2(interp, argv[2], Tcl_GetHashKey(&aliasTable, entryPtr),
-		    Tcl_DStringValue(&list), 0);
-	    Tcl_DStringFree(&list);
+	    oPtr = Tcl_NewObj();
+	    Tcl_ListObjAppendElement(interp, oPtr, aliasPtr->book);
+	    Tcl_ListObjAppendElement(interp, oPtr, aliasPtr->fullname);
+	    Tcl_ListObjAppendElement(interp, oPtr, aliasPtr->content);
+	    Tcl_ListObjAppendElement(interp, oPtr, aliasPtr->comment);
+	    if (aliasPtr->flags & ALIAS_FLAG_NOFULLNAME) {
+		Tcl_ListObjAppendElement(interp, oPtr,
+			Tcl_NewStringObj("nofullname", -1));
+	    } else {
+		Tcl_ListObjAppendElement(interp, oPtr, Tcl_NewObj());
+	    }
+	    Tcl_SetVar2Ex(interp, Tcl_GetString(objv[2]),
+		    Tcl_GetHashKey(&aliasTable, entryPtr), oPtr, 0);
 	}
 	return TCL_OK;
 	
-    } else if (!strcmp(argv[1], "read")) {
-	if (argc != 3) {
-	    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-		    " read filename\"", (char *) NULL);
+    } else if (!strcmp(Tcl_GetString(objv[1]), "read")) {
+	Tcl_Channel channel;
+	Tcl_Obj *oPtr = Tcl_NewObj();
+
+	if (objc != 3) {
+	    Tcl_AppendResult(interp, "wrong # args: should be \"",
+		    Tcl_GetString(objv[0]), " read filename\"", (char *) NULL);
 	    return TCL_ERROR;
 	}
-	return Tcl_EvalFile(interp, argv[2]);
+	if (NULL == (channel = Tcl_OpenFileChannel(interp,
+		Tcl_GetString(objv[2]), "r", 0))) {
+	    return TCL_ERROR;
+	}
+	Tcl_SetChannelOption(interp, channel, "-encoding", "utf-8");
+	while (0 <= Tcl_GetsObj(channel, oPtr) && !Tcl_Eof(channel)) {
+	    Tcl_AppendToObj(oPtr, ";", 1);
+	}
+	return Tcl_EvalObjEx(interp, oPtr,  TCL_EVAL_DIRECT);
 	
-    } else if (!strcmp(argv[1], "save")) {
+    } else if (!strcmp(Tcl_GetString(objv[1]), "save")) {
 	Tcl_HashEntry *entryPtr;
 	Tcl_HashSearch search;
 	AliasInfo *aliasPtr;
-	Tcl_DString list, buffer;
-	char buf[1024];
-	FILE *fp;
+	Tcl_Channel channel;
+	Tcl_Obj *lPtr;
+	int perm;
 
-	if (argc != 4) {
-	    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-		    " save book filename\"", (char *) NULL);
+	if (objc != 4) {
+	    Tcl_AppendResult(interp, "wrong # args: should be \"",
+		    Tcl_GetString(objv[0])," save book filename\"",(char*)NULL);
 	    return TCL_ERROR;
 	}
 
-	if (NULL == (fp = fopen(Tcl_TildeSubst(interp,argv[3],&buffer), "w"))) {
-	    Tcl_DStringFree(&buffer);
-	    sprintf(buf, "Failed to open file \"%s\": %s", argv[2],
-		    Tcl_PosixError(interp));
-	    RatLog(interp, RAT_ERROR, buf, 0);
+	Tcl_GetInt(interp, Tcl_GetVar2(interp, "option", "permissions",
+		TCL_GLOBAL_ONLY), &perm);
+	if (NULL == (channel = Tcl_OpenFileChannel(interp,
+		Tcl_GetString(objv[3]), "w", perm))) {
 	    return TCL_ERROR;
 	}
-	Tcl_DStringFree(&buffer);
 
-	Tcl_DStringInit(&list);
 	for (entryPtr = Tcl_FirstHashEntry(&aliasTable, &search);
 		entryPtr; entryPtr = Tcl_NextHashEntry(&search)) {
 	    aliasPtr = (AliasInfo*) Tcl_GetHashValue(entryPtr);
-	    if (strcmp(argv[2], aliasPtr->book)) {
+	    if (strcmp(Tcl_GetString(objv[2]), Tcl_GetString(aliasPtr->book))) {
 		continue;
 	    }
-	    Tcl_DStringAppendElement(&list, aliasPtr->book);
-	    Tcl_DStringAppendElement(&list,
-		    Tcl_GetHashKey(&aliasTable, entryPtr));
-	    Tcl_DStringAppendElement(&list, aliasPtr->fullname);
-	    Tcl_DStringAppendElement(&list, aliasPtr->content);
-	    Tcl_DStringAppendElement(&list, aliasPtr->comment);
-	    fputs("RatAlias add ", fp);
-	    fputs(Tcl_DStringValue(&list), fp);
-	    fputs("\n", fp);
-	    Tcl_DStringSetLength(&list, 0);
+	    lPtr = Tcl_NewObj();
+	    Tcl_ListObjAppendElement(interp, lPtr, aliasPtr->book);
+	    Tcl_ListObjAppendElement(interp, lPtr,
+		    Tcl_NewStringObj(Tcl_GetHashKey(&aliasTable, entryPtr),-1));
+	    Tcl_ListObjAppendElement(interp, lPtr, aliasPtr->fullname);
+	    Tcl_ListObjAppendElement(interp, lPtr, aliasPtr->content);
+	    Tcl_ListObjAppendElement(interp, lPtr, aliasPtr->comment);
+	    if (aliasPtr->flags & ALIAS_FLAG_NOFULLNAME) {
+		Tcl_ListObjAppendElement(interp, lPtr,
+			Tcl_NewStringObj("nofullname", -1));
+	    } else {
+		Tcl_ListObjAppendElement(interp, lPtr, Tcl_NewObj());
+	    }
+	    Tcl_WriteChars(channel, "RatAlias add ", -1);
+	    Tcl_WriteObj(channel, lPtr);
+	    Tcl_DecrRefCount(lPtr);
+	    Tcl_WriteChars(channel, "\n", 1);
 	}
-	Tcl_DStringFree(&list);
-
-	if ( 0 != fclose(fp)) {
-	    sprintf(buf, "Failed to close file \"%s\": %s", argv[2],
-		    Tcl_PosixError(interp));
-	    RatLog(interp, RAT_ERROR, buf, 0);
-	    return TCL_ERROR;
-	}
-	return TCL_OK;
+	return Tcl_Close(interp, channel);
 	
-    } else if (!strcmp(argv[1], "expand1") || !strcmp(argv[1], "expand2")) {
+    } else if (!strcmp(Tcl_GetString(objv[1]), "expand1")
+	    || !strcmp(Tcl_GetString(objv[1]), "expand2")) {
 	ADDRESS *adrPtr, *baseAdrPtr = NULL;
 	struct passwd *pwPtr;
-	Tcl_HashEntry *entryPtr, *userPtr = NULL;
+	Tcl_HashEntry *entryPtr;
 	Tcl_DString newList;
 	AliasInfo *aliasPtr;
-	char buf[1024], *sPtr, *cPtr;
-	int level, rescan, useTable, new, inGroup;
+	char buf[1024], *sPtr, *cPtr, *userPtr;
+	int level, rescan, useTable, inGroup, length;
 
-	if (argc != 3) {
-	    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-		    " expand[12] adrlist\"", (char *) NULL);
+	if (objc != 3) {
+	    Tcl_AppendResult(interp, "wrong # args: should be \"",
+		    Tcl_GetString(objv[0]), " expand[12] adrlist\"",
+		    (char *) NULL);
 	    return TCL_ERROR;
 	}
 
 	/*
 	 * Ignore empty addresses
 	 */
-	for (sPtr = argv[2]; *sPtr && isspace(*sPtr); sPtr++);
+	for (sPtr = Tcl_GetString(objv[2]);
+		    *sPtr && isspace((unsigned char)*sPtr); sPtr++);
 	if (!*sPtr) {
 	    return TCL_OK;
 	}
@@ -712,15 +810,16 @@ RatAlias(ClientData dummy, Tcl_Interp *interp, int argc, char *argv[])
 	Tcl_GetBoolean(interp, Tcl_GetVar2(interp, "option", "lookup_name",
 			               TCL_GLOBAL_ONLY), &useTable);
 
-	if (!strcmp(argv[1], "expand1")) {
-	    level = 0;
+	if (!strcmp(Tcl_GetString(objv[1]), "expand2")) {
+	    level = 2;
 	} else {
-	    level = 1;
+	    Tcl_GetIntFromObj(interp, Tcl_GetVar2Ex(interp, "option",
+		    "alias_expand", TCL_GLOBAL_ONLY), &level);
 	}
 	Tcl_DStringInit(&newList);
 	Tcl_DStringAppend(&newList, sPtr, -1);
 	for (cPtr = Tcl_DStringValue(&newList); *cPtr; cPtr++) {
-	    if (isspace(*cPtr)) {
+	    if (isspace((unsigned char)*cPtr)) {
 		*cPtr = ' ';
 	    }
 	}
@@ -751,64 +850,76 @@ RatAlias(ClientData dummy, Tcl_Interp *interp, int argc, char *argv[])
 		 * try mailbox@host.
 		 */
 		if (adrPtr->error || (adrPtr->host && adrPtr->host[0] == '.')){
+		    Tcl_DStringFree(&newList);
 		    if (adrPtr->error) {
 			Tcl_SetResult(interp, adrPtr->error, TCL_VOLATILE);
 		    } else {
 			Tcl_SetResult(interp, adrPtr->host, TCL_VOLATILE);
 		    }
-		    Tcl_DStringFree(&newList);
 		    mail_free_address(&baseAdrPtr);
 		    return TCL_ERROR;
 		}
 		entryPtr = NULL;
+		userPtr = NULL;
 		if (adrPtr->host && *adrPtr->host) {
-		    sprintf(buf, "%s@%s", adrPtr->mailbox, adrPtr->host);
+		    snprintf(buf, sizeof(buf), "%s@%s",
+			    adrPtr->mailbox, adrPtr->host);
 		} else {
-		    strcpy(buf, adrPtr->mailbox);
+		    RatStrNCpy(buf, adrPtr->mailbox, sizeof(buf));
 		}
-		lcase(buf);
-		if (useTable && 
-			!(entryPtr = Tcl_FindHashEntry(&aliasTable, buf))) {
+		if (!(entryPtr = Tcl_FindHashEntry(&aliasTable, buf))
+			&& useTable) {
 		    if ((pwPtr = getpwnam(buf))) {
-			userPtr = Tcl_CreateHashEntry(&userTable,
-				pwPtr->pw_name, &new);
-			strcpy(buf, pwPtr->pw_gecos);
+			RatStrNCpy(buf, pwPtr->pw_gecos, sizeof(buf));
 			if ((cPtr = strchr(buf, ','))) {
 			    *cPtr = '\0';
 			}
-			Tcl_SetHashValue(userPtr, (ClientData)cpystr(buf));
+			userPtr = buf;
 		    }
 		}
 		if (entryPtr) {
 		    aliasPtr = (AliasInfo*) Tcl_GetHashValue(entryPtr);
 		    if (aliasPtr->mark == aliasMark) {
-			Tcl_SetResult(interp, "Alias loop detected",TCL_STATIC);
-			return TCL_ERROR;
+			continue;
 		    }
-		    if (level) {
-			rescan = 1;
-			Tcl_DStringAppend(&newList, aliasPtr->content, -1);
-		    } else {
-			sPtr = RatAddressMail(adrPtr);
-			Tcl_DStringAppend(&newList, sPtr, -1);
-			if (aliasPtr->fullname) {
-			    sprintf(buf, " (%s)", aliasPtr->fullname);
+		    switch (level) {
+		    case 0:
+			Tcl_DStringAppend(&newList, buf, -1);
+			break;
+		    case 1:
+			Tcl_DStringAppend(&newList, RatAddressMail(adrPtr), -1);
+			cPtr = Tcl_GetString(aliasPtr->fullname);
+			if (cPtr) {
+			    sprintf(buf, " (%.1020s)", cPtr);
 			    Tcl_DStringAppend(&newList, buf, strlen(buf));
 			}
+			break;
+		    case 2:
+			rescan = 1;
+			cPtr = Tcl_GetStringFromObj(aliasPtr->content, &length);
+			Tcl_DStringAppend(&newList, cPtr, length);
+			if (!(aliasPtr->flags & ALIAS_FLAG_ISLIST)
+				&& !(aliasPtr->flags & ALIAS_FLAG_NOFULLNAME)) {
+			    Tcl_DStringAppend(&newList, " (", 2);
+			    cPtr = Tcl_GetStringFromObj(aliasPtr->fullname,
+				    &length);
+			    Tcl_DStringAppend(&newList, cPtr, length);
+			    Tcl_DStringAppend(&newList, ")", 1);
+			}
+			break;
 		    }
 		    aliasPtr->mark = aliasMark;
 		} else if ((!adrPtr->host || !*adrPtr->host) &&
-			   !adrPtr->personal && !level && userPtr) {
-		    sPtr = RatAddressMail(adrPtr);
-		    Tcl_DStringAppend(&newList, sPtr, -1);
-		    sprintf(buf, " (%s)", (char*)Tcl_GetHashValue(userPtr));
-		    Tcl_DStringAppend(&newList, buf, -1);
+			   !adrPtr->personal && level > 0 && userPtr) {
+		    Tcl_DStringAppend(&newList, RatAddressMail(adrPtr), -1);
+		    Tcl_DStringAppend(&newList, " (", 2);
+		    Tcl_DStringAppend(&newList, userPtr, -1);
+		    Tcl_DStringAppend(&newList, ")", 1);
 
 		} else {
-		    sPtr = RatAddressMail(adrPtr);
-		    Tcl_DStringAppend(&newList, sPtr, -1);
-		    if (adrPtr->personal && strlen(adrPtr->personal)&& !level) {
-			sprintf(buf, " (%s)", adrPtr->personal);
+		    Tcl_DStringAppend(&newList, RatAddressMail(adrPtr), -1);
+		    if (adrPtr->personal && strlen(adrPtr->personal)) {
+			sprintf(buf, " (%.1020s)", adrPtr->personal);
 			Tcl_DStringAppend(&newList, buf, -1);
 		    }
 		}
@@ -822,7 +933,7 @@ RatAlias(ClientData dummy, Tcl_Interp *interp, int argc, char *argv[])
 	return TCL_OK;
 	
     } else {
-	Tcl_AppendResult(interp, "bad option \"", argv[1],
+	Tcl_AppendResult(interp, "bad option \"", Tcl_GetString(objv[1]),
 		"\": must be one of add, delete, get, list, read, save,",
 		" expand1 or expand2",
 		(char *) NULL);
@@ -878,7 +989,7 @@ RatAddressMail(ADDRESS *adrPtr)
 		+(adrPtr->host?strlen(adrPtr->host)+1:0)+1;
 	if (l > length) {
 	    length = l;
-	    store = REALLOC(store, length);
+	    store = ckrealloc(store, length);
 	}
 	i = 0;
 	store[i++] = '"';
@@ -893,17 +1004,20 @@ RatAddressMail(ADDRESS *adrPtr)
 	l = strlen(adrPtr->mailbox)+(adrPtr->host?strlen(adrPtr->host)+1:0)+1;
 	if (l > length) {
 	    length = l;
-	    store = REALLOC(store, length);
+	    store = ckrealloc(store, length);
 	}
 	strcpy(store, adrPtr->mailbox);
 	i = strlen(store);
     }
     if (adrPtr->host && *adrPtr->host) {
 	store[i++] = '@';
-	strcpy(&store[i], adrPtr->host);
+	RatStrNCpy(&store[i], adrPtr->host, length-i);
     } else {
 	store[i++] = '\0';
     }
+#ifdef MEM_DEBUG
+    mem_store = store;
+#endif /* MEM_DEBUG */
     return store;
 }
 
@@ -938,7 +1052,7 @@ RatSplitAddresses(ClientData clientData, Tcl_Interp *interp, int argc,
         return TCL_ERROR;
     }
  
-    adr = (char*)malloc(strlen(argv[1])+1);
+    adr = (char*)ckalloc(strlen(argv[1])+1);
     Tcl_DStringInit(&result);
  
     for (srcPtr = argv[1], dstPtr = adr; *srcPtr; srcPtr++) {
@@ -967,10 +1081,13 @@ RatSplitAddresses(ClientData clientData, Tcl_Interp *interp, int argc,
 	    break;
 	case ',':
 	    if (!inq && !inc) {
-		for (dstPtr--; dstPtr >= adr && isspace(*dstPtr); dstPtr--);
+		for (dstPtr--;
+			dstPtr >= adr && isspace((unsigned char)*dstPtr);
+			dstPtr--);
 		if (++dstPtr != adr) {
 		    *dstPtr = '\0';
-		    for (cPtr = adr; *cPtr && isspace(*cPtr); cPtr++);
+		    for (cPtr = adr;
+			    *cPtr && isspace((unsigned char)*cPtr); cPtr++);
 		    if (cPtr) {
 			Tcl_DStringAppendElement(&result, cPtr);
 		    }
@@ -990,7 +1107,66 @@ RatSplitAddresses(ClientData clientData, Tcl_Interp *interp, int argc,
     }
  
     Tcl_DStringResult(interp, &result);
-    free(adr);
+    ckfree(adr);
     return TCL_OK;
 }
 
+#ifdef MEM_DEBUG
+void ratAddressCleanup()
+{
+    Tcl_HashEntry *e;
+    Tcl_HashSearch s;
+
+    for (e = Tcl_FirstHashEntry(&aliasTable, &s); e; e = Tcl_NextHashEntry(&s)){
+	ckfree(Tcl_GetHashValue(e));
+    }
+    Tcl_DeleteHashTable(&aliasTable);
+
+    if (mem_store) {
+	ckfree(mem_store);
+    }
+}
+#endif /* MEM_DEBUG */
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * RatAddressSize --
+ *
+ *	Calculate the maximum size of and address list (or single address)
+ *
+ * Results:
+ *	The maximum length of the address
+ *
+ * Side effects:
+ *	None.
+ *
+ *
+ *----------------------------------------------------------------------
+ */
+
+size_t
+RatAddressSize(ADDRESS *adrPtr, int all)
+{
+    ADDRESS *a,tadr;
+    char tmp[MAILTMPLEN];
+    size_t len, t;
+
+    tadr.next = NULL;
+    for (len = 0, a = adrPtr; a; a = a->next) {
+        t = (tadr.mailbox = a->mailbox) ? 2*strlen (a->mailbox) : 3;
+        if ((tadr.personal = a->personal)) t += 3 + 2*strlen (a->personal);
+        if ((tadr.adl = a->adl)) t += 1 + 2*strlen (a->adl);
+        if ((tadr.host = a->host)) t += 1 + 2*strlen (a->host);
+        if (t < MAILTMPLEN) {     /* ignore ridiculous addresses */
+	    tmp[0] = '\0';
+	    rfc822_write_address (tmp,&tadr); 
+	    t = strlen(tmp);
+	}
+	len += t+2;
+	if (!all) break;
+    }
+    return len;
+}
+
