@@ -3,8 +3,8 @@
  *
  *	This file handles the delivery status notifications.
  *
- * TkRat software and its included text is Copyright 1996,1997,1998
- * by Martin Forssn
+ * TkRat software and its included text is Copyright 1996-2000 by
+ * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
  * COPYRIGHT, included with this distribution.
@@ -38,8 +38,7 @@ static int RatDSNGet(ClientData clientData, Tcl_Interp *interp,
 	int argc, char **argv);
 static int RatDSNExpire(Tcl_Interp *interp, char *line);
 static char *RatParseDSNLine(char *buf, char *name, char *value, int *length);
-static RatDeliveryStatus *RatParseDS(Tcl_Interp *interp, char *body,
-	int length);
+static RatDeliveryStatus *RatParseDS(Tcl_Interp *interp, Tcl_Obj *body);
 static void RatFreeDeliveryStatus(RatDeliveryStatus *statusPtr);
 
 
@@ -99,7 +98,7 @@ RatDSNStartMessage(Tcl_Interp *interp, char *id, char *subject)
     seconds = time(NULL);
     sprintf(buf, "%d", (int)seconds);
     Tcl_DStringAppendElement(dsPtr, buf);
-    cPtr = header = RatDecodeHeader(interp, subject);
+    cPtr = header = RatDecodeHeader(interp, subject, 0);
     while ((cPtr = strchr(cPtr, '\n'))) {
 	*cPtr++ = ' ';
     }
@@ -199,7 +198,7 @@ RatDSNFinish(Tcl_Interp *interp, DSNhandle handle)
     Tcl_Write(channel, "\n", 1);
     Tcl_Close(interp, channel);
     Tcl_DStringFree(dsPtr);
-    free(dsPtr);
+    ckfree(dsPtr);
 }
 
 /*
@@ -244,7 +243,7 @@ RatDSNHandle (Tcl_Interp *interp, char *msg)
     if (TCL_OK != Tcl_Eval(interp, buf)) {
 	return 0;
     }
-    statusPtr = RatParseDS(interp, interp->result, strlen(interp->result));
+    statusPtr = RatParseDS(interp, Tcl_GetObjResult(interp));
     if (!statusPtr->envid) {
 	RatFreeDeliveryStatus(statusPtr);
 	return 0;
@@ -252,7 +251,6 @@ RatDSNHandle (Tcl_Interp *interp, char *msg)
 
     Tcl_DStringInit(&ds);
     Tcl_DStringInit(&line);
-    Tcl_DStringInit(&dirDS);
     if (NULL == (channel = OpenIndex(interp, "r"))) {
 	RatFreeDeliveryStatus(statusPtr);
 	return 0;
@@ -267,18 +265,18 @@ RatDSNHandle (Tcl_Interp *interp, char *msg)
 	     * This DSN has expired so we should remove all associated files
 	     */
 	    file = RatLindex(interp, Tcl_DStringValue(&line), 0);
-	    sprintf(buf, "%s/%s", dir, file);
+	    snprintf(buf, sizeof(buf), "%s/%s", dir, file);
 	    (void)unlink(buf);
 	    Tcl_SplitList(interp, RatLindex(interp, Tcl_DStringValue(&line), 3),
 		    &argc, &argv);
 	    for (i=0; i < argc; i++) {
 		file = RatLindex(interp, argv[i], 2);
 		if (strlen(file)) {
-		    sprintf(buf, "%s/%s", dir, file);
+		    snprintf(buf, sizeof(buf), "%s/%s", dir, file);
 		    (void)unlink(buf);
 		}
 	    }
-	    free(argv);
+	    ckfree(argv);
 	    changed++;
 	    continue;
 	}
@@ -314,13 +312,13 @@ RatDSNHandle (Tcl_Interp *interp, char *msg)
 		    rfound = 1;
 		    oldId = RatLindex(interp, rargv[i], 2);
 		    RatGenId(NULL, interp, 0, NULL);
-		    strcpy(id, interp->result);
+		    RatStrNCpy(id, Tcl_GetStringResult(interp), sizeof(id));
 		    if (strlen(oldId)) {
-			sprintf(buf, "%s/%s", dir, oldId);
+			snprintf(buf, sizeof(buf), "%s/%s", dir, oldId);
 			(void)unlink(buf);
 		    }
 
-		    sprintf(buf, "%s/%s", dir, id);
+		    snprintf(buf, sizeof(buf), "%s/%s", dir, id);
 		    if (!msgFile) {
 			Tcl_DString msgDS;
 			Tcl_Channel msgCh;
@@ -328,8 +326,9 @@ RatDSNHandle (Tcl_Interp *interp, char *msg)
 			msgFile = cpystr(buf);
 			Tcl_DStringInit(&msgDS);
 			Tcl_GetCommandInfo(interp, msg, &cmdInfo);
-			RatMessageGet(interp,(MessageInfo*)cmdInfo.clientData,
-				&msgDS, NULL, NULL);
+			RatMessageGet(interp,
+				      (MessageInfo*)cmdInfo.objClientData,
+				      &msgDS, NULL, 0, NULL, 0);
 			msgCh = Tcl_OpenFileChannel(interp, msgFile, "w", perm);
 			Tcl_Write(msgCh, Tcl_DStringValue(&msgDS),
 				Tcl_DStringLength(&msgDS));
@@ -343,8 +342,7 @@ RatDSNHandle (Tcl_Interp *interp, char *msg)
 		    Tcl_DStringAppendElement(&ds,statusPtr->recipientPtrPtr[j]);
 		    Tcl_DStringAppendElement(&ds, id);
 		    Tcl_DStringEndSublist(&ds);
-		    Tcl_VarEval(interp, "RatDSNRecieve {",
-			    RatDecodeHeaderFull(interp, argv[2]), "} {",
+		    Tcl_VarEval(interp, "RatDSNRecieve {", argv[2], "} {",
 			    statusPtr->actionPtrPtr[j], "} {",
 			    statusPtr->recipientPtrPtr[j], "} {", id, "}",NULL);
 		}
@@ -372,6 +370,7 @@ RatDSNHandle (Tcl_Interp *interp, char *msg)
 	ckfree(argv);
 	Tcl_Close(interp, channel);
     }
+    Tcl_DStringFree(&dirDS);
     Tcl_DStringFree(&ds);
     Tcl_DStringFree(&line);
     Tcl_SetHashValue(entryPtr, match);
@@ -389,7 +388,7 @@ RatDSNHandle (Tcl_Interp *interp, char *msg)
  *      Extract the DSN data from a dsn body part
  *
  * Results:
- *	A standard tcl result and the requested data in interp->result.
+ *	A standard tcl result and the requested data in the result area
  *
  * Side effects:
  *	None.
@@ -399,9 +398,9 @@ RatDSNHandle (Tcl_Interp *interp, char *msg)
  */
 
 int
-RatDSNExtract (Tcl_Interp *interp, char *data, int length)
+RatDSNExtract (Tcl_Interp *interp, Tcl_Obj *body)
 {
-    RatDeliveryStatus *sPtr = RatParseDS(interp, data, length);
+    RatDeliveryStatus *sPtr = RatParseDS(interp, body);
     Tcl_DString ds;
     int i;
 
@@ -426,7 +425,7 @@ RatDSNExtract (Tcl_Interp *interp, char *data, int length)
  *
  * Results:
  *	A Tcl channel handle. If an error occurs NULL is returned
- *	and an error message is left in interp->result.
+ *	and an error message is left in the result area.
  *
  * Side effects:
  *	None.
@@ -441,6 +440,7 @@ OpenIndex(Tcl_Interp *interp, char *mode)
     Tcl_DString ds;
     char *dir, buf[1024];
     struct stat sbuf;
+    Tcl_Channel channel;
     int perm;
 
     Tcl_GetInt(interp, Tcl_GetVar2(interp, "option", "permissions",
@@ -457,10 +457,14 @@ OpenIndex(Tcl_Interp *interp, char *mode)
 	Tcl_AppendResult(interp, "This is no directory \"", dir, "\"", NULL);
 	return NULL;
     }
-    sprintf(buf, "%s/index", dir);
+    snprintf(buf, sizeof(buf), "%s/index", dir);
     Tcl_DStringFree(&ds);
 
-    return Tcl_OpenFileChannel(interp, buf, mode, perm);
+    if (NULL == (channel = Tcl_OpenFileChannel(interp, buf, mode, perm))) {
+	return NULL;
+    }
+    Tcl_SetChannelOption(interp, channel, "-encoding", "utf-8");
+    return channel;
 }
 
 /*
@@ -552,9 +556,9 @@ RatDSNGet (ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
     Tcl_DStringInit(&ds);
     dir = Tcl_TranslateFileName(interp, Tcl_GetVar2(interp, "option",
 	    "dsn_directory", TCL_GLOBAL_ONLY), &ds);
-    sprintf(buf, "%s/%s", dir, argv[2]);
+    snprintf(buf, sizeof(buf), "%s/%s", dir, argv[2]);
     if (NULL == (channel = Tcl_OpenFileChannel(interp, buf, "r", 0))) {
-       return TCL_ERROR;
+        return TCL_ERROR;
     }
     Tcl_DStringFree(&ds);
     len = Tcl_Seek(channel, 0, SEEK_END);
@@ -567,13 +571,13 @@ RatDSNGet (ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
     ckfree(data);
 
     if (!strcmp(argv[1], "msg")) {
-	Tcl_SetResult(interp, msg, TCL_DYNAMIC);
+	Tcl_SetResult(interp, msg, TCL_VOLATILE);
     } else {
-	sprintf(buf, "[lindex [[%s body] children] 1] data 0", msg);
+	snprintf(buf, sizeof(buf),"[lindex [[%s body] children] 1] data 0",msg);
 	if (TCL_OK != Tcl_Eval(interp, buf)) {
 	    return TCL_ERROR;
 	}
-	statusPtr = RatParseDS(interp, interp->result, strlen(interp->result));
+	statusPtr = RatParseDS(interp, Tcl_GetObjResult(interp));
 	Tcl_DStringResult(interp, &statusPtr->msgFields);
 	for (i=0; i < statusPtr->numRecipients; i++) {
 	    if (!strcmp(statusPtr->recipientPtrPtr[i], argv[3])) {
@@ -705,14 +709,15 @@ RatParseDSNLine(char *buf, char *name, char *value, int *length)
  */
 
 static RatDeliveryStatus*
-RatParseDS(Tcl_Interp *interp, char *body, int length)
+RatParseDS(Tcl_Interp *interp, Tcl_Obj *bPtr)
 {
     RatDeliveryStatus *sPtr = (RatDeliveryStatus*)ckalloc(sizeof(*sPtr));
-    char *name, *value, *cPtr;
-    int allocated, i;
+    char *name, *value, *cPtr, *body;
+    int allocated, i, length;
 
-    name = (char*)ckalloc(strlen(body)+1);
-    value = (char*)ckalloc(strlen(body)+1);
+    body = Tcl_GetStringFromObj(bPtr, &length);
+    name = (char*)ckalloc(length+1);
+    value = (char*)ckalloc(length+1);
 
     /*
      * Parse the per message fields.
@@ -735,6 +740,10 @@ RatParseDS(Tcl_Interp *interp, char *body, int length)
 	if (!strcasecmp("original-envelope-id", name)) {
 	    sPtr->envid = cpystr(value);
 	}
+	/* Added by Lou Ruppert to account for odd DSN implementations */
+	if (!strcasecmp("arrival-date",name) && '\n'!=*body && '\r'!=*body) {
+	    break;
+	}
     }
 
     /*
@@ -749,18 +758,19 @@ RatParseDS(Tcl_Interp *interp, char *body, int length)
     while (length > 0) {
 	if (allocated <= sPtr->numRecipients) {
 	    allocated += 32;
-	    sPtr->actionPtrPtr = (char**)REALLOC(sPtr->actionPtrPtr, 
+	    sPtr->actionPtrPtr = (char**)ckrealloc(sPtr->actionPtrPtr, 
 		    allocated*sizeof(char*));
-	    sPtr->recTypePtrPtr = (char**)REALLOC(sPtr->recTypePtrPtr, 
+	    sPtr->recTypePtrPtr = (char**)ckrealloc(sPtr->recTypePtrPtr, 
 		    allocated*sizeof(char*));
-	    sPtr->recipientPtrPtr = (char**)REALLOC(sPtr->recipientPtrPtr, 
+	    sPtr->recipientPtrPtr = (char**)ckrealloc(sPtr->recipientPtrPtr, 
 		    allocated*sizeof(char*));
-	    sPtr->rListPtrPtr = (Tcl_DString*)REALLOC(sPtr->rListPtrPtr, 
+	    sPtr->rListPtrPtr = (Tcl_DString*)ckrealloc(sPtr->rListPtrPtr, 
 		    allocated*sizeof(Tcl_DString));
 	}
 	i = sPtr->numRecipients++;
 	sPtr->actionPtrPtr[i] = NULL;
 	sPtr->recTypePtrPtr[i] = NULL;
+	sPtr->recipientPtrPtr[i] = NULL;
 	Tcl_DStringInit(&sPtr->rListPtrPtr[i]);
 	while (length > 0) {
 	    body = RatParseDSNLine(body, name, value, &length);
@@ -781,6 +791,11 @@ RatParseDS(Tcl_Interp *interp, char *body, int length)
 		sPtr->actionPtrPtr[i] = cpystr(value);
 	    }
 	}
+	if (!sPtr->actionPtrPtr[i]) {
+	    sPtr->numRecipients--;
+	    sPtr->recTypePtrPtr[i] = NULL;
+	    sPtr->recipientPtrPtr[i] = NULL;
+	}
     }
     ckfree(name);
     ckfree(value);
@@ -818,8 +833,12 @@ RatFreeDeliveryStatus(RatDeliveryStatus *statusPtr)
     Tcl_DStringFree(&statusPtr->msgFields);
     if (statusPtr->numRecipients) {
 	for (i=0; i<statusPtr->numRecipients; i++) {
-	    ckfree(statusPtr->actionPtrPtr[i]);
-	    ckfree(statusPtr->recTypePtrPtr[i]);
+	    if (statusPtr->actionPtrPtr[i]) {
+		ckfree(statusPtr->actionPtrPtr[i]);
+	    }
+	    if (statusPtr->recTypePtrPtr[i]) {
+		ckfree(statusPtr->recTypePtrPtr[i]);
+	    }
 	    Tcl_DStringFree(&statusPtr->rListPtrPtr[i]);
 	}
 	ckfree(statusPtr->actionPtrPtr);
