@@ -3,8 +3,8 @@
  *
  *      This file contains code which implements standard c-client folders.
  *
- * TkRat software and its included text is Copyright 1996,1997,1998
- * by Martin Forssn
+ * TkRat software and its included text is Copyright 1996-2000 by
+ * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
  * COPYRIGHT, included with this distribution.
@@ -25,6 +25,7 @@ typedef struct DbFolderInfo {
     char *keywords;		/* Keywords to add to inserted messages */
     char *exDate;		/* Expiration date of inserted messages */
     char *exType;		/* Expiration type of new messages */
+    Tcl_Obj **infoPtr;		/* List of information caches */
 } DbFolderInfo;
 
 typedef enum {Db_Name, Db_Mail} DbAdrInfo;
@@ -32,12 +33,14 @@ typedef enum {Db_Name, Db_Mail} DbAdrInfo;
 /*
  * Procedures private to this module.
  */
+static RatInitProc Db_InitProc;
 static RatCloseProc Db_CloseProc;
 static RatUpdateProc Db_UpdateProc;
 static RatInsertProc Db_InsertProc;
 static RatSetFlagProc Db_SetFlagProc;
 static RatGetFlagProc Db_GetFlagProc;
 static RatCreateProc Db_CreateProc;
+static RatSetInfoProc Db_SetInfoProc;
 static int GetAddressInfo(Tcl_Interp *interp, Tcl_DString *dsPtr, char *adr,
 	DbAdrInfo info);
 
@@ -52,7 +55,7 @@ static int GetAddressInfo(Tcl_Interp *interp, Tcl_DString *dsPtr, char *adr,
  * Results:
  *      The return value is normally TCL_OK; if something goes wrong
  *	TCL_ERROR is returned and an error message will be left in
- *	interp->result.
+ *	the result area.
  *
  * Side effects:
  *	The C-client library is initialized and the apropriate mail drivers
@@ -81,7 +84,7 @@ RatDbFolderInit(Tcl_Interp *interp)
  * Results:
  *      The return value is normally TCL_OK; if something goes wrong
  *	TCL_ERROR is returned and an error message will be left in
- *	interp->result.
+ *	the result area.
  *
  * Side effects:
  *	A db folder is created.
@@ -91,27 +94,28 @@ RatDbFolderInit(Tcl_Interp *interp)
  */
 
 RatFolderInfo*
-RatDbFolderCreate(Tcl_Interp *interp, int argc, char *argv[])
+RatDbFolderCreate(Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[])
 {
     RatFolderInfo *infoPtr;
     DbFolderInfo *dbPtr;
     int *listPtr, number, i;
     Tcl_DString expression;
+    RatDbEntry *entryPtr;
 
-    if (argc < 6) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-		" ", argv[1], " keywords exDate exType search_expression\"",
-		(char *) NULL);
+    if (objc < 6) {
+	Tcl_AppendResult(interp, "wrong # args: should be \"",
+		Tcl_GetString(objv[0]), " ", Tcl_GetString(objv[1]),
+		" keywords exDate exType search_expression\"", (char *) NULL);
 	return (RatFolderInfo *) NULL;
     }
     Tcl_DStringInit(&expression);
-    for (i=5; i < argc; i++) {
-	Tcl_DStringAppendElement(&expression, argv[i]);
+    for (i=6; i < objc; i++) {
+	Tcl_DStringAppendElement(&expression, Tcl_GetString(objv[i]));
     }
     if (TCL_OK != RatDbSearch(interp, Tcl_DStringValue(&expression),
 	    &number, &listPtr)) {
-	Tcl_VarEval(interp, "RatLog 4 \"$t(dbase_error): ", interp->result,
-		"\" time", (char*)NULL);
+	RatLogF(interp, RAT_ERROR, "dbase_error", RATLOG_TIME,
+		Tcl_GetStringResult(interp));
 	Tcl_ResetResult(interp);
 	Tcl_AppendResult(interp, "Failed to search dbase \"",
 		Tcl_DStringValue(&expression), "\"", (char *) NULL);
@@ -125,28 +129,69 @@ RatDbFolderCreate(Tcl_Interp *interp, int argc, char *argv[])
     infoPtr->name = cpystr("Database search");
     infoPtr->type = "dbase";
     infoPtr->number = number;
+    infoPtr->recent = 0;
+    infoPtr->unseen = 0;
+    for (i=0; i<infoPtr->number; i++) {
+	entryPtr = RatDbGetEntry(listPtr[i]);
+	if (!strchr(entryPtr->content[STATUS], 'O')) {
+	    infoPtr->recent++;
+	}
+	if (!strchr(entryPtr->content[STATUS], 'R')) {
+	    infoPtr->unseen++;
+	}
+    }
     infoPtr->size = 0;
     for (i=0; i<number; i++) {
 	infoPtr->size += atoi(RatDbGetEntry(listPtr[i])->content[RSIZE]);
     }
+    infoPtr->initProc = Db_InitProc;
     infoPtr->closeProc = Db_CloseProc;
     infoPtr->updateProc = Db_UpdateProc;
     infoPtr->insertProc = Db_InsertProc;
     infoPtr->setFlagProc = Db_SetFlagProc;
     infoPtr->getFlagProc = Db_GetFlagProc;
     infoPtr->infoProc = Db_InfoProc;
+    infoPtr->setInfoProc = Db_SetInfoProc;
     infoPtr->createProc = Db_CreateProc;
+    infoPtr->syncProc = NULL;
     infoPtr->private = (ClientData) dbPtr;
     dbPtr->listPtr = listPtr;
     dbPtr->searchExpr = cpystr(Tcl_DStringValue(&expression));
-    dbPtr->keywords = cpystr(argv[2]);
-    dbPtr->exDate = cpystr(argv[3]);
-    dbPtr->exType = cpystr(argv[4]);
+    dbPtr->keywords = cpystr(Tcl_GetString(objv[3]));
+    dbPtr->exDate = cpystr(Tcl_GetString(objv[4]));
+    dbPtr->exType = cpystr(Tcl_GetString(objv[5]));
+    dbPtr->infoPtr = (Tcl_Obj**)ckalloc(sizeof(Tcl_Obj*)*number*RAT_FOLDER_END);
+    for (i=0; i<number*RAT_FOLDER_END; i++) {
+	dbPtr->infoPtr[i] = NULL;
+    }
     Tcl_DStringFree(&expression);
 
     return infoPtr;
 }
 
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Db_InitProc --
+ *
+ *      See the documentation for initProc in ratFolder.h
+ *
+ * Results:
+ *	A standard Tcl result.
+ *
+ * Side effects:
+ *	See the documentation for initProc in ratFolder.h
+ *
+ *
+ *----------------------------------------------------------------------
+ */
+static void
+Db_InitProc(RatFolderInfoPtr infoPtr, Tcl_Interp *interp, int index)
+{
+    return;
+}
+
 
 /*
  *----------------------------------------------------------------------
@@ -165,15 +210,25 @@ RatDbFolderCreate(Tcl_Interp *interp, int argc, char *argv[])
  *----------------------------------------------------------------------
  */
 static int
-Db_CloseProc(RatFolderInfoPtr infoPtr, Tcl_Interp *interp)
+Db_CloseProc(RatFolderInfoPtr infoPtr, Tcl_Interp *interp, int expunge)
 {
     DbFolderInfo *dbPtr = (DbFolderInfo *) infoPtr->private;
-    free(dbPtr->listPtr);
-    free(dbPtr->searchExpr);
-    free(dbPtr->keywords);
-    free(dbPtr->exDate);
-    free(dbPtr->exType);
-    free(dbPtr);
+    int i;
+
+    if (dbPtr->listPtr) {
+	ckfree(dbPtr->listPtr);
+    }
+    ckfree(dbPtr->searchExpr);
+    ckfree(dbPtr->keywords);
+    ckfree(dbPtr->exDate);
+    ckfree(dbPtr->exType);
+    for (i=0; i<infoPtr->number*RAT_FOLDER_END; i++) {
+	if (dbPtr->infoPtr[i]) {
+	    Tcl_DecrRefCount(dbPtr->infoPtr[i]);
+	}
+    }
+    ckfree(dbPtr->infoPtr);
+    ckfree(dbPtr);
     return TCL_OK;
 }
 
@@ -195,14 +250,14 @@ Db_CloseProc(RatFolderInfoPtr infoPtr, Tcl_Interp *interp)
  *----------------------------------------------------------------------
  */
 static int
-Db_UpdateProc(RatFolderInfoPtr infoPtr, Tcl_Interp *interp, int expunge)
+Db_UpdateProc(RatFolderInfoPtr infoPtr, Tcl_Interp *interp, RatUpdateType mode)
 {
     DbFolderInfo *dbPtr = (DbFolderInfo *) infoPtr->private;
-    int *listPtr, number;
+    int *listPtr, number, numNew, i;
+    RatDbEntry *entryPtr;
 
-    if (expunge) {
-	RatDbEntry *entryPtr;
-	int i, dst;
+    if (RAT_SYNC == mode) {
+	int i, j, dst;
 
 	if (TCL_OK != RatDbExpunge(interp)) {
 	    return -1;
@@ -211,29 +266,71 @@ Db_UpdateProc(RatFolderInfoPtr infoPtr, Tcl_Interp *interp, int expunge)
 	for (i=dst=0; i<infoPtr->number; i++) {
 	    if ((entryPtr = RatDbGetEntry(dbPtr->listPtr[i]))) {
 		dbPtr->listPtr[dst] = dbPtr->listPtr[i];
-		infoPtr->msgCmdPtr[dst++] = infoPtr->msgCmdPtr[i];
+		infoPtr->msgCmdPtr[dst] = infoPtr->msgCmdPtr[i];
 		infoPtr->size += atoi(entryPtr->content[RSIZE]);
+		for (j=0; j<RAT_FOLDER_END; j++) {
+		    dbPtr->infoPtr[dst*RAT_FOLDER_END+j] =
+			    dbPtr->infoPtr[i*RAT_FOLDER_END+j];
+		}
+		dst++;
 	    } else {
 		if (infoPtr->msgCmdPtr[i]) {
 		    RatMessageDelete(interp, infoPtr->msgCmdPtr[i]);
 		}
+		for (j=0; j<RAT_FOLDER_END; j++) {
+		    if (dbPtr->infoPtr[i*RAT_FOLDER_END+j]) {
+			Tcl_DecrRefCount(dbPtr->infoPtr[i*RAT_FOLDER_END+j]);
+		    }
+		}
 	    }
 	}
 	infoPtr->number = dst;
     }
-    if (TCL_OK != RatDbSearch(interp, dbPtr->searchExpr, &number, &listPtr)) {
-	Tcl_VarEval(interp, "RatLog 4 \"$t(dbase_error): ", interp->result,
-		"\" time", (char*)NULL);
-	Tcl_ResetResult(interp);
-	Tcl_AppendResult(interp, "Failed to search dbase \"",
-		dbPtr->searchExpr, "\"", (char *) NULL);
-	return -1;
+
+    numNew = 0;
+    if (RAT_SYNC == mode || RAT_UPDATE == mode) {
+	if (TCL_OK != RatDbSearch(interp, dbPtr->searchExpr, &number,&listPtr)){
+	    RatLogF(interp, RAT_ERROR, "dbase_error", RATLOG_TIME,
+		    Tcl_GetStringResult(interp));
+	    Tcl_ResetResult(interp);
+	    Tcl_AppendResult(interp, "Failed to search dbase \"",
+		    dbPtr->searchExpr, "\"", (char *) NULL);
+	    return -1;
+	}
+	for (i=0 ; number == infoPtr->number
+		&& i<number
+		&& listPtr[i] == dbPtr->listPtr[i];
+		i++);
+	if (i != number || i != infoPtr->number) {
+	    for (i=0; i<infoPtr->number*RAT_FOLDER_END; i++) {
+		if (dbPtr->infoPtr[i]) {
+		    Tcl_DecrRefCount(dbPtr->infoPtr[i]);
+		}
+	    }
+	    ckfree(dbPtr->infoPtr);
+	    ckfree(dbPtr->listPtr);
+	    dbPtr->listPtr = listPtr;
+	    numNew = infoPtr->number - number;
+	    infoPtr->number = number;
+	    dbPtr->infoPtr = (Tcl_Obj**)
+		    ckalloc(sizeof(Tcl_Obj*)*number*RAT_FOLDER_END);
+	    for (i=0; i<number*RAT_FOLDER_END; i++) {
+		dbPtr->infoPtr[i] = NULL;
+	    }
+	}
+	infoPtr->recent = 0;
+	infoPtr->unseen = 0;
+	for (i=0; i<infoPtr->number; i++) {
+	    entryPtr = RatDbGetEntry(dbPtr->listPtr[i]);
+	    if (!strchr(entryPtr->content[RAT_FOLDER_STATUS], 'O')) {
+		infoPtr->recent++;
+	    }
+	    if (!strchr(entryPtr->content[RAT_FOLDER_STATUS], 'R')) {
+		infoPtr->unseen++;
+	    }
+	}
     }
-    if (number >= infoPtr->number) {
-	ckfree(dbPtr->listPtr);
-	dbPtr->listPtr = listPtr;
-    }
-    return infoPtr->number;
+    return numNew;
 }
 
 
@@ -266,7 +363,7 @@ Db_InsertProc(RatFolderInfoPtr infoPtr, Tcl_Interp *interp, int argc,
 	    Tcl_AppendResult(interp, "No such message: ", argv[i], NULL);
 	    return TCL_ERROR;
 	}
-	RatInsertMsg(interp, (MessageInfo*)cmdInfo.clientData,
+	RatInsertMsg(interp, (MessageInfo*)cmdInfo.objClientData,
 		dbPtr->keywords, dbPtr->exDate, dbPtr->exType);
     }
     return TCL_OK;
@@ -305,6 +402,8 @@ Db_SetFlagProc(RatFolderInfoPtr infoPtr, Tcl_Interp *interp, int index,
 	    case 'D':	flagArray[RAT_DELETED] = 1; break;
 	    case 'F':	flagArray[RAT_FLAGGED] = 1; break;
 	    case 'A':	flagArray[RAT_ANSWERED] = 1; break;
+	    case 'T':	flagArray[RAT_DRAFT] = 1; break;
+	    case 'O':	flagArray[RAT_RECENT] = 1; break;
 	}
     }
     flagArray[flag] = value;
@@ -313,7 +412,14 @@ Db_SetFlagProc(RatFolderInfoPtr infoPtr, Tcl_Interp *interp, int index,
     if (flagArray[RAT_DELETED]) { newStatus[dst++] = 'D'; }
     if (flagArray[RAT_FLAGGED]) { newStatus[dst++] = 'F'; }
     if (flagArray[RAT_ANSWERED]) { newStatus[dst++] = 'A'; }
+    if (flagArray[RAT_DRAFT]) { newStatus[dst++] = 'T'; }
+    if (flagArray[RAT_RECENT]) { newStatus[dst++] = 'O'; }
     newStatus[dst] = '\0';
+    i = index*RAT_FOLDER_END+RAT_FOLDER_STATUS;
+    if (dbPtr->infoPtr[i]) {
+	Tcl_DecrRefCount(dbPtr->infoPtr[i]);
+	dbPtr->infoPtr[i] = NULL;
+    }
     return RatDbSetStatus(interp, dbPtr->listPtr[index], newStatus);
 }
 
@@ -348,6 +454,8 @@ Db_GetFlagProc(RatFolderInfoPtr infoPtr, Tcl_Interp *interp, int index,
 	case RAT_DELETED:	flagChar = 'D'; break;
 	case RAT_FLAGGED:	flagChar = 'F'; break;
 	case RAT_ANSWERED:	flagChar = 'A'; break;
+	case RAT_DRAFT:		flagChar = 'T'; break;
+	case RAT_RECENT:	flagChar = 'O'; break;
 	default:		return 0;
     }
     for (i=0; entryPtr->content[STATUS][i]; i++) {
@@ -386,7 +494,7 @@ GetAddressInfo(Tcl_Interp *interp, Tcl_DString *dsPtr, char *adr,
     int ret;
 
     rfc822_parse_adrlist(&addressPtr, s, currentHost);
-    free(s);
+    ckfree(s);
     if (!addressPtr) {
 	return 0;
     }
@@ -419,28 +527,67 @@ GetAddressInfo(Tcl_Interp *interp, Tcl_DString *dsPtr, char *adr,
  *----------------------------------------------------------------------
  */
 
-char*
+Tcl_Obj*
 Db_InfoProc(Tcl_Interp *interp, ClientData clientData, RatFolderInfoType type,
 	int index)
+{
+    RatFolderInfo *infoPtr = (RatFolderInfo*)clientData;
+
+    return Db_InfoProcInt(interp, infoPtr, type, index);
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Db_InfoProcInt --
+ *
+ *      See the documentation for infoProc in ratFolder.h. The difference
+ *	between this function and Db_InfoProc is that this expects the
+ *	index (rIndex) to be the real folder index.
+ *
+ * Results:
+ *	A standard Tcl result.
+ *
+ * Side effects:
+ *	See the documentation for infoProc in ratFolder.h
+ *
+ *
+ *----------------------------------------------------------------------
+ */
+
+Tcl_Obj*
+Db_InfoProcInt(Tcl_Interp *interp, RatFolderInfo *infoPtr,
+	RatFolderInfoType type, int rIndex)
 {
     static Tcl_DString ds;
     static int initialized = 0;
     static char buf[1024];
     int i, seen, deleted, marked, answered, me, dbIndex, zone;
-    RatFolderInfo *infoPtr = (RatFolderInfo*)clientData;
     DbFolderInfo *dbPtr = (DbFolderInfo*)infoPtr->private;
     ADDRESS *addressPtr, *address2Ptr;
+    Tcl_Obj *oPtr = NULL;
     MESSAGECACHE elt;
     RatDbEntry *entryPtr;
     struct tm *tmPtr;
     Tcl_CmdInfo info;
     time_t time;
 
-    if (index >= 0) {
-	dbIndex = dbPtr->listPtr[infoPtr->presentationOrder[index]];
-    } else {
-	dbIndex = -index;
+    dbIndex = dbPtr->listPtr[rIndex];
+
+    if (dbPtr->infoPtr[rIndex*RAT_FOLDER_END+type]) {
+	if (type == RAT_FOLDER_INDEX) {
+	    Tcl_GetIntFromObj(interp,
+		    dbPtr->infoPtr[rIndex*RAT_FOLDER_END+type], &i);
+	    if (i < infoPtr->visible
+		    && dbIndex==dbPtr->listPtr[infoPtr->presentationOrder[i]]) {
+		return dbPtr->infoPtr[rIndex*RAT_FOLDER_END+type];
+	    }
+	} else {
+	    return dbPtr->infoPtr[rIndex*RAT_FOLDER_END+type];
+	}
     }
+
     entryPtr = RatDbGetEntry(dbIndex);
 
     if (!initialized) {
@@ -450,7 +597,11 @@ Db_InfoProc(Tcl_Interp *interp, ClientData clientData, RatFolderInfoType type,
 
     switch (type) {
 	case RAT_FOLDER_SUBJECT:
-	    return entryPtr->content[SUBJECT];
+	    oPtr = Tcl_NewStringObj(entryPtr->content[SUBJECT], -1);
+	    break;
+	case RAT_FOLDER_CANONSUBJECT:
+	    oPtr = RatFolderCanonalizeSubject(entryPtr->content[SUBJECT]);
+	    break;
 	case RAT_FOLDER_NAME:
 	    Tcl_DStringSetLength(&ds, 0);
 	    if (GetAddressInfo(interp, &ds, entryPtr->content[FROM], Db_Name)) {
@@ -460,11 +611,13 @@ Db_InfoProc(Tcl_Interp *interp, ClientData clientData, RatFolderInfoType type,
 		Tcl_DStringAppend(&ds, ": ", 2);
 		GetAddressInfo(interp, &ds, entryPtr->content[TO], Db_Name);
 	    }
-	    return Tcl_DStringValue(&ds);
+	    oPtr = Tcl_NewStringObj(Tcl_DStringValue(&ds), -1);
+	    break;
 	case RAT_FOLDER_MAIL_REAL:
 	    Tcl_DStringSetLength(&ds, 0);
 	    GetAddressInfo(interp, &ds, entryPtr->content[FROM], Db_Mail);
-	    return Tcl_DStringValue(&ds);
+	    oPtr = Tcl_NewStringObj(Tcl_DStringValue(&ds), -1);
+	    break;
 	case RAT_FOLDER_MAIL:
 	    Tcl_DStringSetLength(&ds, 0);
 	    if (GetAddressInfo(interp, &ds, entryPtr->content[FROM], Db_Mail)) {
@@ -474,10 +627,12 @@ Db_InfoProc(Tcl_Interp *interp, ClientData clientData, RatFolderInfoType type,
 		Tcl_DStringAppend(&ds, ": ", 2);
 		GetAddressInfo(interp, &ds, entryPtr->content[TO], Db_Mail);
 	    }
-	    return Tcl_DStringValue(&ds);
+	    oPtr = Tcl_NewStringObj(Tcl_DStringValue(&ds), -1);
+	    break;
 	case RAT_FOLDER_NAME_RECIPIENT:
 	    if (RatIsEmpty(entryPtr->content[TO])) {
-		return entryPtr->content[TO];
+		oPtr = Tcl_NewStringObj(entryPtr->content[TO], -1);
+		break;
 	    }
 	    Tcl_DStringSetLength(&ds, 0);
 	    if (GetAddressInfo(interp, &ds, entryPtr->content[TO], Db_Name)) {
@@ -487,10 +642,12 @@ Db_InfoProc(Tcl_Interp *interp, ClientData clientData, RatFolderInfoType type,
 		Tcl_DStringAppend(&ds, ": ", 2);
 		GetAddressInfo(interp, &ds, entryPtr->content[FROM], Db_Name);
 	    }
-	    return Tcl_DStringValue(&ds);
+	    oPtr = Tcl_NewStringObj(Tcl_DStringValue(&ds), -1);
+	    break;
 	case RAT_FOLDER_MAIL_RECIPIENT:
 	    if (RatIsEmpty(entryPtr->content[TO])) {
-		return entryPtr->content[TO];
+		oPtr = Tcl_NewStringObj(entryPtr->content[TO], -1);
+		break;
 	    }
 	    Tcl_DStringSetLength(&ds, 0);
 	    if (GetAddressInfo(interp, &ds, entryPtr->content[TO], Db_Mail)) {
@@ -500,17 +657,22 @@ Db_InfoProc(Tcl_Interp *interp, ClientData clientData, RatFolderInfoType type,
 		Tcl_DStringAppend(&ds, ": ", 2);
 		GetAddressInfo(interp, &ds, entryPtr->content[FROM], Db_Mail);
 	    }
-	    return Tcl_DStringValue(&ds);
+	    oPtr = Tcl_NewStringObj(Tcl_DStringValue(&ds), -1);
+	    break;
 	case RAT_FOLDER_SIZE:
-	    return entryPtr->content[RSIZE];
+	    oPtr = Tcl_NewIntObj(atoi(entryPtr->content[RSIZE]));
+	    break;
 	case RAT_FOLDER_SIZE_F:
-	    return RatMangleNumber(atoi(entryPtr->content[RSIZE]));
+	    oPtr = RatMangleNumber(atoi(entryPtr->content[RSIZE]));
+	    break;
 	case RAT_FOLDER_DATE_F:
 	    time = atoi(entryPtr->content[DATE]);
 	    tmPtr = localtime(&time);
-	    return RatFormatDate(interp, tmPtr->tm_mon, tmPtr->tm_mday);
+	    oPtr = RatFormatDate(interp, tmPtr->tm_mon, tmPtr->tm_mday);
+	    break;
 	case RAT_FOLDER_DATE_N:
-	    return entryPtr->content[DATE];
+	    oPtr = Tcl_NewStringObj(entryPtr->content[DATE], -1);
+	    break;
 	case RAT_FOLDER_DATE_IMAP4:
 	    Tcl_DStringSetLength(&ds, 256);
 	    time = atoi(entryPtr->content[DATE]);
@@ -531,7 +693,8 @@ Db_InfoProc(Tcl_Interp *interp, ClientData clientData, RatFolderInfoType type,
 		elt.zhours = (-1*zone)/(60*60);
 		elt.zminutes = ((-1*zone)%(60*60))/60;
 	    }
-	    return mail_date(Tcl_DStringValue(&ds), &elt);
+	    oPtr = Tcl_NewStringObj(mail_date(Tcl_DStringValue(&ds), &elt), -1);
+	    break;
 	case RAT_FOLDER_STATUS:
 	    seen = deleted = marked = answered = me = 0;
 	    for (i=0; entryPtr->content[STATUS][i]; i++) {
@@ -575,7 +738,8 @@ Db_InfoProc(Tcl_Interp *interp, ClientData clientData, RatFolderInfoType type,
 		buf[i++] = ' ';
 	    }
 	    buf[i] = '\0';
-	    return buf;
+	    oPtr = Tcl_NewStringObj(buf, -1);
+	    break;
 	case RAT_FOLDER_TYPE:		
 	    return NULL;
 	case RAT_FOLDER_PARAMETERS:
@@ -584,22 +748,30 @@ Db_InfoProc(Tcl_Interp *interp, ClientData clientData, RatFolderInfoType type,
 	    for (i=0; i < infoPtr->number; i++) {
 		if (dbIndex == dbPtr->listPtr[infoPtr->presentationOrder[i]]) {
 		    sprintf(buf, "%d", i+1);
-		    return buf;
+		    oPtr = Tcl_NewStringObj(buf, -1);
+		    break;
 		}
 	    }
-	    return "1";
+	    if (i == infoPtr->number) {
+		oPtr = Tcl_NewStringObj("1", 1);
+	    }
+	    break;
 	case RAT_FOLDER_TO:
-	    return entryPtr->content[TO];
+	    oPtr = Tcl_NewStringObj(entryPtr->content[TO], -1);
+	    break;
 	case RAT_FOLDER_FROM:
-	    return entryPtr->content[FROM];
+	    oPtr = Tcl_NewStringObj(entryPtr->content[FROM], -1);
+	    break;
 	case RAT_FOLDER_SENDER:		/* fallthrough */
 	case RAT_FOLDER_CC:		/* fallthrough */
 	case RAT_FOLDER_REPLY_TO:
-	    if (NULL == infoPtr->msgCmdPtr[index]) {
-		infoPtr->msgCmdPtr[index] = Db_CreateProc(infoPtr,interp,index);
+	    if (NULL == infoPtr->msgCmdPtr[rIndex]) {
+		infoPtr->msgCmdPtr[rIndex] =
+		    Db_CreateProc(infoPtr, interp, rIndex);
 	    }
-	    Tcl_GetCommandInfo(interp, infoPtr->msgCmdPtr[index], &info);
-	    return (*infoPtr->infoProc)(interp, info.clientData, type, index);
+	    Tcl_GetCommandInfo(interp, infoPtr->msgCmdPtr[rIndex], &info);
+	    oPtr = RatMsgInfo(interp, (MessageInfo*)info.objClientData, type);
+	    break;
 	case RAT_FOLDER_FLAGS:
 	    Tcl_DStringSetLength(&ds, 0);
 	    for (i=0; entryPtr->content[STATUS][i]; i++) {
@@ -612,17 +784,32 @@ Db_InfoProc(Tcl_Interp *interp, ClientData clientData, RatFolderInfoType type,
 			Tcl_DStringAppend(&ds, " " RAT_FLAGGED_STR, -1); break;
 		    case 'A':
 			Tcl_DStringAppend(&ds, " " RAT_ANSWERED_STR, -1); break;
+		    case 'T':
+			Tcl_DStringAppend(&ds, " " RAT_DRAFT_STR, -1); break;
 		}
 	    }
 	    if (Tcl_DStringLength(&ds)) {
-		return Tcl_DStringValue(&ds)+1;
+		oPtr = Tcl_NewStringObj(Tcl_DStringValue(&ds)+1, -1);
 	    } else {
-		return "";
+		oPtr = Tcl_NewStringObj("", 0);
 	    }
+	    break;
+	case RAT_FOLDER_UNIXFLAGS:
+	    oPtr = Tcl_NewStringObj(entryPtr->content[STATUS], -1);
+	    break;
+	case RAT_FOLDER_MSGID:
+	    oPtr = Tcl_NewStringObj(entryPtr->content[MESSAGE_ID], -1);
+	    break;
+	case RAT_FOLDER_REF:
+	    oPtr = Tcl_NewStringObj(entryPtr->content[REFERENCE], -1);
+	    break;
+	case RAT_FOLDER_THREADING:
+	    return NULL;
 	case RAT_FOLDER_END:
 	    break;
     }
-    return NULL;
+    dbPtr->infoPtr[rIndex*RAT_FOLDER_END+type] = oPtr;
+    return oPtr;
 }
 
 /*
@@ -645,5 +832,38 @@ static char*
 Db_CreateProc(RatFolderInfoPtr infoPtr, Tcl_Interp *interp, int index)
 {
     DbFolderInfo *dbPtr = (DbFolderInfo *) infoPtr->private;
-    return RatDbMessageCreate(interp, infoPtr, dbPtr->listPtr[index]);
+    return RatDbMessageCreate(interp, infoPtr, index, dbPtr->listPtr[index]);
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Db_SetInfoProc --
+ *
+ *      Sets information about a message
+ *
+ * Results:
+ *	None
+ *
+ * Side effects:
+ *	None
+ *
+ *
+ *----------------------------------------------------------------------
+ */
+static void
+Db_SetInfoProc(Tcl_Interp *interp, ClientData clientData,
+	RatFolderInfoType type, int index, Tcl_Obj *oPtr)
+{
+    RatFolderInfo *infoPtr = (RatFolderInfo*)clientData;
+    DbFolderInfo *dbPtr = (DbFolderInfo*)infoPtr->private;
+    int i = index*RAT_FOLDER_END+type;
+
+    if (dbPtr->infoPtr[i]) {
+	Tcl_DecrRefCount(dbPtr->infoPtr[i]);
+    }
+    dbPtr->infoPtr[i] = oPtr;
+    if (oPtr) {
+	Tcl_IncrRefCount(oPtr);
+    }
 }
