@@ -3,8 +3,8 @@
  *
  *	This file contains code which implements dbase messages.
  *
- * TkRat software and its included text is Copyright 1996,1997,1998
- * by Martin Forssn
+ * TkRat software and its included text is Copyright 1996-2000 by
+ * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
  * COPYRIGHT, included with this distribution.
@@ -16,6 +16,7 @@
  * The ClientData for each message entity
  */
 typedef struct DbMessageInfo {
+    int index;
     char *buffer;
     MESSAGE *messagePtr;
 } DbMessageInfo;
@@ -24,7 +25,7 @@ typedef struct DbMessageInfo {
  * The ClientData for each bodypart entity
  */
 typedef struct DbBodyInfo {
-    char *text;
+    unsigned char *text;
 } DbBodyInfo;
 
 /*
@@ -39,11 +40,11 @@ static RatInfoProc Db_GetInfoProc;
 static RatCreateBodyProc Db_CreateBodyProc;
 static RatFetchTextProc Db_FetchTextProc;
 static RatEnvelopeProc Db_EnvelopeProc;
-static RatSetIndexProc Db_SetIndexProc;
 static RatMsgDeleteProc Db_MsgDeleteProc;
 static RatMakeChildrenProc Db_MakeChildrenProc;
 static RatFetchBodyProc Db_FetchBodyProc;
 static RatBodyDeleteProc Db_BodyDeleteProc;
+static RatGetInternalDateProc Db_GetInternalDateProc;
 
 
 /*
@@ -72,11 +73,11 @@ RatDbMessagesInit(MessageProcInfo *messageProcInfoPtr)
     messageProcInfoPtr->createBodyProc = Db_CreateBodyProc;
     messageProcInfoPtr->fetchTextProc = Db_FetchTextProc;
     messageProcInfoPtr->envelopeProc = Db_EnvelopeProc;
-    messageProcInfoPtr->setIndexProc = Db_SetIndexProc;
     messageProcInfoPtr->msgDeleteProc = Db_MsgDeleteProc;
     messageProcInfoPtr->makeChildrenProc = Db_MakeChildrenProc;
     messageProcInfoPtr->fetchBodyProc = Db_FetchBodyProc;
     messageProcInfoPtr->bodyDeleteProc = Db_BodyDeleteProc;
+    messageProcInfoPtr->getInternalDateProc = Db_GetInternalDateProc;
 }
 
 
@@ -99,24 +100,29 @@ RatDbMessagesInit(MessageProcInfo *messageProcInfoPtr)
  */
 
 char*
-RatDbMessageCreate(Tcl_Interp *interp, RatFolderInfoPtr infoPtr, int index)
+RatDbMessageCreate(Tcl_Interp *interp, RatFolderInfoPtr infoPtr, int index,
+	int dbIndex)
 {
-    DbMessageInfo *msgDbPtr=(DbMessageInfo*)ckalloc(sizeof(DbMessageInfo));
+    DbMessageInfo *dbMsgPtr=(DbMessageInfo*)ckalloc(sizeof(DbMessageInfo));
     MessageInfo *msgPtr=(MessageInfo*)ckalloc(sizeof(MessageInfo));
-    char *name = (char*)ckalloc(16);
+    int i;
 
     msgPtr->folderInfoPtr = infoPtr;
-    msgPtr->name = name;
     msgPtr->type = RAT_DBASE_MESSAGE;
     msgPtr->bodyInfoPtr = NULL;
     msgPtr->msgNo = index;
     msgPtr->fromMe = RAT_ISME_UNKOWN;
     msgPtr->toMe = RAT_ISME_UNKOWN;
-    msgPtr->clientData = (ClientData)msgDbPtr;
-    msgDbPtr->messagePtr = RatDbGetMessage(interp, index, &msgDbPtr->buffer);
-    sprintf(name, "RatDbMsg%d", numDbMessages++);
-    Tcl_CreateCommand(interp, name, RatMessageCmd, (ClientData) msgPtr, NULL);
-    return name;
+    msgPtr->clientData = (ClientData)dbMsgPtr;
+    for (i=0; i<sizeof(msgPtr->info)/sizeof(*msgPtr->info); i++) {
+	msgPtr->info[i] = NULL;
+    }
+    dbMsgPtr->index = dbIndex;
+    dbMsgPtr->messagePtr = RatDbGetMessage(interp, dbIndex, &dbMsgPtr->buffer);
+    sprintf(msgPtr->name, "RatDbMsg%d", numDbMessages++);
+    Tcl_CreateObjCommand(interp, msgPtr->name, RatMessageCmd,
+	    (ClientData) msgPtr, NULL);
+    return msgPtr->name;
 }
 
 
@@ -133,7 +139,8 @@ RatDbMessageCreate(Tcl_Interp *interp, RatFolderInfoPtr infoPtr, int index)
 static char*
 Db_GetHeadersProc(Tcl_Interp *interp, MessageInfo *msgPtr)
 {
-    return RatDbGetHeaders(interp, msgPtr->msgNo);
+    DbMessageInfo *dbMsgPtr=(DbMessageInfo*)msgPtr->clientData;
+    return RatDbGetHeaders(interp, dbMsgPtr->index);
 }
 
 
@@ -151,16 +158,17 @@ static char*
 Db_GetEnvelopeProc(Tcl_Interp *interp, MessageInfo *msgPtr)
 {
     static char buf[1024];
-    RatDbEntry *entryPtr = RatDbGetEntry(msgPtr->msgNo);
+    DbMessageInfo *dbMsgPtr=(DbMessageInfo*)msgPtr->clientData;
+    RatDbEntry *entryPtr = RatDbGetEntry(dbMsgPtr->index);
     struct tm *tmPtr;
     time_t date;
 
     date = atoi(entryPtr->content[DATE]);
     tmPtr = gmtime(&date);
-    sprintf(buf, "From %s %s %s %2d %02d:%02d GMT 19%02d\n",
+    snprintf(buf, sizeof(buf), "From %s %s %s %2d %02d:%02d GMT %04d\n",
 	    entryPtr->content[FROM], dayName[tmPtr->tm_wday],
 	    monthName[tmPtr->tm_mon], tmPtr->tm_mday, tmPtr->tm_hour,
-	    tmPtr->tm_min, tmPtr->tm_year);
+	    tmPtr->tm_min, tmPtr->tm_year+1900);
     return buf;
 }
 
@@ -179,7 +187,7 @@ static BodyInfo*
 Db_CreateBodyProc(Tcl_Interp *interp, MessageInfo *msgPtr)
 {
     DbMessageInfo *dbMsgPtr = (DbMessageInfo*)msgPtr->clientData;
-    DbBodyInfo *dbBodyInfoPtr = (DbBodyInfo*)malloc(sizeof(DbBodyInfo));
+    DbBodyInfo *dbBodyInfoPtr = (DbBodyInfo*)ckalloc(sizeof(DbBodyInfo));
     msgPtr->bodyInfoPtr = CreateBodyInfo(msgPtr);
 
     msgPtr->bodyInfoPtr->bodyPtr = dbMsgPtr->messagePtr->body;
@@ -203,7 +211,8 @@ Db_CreateBodyProc(Tcl_Interp *interp, MessageInfo *msgPtr)
 static char*
 Db_FetchTextProc(Tcl_Interp *interp, MessageInfo *msgPtr)
 {
-    return RatDbGetText(interp, msgPtr->msgNo);
+    DbMessageInfo *dbMsgPtr = (DbMessageInfo*)msgPtr->clientData;
+    return RatDbGetText(interp, dbMsgPtr->index);
 }
 
 
@@ -223,23 +232,6 @@ Db_EnvelopeProc(MessageInfo *msgPtr)
     return ((DbMessageInfo*)msgPtr->clientData)->messagePtr->env;
 }
 
-
-/*
- *----------------------------------------------------------------------
- *
- * Db_SetIndexProc --
- *
- *      See ratFolder.h
- *
- *----------------------------------------------------------------------
- */
-
-static void
-Db_SetIndexProc(MessageInfo *msgPtr, int index)
-{
-     return;
-}
-
 
 /*
  *----------------------------------------------------------------------
@@ -287,7 +279,7 @@ Db_MakeChildrenProc(Tcl_Interp *interp, BodyInfo *bodyInfoPtr)
 	for (partPtr = bodyPtr->nested.part; partPtr;
 		partPtr = partPtr->next) {
 	    partInfoPtr = CreateBodyInfo(bodyInfoPtr->msgPtr);
-	    dbPartInfoPtr = (DbBodyInfo*)malloc(sizeof(DbBodyInfo));
+	    dbPartInfoPtr = (DbBodyInfo*)ckalloc(sizeof(DbBodyInfo));
 	    *partInfoPtrPtr = partInfoPtr;
 	    partInfoPtr->bodyPtr = &partPtr->body;
 	    partInfoPtrPtr = &partInfoPtr->nextPtr;
@@ -319,7 +311,7 @@ Db_FetchBodyProc(BodyInfo *bodyInfoPtr, unsigned long *lengthPtr)
 	return Tcl_DStringValue(bodyInfoPtr->decodedTextPtr);
     }
     *lengthPtr = bodyInfoPtr->bodyPtr->contents.text.size;
-    return dbBodyInfoPtr->text;
+    return (char*)dbBodyInfoPtr->text;
 }
 
 
@@ -339,6 +331,27 @@ Db_BodyDeleteProc(BodyInfo *bodyInfoPtr)
     DbBodyInfo *dbBodyInfoPtr = (DbBodyInfo*)bodyInfoPtr->clientData;
     ckfree(dbBodyInfoPtr);
 }
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Db_GetInternalDateProc --
+ *
+ *      See ratFolder.h
+ *
+ *----------------------------------------------------------------------
+ */
+
+static MESSAGECACHE*
+Db_GetInternalDateProc(Tcl_Interp *interp, MessageInfo *msgPtr)
+{
+    DbMessageInfo *dbMsgPtr = (DbMessageInfo*)msgPtr->clientData;
+    char *from;
+
+    from = RatDbGetFrom(interp, dbMsgPtr->index);
+    return RatParseFrom(from);
+}
 
 /*
  *----------------------------------------------------------------------
@@ -350,22 +363,18 @@ Db_BodyDeleteProc(BodyInfo *bodyInfoPtr)
  *----------------------------------------------------------------------
  */
 
-char*
+Tcl_Obj*
 Db_GetInfoProc(Tcl_Interp *interp, ClientData clientData,
 	RatFolderInfoType type, int notused)
 {
     MessageInfo *msgPtr = (MessageInfo*)clientData;
     MESSAGE *messagePtr = ((DbMessageInfo*)msgPtr->clientData)->messagePtr;
-    static Tcl_DString ds;
-    static int initialized = 0;
-
-    if (!initialized) {
-	Tcl_DStringInit(&ds);
-	initialized = 1;
-    }
+    ADDRESS *adrPtr;
+    Tcl_Obj *oPtr;
 
     switch (type) {
 	case RAT_FOLDER_SUBJECT:	/* fallthrough */
+	case RAT_FOLDER_CANONSUBJECT:	/* fallthrough */
 	case RAT_FOLDER_NAME:		/* fallthrough */
 	case RAT_FOLDER_MAIL_REAL:	/* fallthrough */
 	case RAT_FOLDER_MAIL:		/* fallthrough */
@@ -382,24 +391,29 @@ Db_GetInfoProc(Tcl_Interp *interp, ClientData clientData,
 	case RAT_FOLDER_FROM:		/* fallthrough */
 	case RAT_FOLDER_STATUS:		/* fallthrough */
 	case RAT_FOLDER_FLAGS:		/* fallthrough */
+	case RAT_FOLDER_UNIXFLAGS:	/* fallthrough */
+	case RAT_FOLDER_MSGID:		/* fallthrough */
+	case RAT_FOLDER_REF:		/* fallthrough */
+	case RAT_FOLDER_THREADING:	/* fallthrough */
 	case RAT_FOLDER_INDEX:
-	    return Db_InfoProc(interp, (ClientData)msgPtr->folderInfoPtr, type,
-		    -msgPtr->msgNo);
+	    return Db_InfoProcInt(interp, msgPtr->folderInfoPtr,
+		    type, msgPtr->msgNo);
 	case RAT_FOLDER_SENDER:		/* fallthrough */
 	case RAT_FOLDER_CC:		/* fallthrough */
 	case RAT_FOLDER_REPLY_TO:
-	    Tcl_DStringSetLength(&ds, 2048);
 	    if (type == RAT_FOLDER_SENDER) {
-		rfc822_write_address(Tcl_DStringValue(&ds),
-			messagePtr->env->sender);
+		adrPtr = messagePtr->env->sender;
 	    } else if (type == RAT_FOLDER_CC) {
-		rfc822_write_address(Tcl_DStringValue(&ds),
-			messagePtr->env->cc);
+		adrPtr = messagePtr->env->cc;
 	    } else {
-		rfc822_write_address(Tcl_DStringValue(&ds),
-			messagePtr->env->reply_to);
+		adrPtr = messagePtr->env->reply_to;
 	    }
-	    return Tcl_DStringValue(&ds);
+	    oPtr = Tcl_NewStringObj("", 0);
+	    Tcl_SetObjLength(oPtr, RatAddressSize(adrPtr, 1));
+	    Tcl_GetString(oPtr)[0] = '\0';
+	    rfc822_write_address(Tcl_GetString(oPtr), adrPtr);
+	    Tcl_SetObjLength(oPtr, strlen(Tcl_GetString(oPtr)));
+	    return oPtr;
 	case RAT_FOLDER_END:
 	    break;
     }
