@@ -3,8 +3,8 @@
  *
  *	This file handles the expressions.
  *
- * TkRat software and its included text is Copyright 1996,1997,1998
- * by Martin Forssn
+ * TkRat software and its included text is Copyright 1996-2000 by
+ * Martin Forssn
  *
  * The full text of my legal notices is contained in the file called
  * COPYRIGHT, included with this distribution.
@@ -109,7 +109,7 @@ GetToken(char **sPtr)
     char *cPtr = *sPtr;
     int i;
 
-    while (isspace(*cPtr)) {
+    while (isspace((unsigned char)*cPtr)) {
 	cPtr++;
     }
     *sPtr = cPtr;
@@ -151,7 +151,7 @@ GetString(char **sPtr)
     int i;
 
 
-    while (isspace(*cPtr)) {
+    while (isspace((unsigned char)*cPtr)) {
 	cPtr++;
     }
     if ('\'' == *cPtr || '"' == *cPtr || '{' == *cPtr) {
@@ -161,14 +161,15 @@ GetString(char **sPtr)
     if ('{' == quote) {
 	quote = '}';
     }
-    result = (char*)malloc(strlen(cPtr)+1);
+    result = (char*)ckalloc(strlen(cPtr)+1);
     i=0;
-    while (*cPtr && !(quote == *cPtr || (!quote && isspace(*cPtr)))) {
+    while (*cPtr && !(quote == *cPtr
+		      || (!quote && isspace((unsigned char)*cPtr)))) {
 	if ('\\' == *cPtr && cPtr[1]) {
 	    cPtr++;
 	}
-	if (isupper(*cPtr)) {
-	    result[i] = tolower(*cPtr);
+	if (isupper((unsigned char)*cPtr)) {
+	    result[i] = tolower((unsigned char)*cPtr);
 	} else {
 	    result[i] = *cPtr;
 	}
@@ -212,10 +213,10 @@ FreeExp(Expression *expPtr)
 	FreeExp(expPtr->arg2.expPtr);
     } else {
 	if (expPtr->arg2.string) {
-	    free(expPtr->arg2.string);
+	    ckfree(expPtr->arg2.string);
 	}
     }
-    free(expPtr);
+    ckfree(expPtr);
 }
 
 /*
@@ -260,28 +261,28 @@ ParseExpression(char **sPtr, char **errPtr, int inParen)
 		    *errPtr = "Expected boolean or ')'";
 		    return expPtr;
 		}
-		exp2Ptr = (Expression*)malloc(sizeof(Expression));
+		exp2Ptr = (Expression*)ckalloc(sizeof(Expression));
 		exp2Ptr->negate = negated;
 		exp2Ptr->arg1.info = tokPtr->info;
 		tokPtr = GetToken(sPtr);
 		if (!tokPtr || tokPtr->type != TT_Operator) {
 		    *errPtr = "Expected operator";
-		    free(exp2Ptr);
+		    ckfree(exp2Ptr);
 		    return expPtr;
 		}
 		exp2Ptr->op = tokPtr->token;
 		exp2Ptr->arg2.string = GetString(sPtr);
 		if (!exp2Ptr->arg2.string) {
 		    *errPtr = "String expected";
-		    free(exp2Ptr);
+		    ckfree(exp2Ptr);
 		    return expPtr;
 		} else if (T_Is == exp2Ptr->op) {
 		    exp2Ptr->op = T_Has;
-		    newString = (char*)malloc(strlen(exp2Ptr->arg2.string)+2);
+		    newString = (char*)ckalloc(strlen(exp2Ptr->arg2.string)+3);
 		    strcpy(newString, "^");
 		    strcat(newString, exp2Ptr->arg2.string);
 		    strcat(newString, "$");
-		    free(exp2Ptr->arg2.string);
+		    ckfree(exp2Ptr->arg2.string);
 		    exp2Ptr->arg2.string = newString;
 		}
 		if (expPtr) {
@@ -295,7 +296,7 @@ ParseExpression(char **sPtr, char **errPtr, int inParen)
 		    *errPtr = "Must have a valid expression before a boolean";
 		    return expPtr;
 		}
-		exp2Ptr = (Expression*)malloc(sizeof(Expression));
+		exp2Ptr = (Expression*)ckalloc(sizeof(Expression));
 		exp2Ptr->negate = negated;
 		exp2Ptr->op = tokPtr->token;
 		exp2Ptr->arg1.expPtr = expPtr;
@@ -382,12 +383,12 @@ RatParseExpCmd(ClientData clientData, Tcl_Interp *interp, int argc,
 	return TCL_ERROR;
     }
 
-    elemPtr = (ExpList*)malloc(sizeof(ExpList));
+    elemPtr = (ExpList*)ckalloc(sizeof(ExpList));
     elemPtr->id = expCounter;
     elemPtr->expPtr = expPtr;
     elemPtr->next = expListPtr;
     expListPtr = elemPtr;
-    sprintf(interp->result, "%d", expCounter++);
+    Tcl_SetObjResult(interp, Tcl_NewIntObj(expCounter++));
     return TCL_OK;
 }
 
@@ -443,7 +444,7 @@ GetExpression(Tcl_DString *dsPtr, Expression *expPtr)
  *      See ../doc/interface
  *
  * Results:
- *	The identified expression is returned as a string in interp->result.
+ *	The identified expression is returned as a string in the result area.
  *
  * Side effects:
  *	None.
@@ -514,7 +515,7 @@ RatFreeExpCmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
 	    elemPtr = *elemPtrPtr;
 	    FreeExp(elemPtr->expPtr);
 	    *elemPtrPtr = elemPtr->next;
-	    free(elemPtr);
+	    ckfree(elemPtr);
 	    return TCL_OK;
 	}
     }
@@ -557,7 +558,7 @@ RatExpMatch(Tcl_Interp *interp, int expId, RatInfoProc *infoProc,
 /*
  *----------------------------------------------------------------------
  *
- * RatExpMatch --
+ * RatExpMatchDo --
  *
  *      Checks if a given expression matches to given message.
  *	This routine actually does the checking and may call itself
@@ -577,8 +578,10 @@ int
 RatExpMatchDo(Tcl_Interp *interp, Expression *expPtr, RatInfoProc *infoProc,
 	ClientData clientData, int index)
 {
-    char *s, *sLower, *cPtr;
+    char *sLower, *cPtr;
     int opIndex, val;
+    static Tcl_Obj *sPtr = NULL;
+    Tcl_Obj *oPtr;
 
     for (opIndex=0; tokenList[opIndex].token != expPtr->op; opIndex++);
 
@@ -595,26 +598,31 @@ RatExpMatchDo(Tcl_Interp *interp, Expression *expPtr, RatInfoProc *infoProc,
 	}
 	return val;
     } else {
-	s = (*infoProc)(interp, clientData, expPtr->arg1.info, index);
-	if (!s) {
-	    s = "";
+	oPtr = (*infoProc)(interp, clientData, expPtr->arg1.info, index);
+	if (!oPtr) {
+	    if (!sPtr) {
+		sPtr = Tcl_NewObj();
+		Tcl_IncrRefCount(sPtr);
+	    }
+	    oPtr = sPtr;
 	}
 	if (T_Has == tokenList[opIndex].token
 		|| T_Is == tokenList[opIndex].token) {
-	    sLower = cpystr(s);
+	    sLower = cpystr(Tcl_GetString(oPtr));
 	    for (cPtr = sLower; *cPtr; cPtr++) {
-		if (isupper(*cPtr)) {
-		    *cPtr = tolower(*cPtr);
+		if (isupper((unsigned char)*cPtr)) {
+		    *cPtr = tolower((unsigned char)*cPtr);
 		}
 	    }
 	    val = Tcl_RegExpMatch(interp, sLower, expPtr->arg2.string);
-	    free(sLower);
+	    ckfree(sLower);
 	    return val;
 	} else if (expPtr->arg1.info == RAT_FOLDER_SIZE) {
+	    Tcl_GetIntFromObj(interp, oPtr, &val);
 	    if (T_Gt == tokenList[opIndex].token) {
-		return atoi(s) > atoi(expPtr->arg2.string);
+		return val > atoi(expPtr->arg2.string);
 	    } else {
-		return atoi(s) < atoi(expPtr->arg2.string);
+		return val < atoi(expPtr->arg2.string);
 	    }
 	}
     }
