@@ -3,8 +3,8 @@
  *
  *      Declarations of types used in the folder and messages system.
  *
- * TkRat software and its included text is Copyright 1996,1997,1998
- * by Martin Forssn
+ * TkRat software and its included text is Copyright 1996-2000 by
+ * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
  * COPYRIGHT, included with this distribution.
@@ -21,6 +21,8 @@
  */
 typedef enum {
     RAT_FOLDER_SUBJECT,		/* The subject of the message */
+    RAT_FOLDER_CANONSUBJECT,	/* The canonical subject of the message
+				   This is the subject without any leading Re:*/
     RAT_FOLDER_NAME,		/* The full name of the sender (From:) if
 				 * available otherwise the mail address */
     RAT_FOLDER_MAIL_REAL,	/* The mail address of the sender (From:) */
@@ -45,7 +47,11 @@ typedef enum {
     RAT_FOLDER_SENDER,		/* The Sender: header line */
     RAT_FOLDER_CC,		/* The CC: header line */
     RAT_FOLDER_REPLY_TO,	/* The Reply-To: header line */
-    RAT_FOLDER_FLAGS,		/* The flags list */
+    RAT_FOLDER_FLAGS,		/* The flags list in imap4 format */
+    RAT_FOLDER_UNIXFLAGS,	/* The flags list in unix format*/
+    RAT_FOLDER_MSGID,		/* The message ID */
+    RAT_FOLDER_REF,		/* The In-Reply-to header */
+    RAT_FOLDER_THREADING,	/* Threading information */
     RAT_FOLDER_END
 } RatFolderInfoType;
 
@@ -59,32 +65,53 @@ typedef enum {
     RAT_SEEN,			/* Message content has been seen by the user */
     RAT_DELETED,		/* Message is marked for deletion */
     RAT_FLAGGED,		/* Message is flagged */
-    RAT_ANSWERED		/* Message has been answered */
+    RAT_ANSWERED,		/* Message has been answered */
+    RAT_DRAFT,			/* Message is a draft */
+    RAT_RECENT			/* Message is a not seen but has been in the
+				   folder for some time */
 } RatFlag;
 #define RAT_SEEN_STR		"\\Seen"
 #define RAT_DELETED_STR		"\\Deleted"
 #define RAT_FLAGGED_STR		"\\Flagged"
 #define RAT_ANSWERED_STR	"\\Answered"
+#define RAT_DRAFT_STR		"\\Draft"
+
+/*
+ * The different types of updates which can be performed
+ */
+typedef enum {
+    RAT_UPDATE,		/* Only check for new mail */
+    RAT_CHECKPOINT,	/* Checkpoint flags etc */
+    RAT_SYNC		/* Do an expunge on the folder */
+} RatUpdateType;
+
+/*
+ * The different sort methods
+ */
+typedef enum {SORT_NONE, SORT_SUBJECT, SORT_SUBJDATE, SORT_THREADED,
+	      SORT_SENDER, SORT_SENDERDATE, SORT_DATE, SORT_SIZE} SortOrder;
 
 /*
  * Below follows typedefs which declares the types of the function pointers
  * that can be found in the RatFolderInfo structure. Each folder MUST
  * provide all of these functions. The functions in question are:
  *
- * int closeProc(RatFolderInfo *infoPtr, Tcl_Interp *interp)
+ * void initProc(RatFolderInfo *infoPtr, Tcl_Interp *interp, int index)
+ *
+ * 	This procedure should initialize the privatePtr part of
+ *	the folder structure.
+ *
+ * int closeProc(RatFolderInfo *infoPtr, Tcl_Interp *interp, int expunge)
  *
  *	This procedure should close the folder and free all private
  *	data structures. It should normally return TCL_OK, unless
  *	something goes wrong in which case it should return TCL_ERROR
  *	and leave an error message in interp->result.
  *
- * int updateProc(RatFolderInfo *infoPtr, Tcl_Interp *interp, int expunge)
+ * int updateProc(RatFolderInfo *infoPtr, Tcl_Interp *interp,RatUpdateType mode)
  *
- *	This procedure should check to see if any new messages has
- *	arrived in the folder. If the expunge argument is non null it
- *	should also expunge messages marked for deletion. It should
- *	return the number of messages in the mailbox or -1 if an error
- *	occurs.
+ *	This procedure should update the folder. It return -1 on errors,
+ *	otherwise the number of new messages is returned.
  *
  * int insertProc(RatFolderInfo *infoPtr, Tcl_Interp *interp, int argc,
  *		  char *argv[])
@@ -107,34 +134,46 @@ typedef enum {
  *
  *	Gets the value of the specified flag for message specified by 'index'.
  *
- * char *infoProc(Tcl_Interp *interp, ClientData clientData,
- *		  RatFolderInfoType type, int index)
+ * Tcl_Obj *infoProc(Tcl_Interp *interp, ClientData clientData,
+ *		     RatFolderInfoType type, int index)
  *
- *	Returns information about a message. The returned value is a pointer
- *	to a static area which may be overwritten by the next call.
+ *	Returns information about a message.
+ *
+ * void setInfoProc(Tcl_Interp *interp, ClientData clientData,
+ *		  RatFolderInfoType type, int index, Tcl_Obj *oPtr)
+ *
+ *	Sets information about a message.
  *
  * char *createProc(RatFolderInfo *infoPtr, Tcl_Interp *interp, int index)
  *
  *	This function should create a message command for the message
  *	specified by "index". The message command is returned.
+ *
+ * int syncProc(RatFolderInfoPtr infoPtr, Tcl_Interp *interp)
+ *	Does a network synchronization for the given folder
  */
 
 
 
-typedef int (RatCloseProc) (RatFolderInfoPtr infoPtr,
-	Tcl_Interp *interp);
+typedef void (RatInitProc) (RatFolderInfoPtr infoPtr,
+	Tcl_Interp *interp, int index);
+typedef int (RatCloseProc) (RatFolderInfoPtr infoPtr, Tcl_Interp *interp,
+	int expunge);
 typedef int (RatUpdateProc) (RatFolderInfoPtr infoPtr,
-	Tcl_Interp *interp, int expunge);
+	Tcl_Interp *interp, RatUpdateType mode);
 typedef int (RatInsertProc) (RatFolderInfoPtr infoPtr,
 	Tcl_Interp *interp, int argc, char *argv[]);
 typedef int (RatSetFlagProc) (RatFolderInfoPtr infoPtr,
 	Tcl_Interp *interp, int index, RatFlag flag, int value);
 typedef int (RatGetFlagProc) (RatFolderInfoPtr infoPtr,
 	Tcl_Interp *interp, int index, RatFlag flag);
-typedef char* (RatInfoProc) (Tcl_Interp *interp, ClientData clientData,
+typedef Tcl_Obj* (RatInfoProc) (Tcl_Interp *interp, ClientData clientData,
 	RatFolderInfoType type, int index);
+typedef void (RatSetInfoProc) (Tcl_Interp *interp, ClientData clientData,
+	RatFolderInfoType type, int index,Tcl_Obj *oPtr);
 typedef char* (RatCreateProc) (RatFolderInfoPtr infoPtr,
 	Tcl_Interp *interp, int index);
+typedef int (RatSyncProc) (RatFolderInfoPtr infoPtr, Tcl_Interp *interp);
 
 /*
  * An instance of the structure below is created for each folder. It is then
@@ -145,7 +184,14 @@ typedef struct RatFolderInfo {
     char *name;			/* Name of folder (this is a pointer which
 				 * may be ckfree():ed) */
     char *type;			/* Type of folder */
+    char *definition;		/* Definition id of folder */
+    int refCount;		/* Reference count (<=0 means closing) */
+    SortOrder sortOrder;	/* Sort order for folder */
+    int sortOrderChanged;	/* Non null if sort order has changed */
+    int reverse;		/* If the sort order should be reversed */
     int number;			/* Number of messages in folder */
+    int recent;			/* Number of recent messages in folder */
+    int unseen;			/* Number of unseen messages in folder */
     int visible;		/* The number of messages that the user sees */
     int size;			/* Approximate size of folder, or -1 if the
 				 * folder type doesn't support size. */
@@ -161,18 +207,31 @@ typedef struct RatFolderInfo {
 				 * of this list is the index of the first
 				 * message to show etc. */
     int *hidden;		/* A true value here means that we should
-				 * pretednd that the corresponding message
+				 * pretend that the corresponding message
 				 * does not exists. */
+    int flagsChanged;		/* Non null if the flags has been changed
+				 * since the last checkpoint */
+    int watcherInterval;	/* Interval between updates */
+    Tcl_TimerToken timerToken;	/* Token for update timer */
+    RatInitProc *initProc;
     RatCloseProc *closeProc;
     RatUpdateProc *updateProc;
     RatInsertProc *insertProc;
     RatSetFlagProc *setFlagProc;
     RatGetFlagProc *getFlagProc;
     RatInfoProc *infoProc;
+    RatSetInfoProc *setInfoProc;
     RatCreateProc *createProc;
-    ClientData private;		/* Data private for each folder type */
+    RatSyncProc *syncProc;
+    ClientData private, private2;  /* Data private for each folder type */
+    struct RatFolderInfo *nextPtr; /* Pointer to next folder (if any)*/
 } RatFolderInfo;
 
+/*
+ * Global list of folders
+ */
+extern RatFolderInfo *ratFolderList;
+
 /*
  * The different types of messages. These are for internal use only.
  */
@@ -193,13 +252,14 @@ typedef enum { RAT_ISME_YES,
 typedef struct BodyInfo BodyInfo;
 typedef struct MessageInfo {
     RatFolderInfo *folderInfoPtr;
-    char *name;
+    char name[16];
     RatMessageType type;
     int msgNo;
     RatIsMeStatus fromMe;
     RatIsMeStatus toMe;
     BodyInfo *bodyInfoPtr;
     ClientData clientData;
+    Tcl_Obj *info[RAT_FOLDER_END];
 } MessageInfo;
 
 /*
@@ -239,11 +299,12 @@ typedef char* (RatGetEnvelopeProc) (Tcl_Interp *interp, MessageInfo *msgPtr);
 typedef BodyInfo* (RatCreateBodyProc) (Tcl_Interp *interp, MessageInfo *msgPtr);
 typedef char* (RatFetchTextProc) (Tcl_Interp *interp, MessageInfo *msgPtr);
 typedef ENVELOPE* (RatEnvelopeProc) (MessageInfo *msgPtr);
-typedef void (RatSetIndexProc) (MessageInfo *msgPtr, int index);
 typedef void (RatMsgDeleteProc) (MessageInfo *msgPtr);
 typedef void (RatMakeChildrenProc) (Tcl_Interp *interp, BodyInfo *bodyPtr);
 typedef char* (RatFetchBodyProc) (BodyInfo *bodyPtr, unsigned long *lengthPtr);
 typedef void (RatBodyDeleteProc) (BodyInfo *bodyPtr);
+typedef MESSAGECACHE* (RatGetInternalDateProc) (Tcl_Interp *interp,
+	MessageInfo *msgPtr);
 
 /*
  * The following structure defines which functions to call for to
@@ -256,11 +317,11 @@ typedef struct {
     RatCreateBodyProc *createBodyProc;
     RatFetchTextProc *fetchTextProc;
     RatEnvelopeProc *envelopeProc;
-    RatSetIndexProc *setIndexProc;
     RatMsgDeleteProc *msgDeleteProc;
     RatMakeChildrenProc *makeChildrenProc;
     RatFetchBodyProc *fetchBodyProc;
     RatBodyDeleteProc *bodyDeleteProc;
+    RatGetInternalDateProc *getInternalDateProc;
 } MessageProcInfo;
 
 /*
@@ -271,54 +332,82 @@ typedef struct {
     char **preString;		/* Any characters that should be inserted
 				   befor the next data part. */
     RatFolderInfoType *typeList;/* The type of the data */
-    int *fieldWidth;		/* How wide this field should be (0=variable) */
+    int *fieldWidth;		/* How wide this field should be (0=variable)*/
     int *leftJust;		/* True if it should be left justified */
     char *postString;		/* Any character sthat should be appended. */
 } ListExpression;
 
-/* ratFolder.h (not that this file also exports finctions in rat.h) */
-extern char *RatGetMsgInfo(Tcl_Interp *interp, RatFolderInfoType type,
+/* ratFolder.c (note that this file also exports functions in rat.h) */
+extern RatFolderInfo *RatGetOpenFolder(char *def);
+extern Tcl_Obj *RatFolderCanonalizeSubject (const char *s);
+extern Tcl_Obj *RatGetMsgInfo(Tcl_Interp *interp, RatFolderInfoType type,
 	MessageInfo *msgPtr, ENVELOPE *envPtr, BODY *bodyPtr,
 	MESSAGECACHE *eltPtr, int size);
+extern char* MsgFlags(MESSAGECACHE *eltPtr);
+extern MESSAGECACHE *RatParseFrom(const char *from);
+extern int RatFolderClose(Tcl_Interp *interp, RatFolderInfo *infoPtr,
+	int force);
+extern int RatFolderInsert(Tcl_Interp *interp, RatFolderInfo *infoPtr,
+	int num, char **msgs);
 
 /* ratStdFolder.c */
 extern int RatStdFolderInit (Tcl_Interp *interp);
-extern RatFolderInfo *RatStdFolderCreate (Tcl_Interp *interp, int argc,
-	char *argv[]);
+extern RatFolderInfo *RatStdFolderCreate (Tcl_Interp *interp, int objc,
+	Tcl_Obj *CONST objv[]);
 extern MAILSTREAM* OpenStdFolder(Tcl_Interp *interp, char *name, char *prot,
 	char *user, void *stdPtr);
 extern void CloseStdFolder(Tcl_Interp *interp, MAILSTREAM *stream);
 
 /* ratDbFolder.c */
 extern int RatDbFolderInit (Tcl_Interp *interp);
-extern RatFolderInfo *RatDbFolderCreate (Tcl_Interp *interp, int argc,
-	char *argv[]);
+extern RatFolderInfo *RatDbFolderCreate (Tcl_Interp *interp, int objc,
+	Tcl_Obj *CONST objv[]);
 extern RatInfoProc Db_InfoProc;
+extern Tcl_Obj* Db_InfoProcInt(Tcl_Interp *interp, RatFolderInfo *infoPtr,
+	RatFolderInfoType type, int rIndex);
+
+/* ratDisFolder.c */
+extern int RatDisFolderInit (Tcl_Interp *interp);
+extern MAILSTREAM* RatDisFolderOpenStream (Tcl_Interp *interp, int objc,
+	Tcl_Obj *CONST objv[], char **mailbox);
+extern RatFolderInfo *RatDisFolderCreate (Tcl_Interp *interp, int objc,
+	Tcl_Obj *CONST objv[]);
+extern char* RatDisFolderDir(Tcl_Interp *interp, const char *spec,
+	const char *user, const char *prot);
+extern const char* RatDisPrepareDir(Tcl_Interp *interp, const char *name,
+	const char *folder_spec, const char *user, const char *prot);
 
 /* ratMessage.c */
 extern void RatInitMessages (void);
-extern Tcl_CmdProc RatMessageCmd;
+extern Tcl_ObjCmdProc RatMessageCmd;
 extern BodyInfo *CreateBodyInfo(MessageInfo *msgPtr);
 extern Tcl_CmdProc RatBodyCmd;
-extern void RatMessageSetIndex (Tcl_Interp *interp, char *cmdName, int index);
 extern int RatMessageDelete (Tcl_Interp *interp, char *msgCmd);
-extern void RatMessageSave (Tcl_Interp *interp,Tcl_Channel channel,
-	MessageInfo *msgPtr);
 extern void RatMessageGet (Tcl_Interp *interp, MessageInfo *msgPtr,
-	Tcl_DString *ds, char *flags, char *date);
+	Tcl_DString *ds, char *flags, size_t flaglen, char *date,
+	size_t datelen);
 extern Tcl_CmdProc RatInsertCmd;
 extern int RatInsertMsg (Tcl_Interp *interp, MessageInfo *msgPtr,
 	char *keywords, char *exDate, char *exType);
 extern ListExpression *RatParseList(char *format);
 extern void RatFreeListExpression(ListExpression *exprPtr);
-extern char *RatDoList(Tcl_Interp *interp, ListExpression *exprPtr,
+extern Tcl_Obj *RatDoList(Tcl_Interp *interp, ListExpression *exprPtr,
 	RatInfoProc *infoProc, ClientData clientData, int index);
+extern Tcl_Obj *RatMsgInfo(Tcl_Interp *interp, MessageInfo *msgPtr,
+	RatFolderInfoType type);
 extern int RatBodySave(Tcl_Interp *interp,Tcl_Channel channel,
 		BodyInfo *bodyInfoPtr, int encoded, int convertNL);
+extern Tcl_Obj *RatBodyType(BodyInfo *bodyInfoPtr);
+extern Tcl_Obj *RatBodyData(Tcl_Interp *interp, BodyInfo *bodyInfoPtr,
+	int encoded, char *charset);
+extern MESSAGECACHE *RatMessageInternalDate(Tcl_Interp *interp,
+	MessageInfo *msgPtr);
+extern void RatPurgeFlags(char *flags);
+
 
 /* ratDbMessage.c */
 extern char *RatDbMessageCreate (Tcl_Interp *interp, RatFolderInfoPtr infoPtr,
-	int index);
+	int index, int dbIndex);
 extern void RatDbMessagesInit (MessageProcInfo* procInfo);
 
 /* ratFrMessage.c */
@@ -344,4 +433,12 @@ extern int RatExpMatch(Tcl_Interp *interp, int expId,
 /* ratMailcap.c */
 int RatMcapFindCmd(Tcl_Interp *interp, BodyInfo *bodyInfoPtr);
 
+#ifdef MEM_DEBUG
+void ratStdMessageCleanup(void);
+void ratMessageCleanup(void);
+void ratStdFolderCleanup(void);
+void ratAddressCleanup(void);
+void ratCodeCleanup(void);
+#endif /* MEM_DEBUG */
+
 #endif /* _RATFOLDER */
