@@ -3,8 +3,8 @@
  *
  *	This file contains code which implements free messages.
  *
- * TkRat software and its included text is Copyright 1996,1997,1998
- * by Martin Forssn
+ * TkRat software and its included text is Copyright 1996-2000 by
+ * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
  * COPYRIGHT, included with this distribution.
@@ -17,7 +17,9 @@
  */
 typedef struct FrMessageInfo {
     MESSAGE *messagePtr;
+    char *from;
     char *headers;
+    char *msgData;
     char *bodyData;
 } FrMessageInfo;
 
@@ -25,8 +27,7 @@ typedef struct FrMessageInfo {
  * The ClientData for each bodypart entity
  */
 typedef struct FrBodyInfo {
-    char *mark;
-    char *text;
+    unsigned char *text;
 } FrBodyInfo;
 
 /*
@@ -39,12 +40,12 @@ static RatGetHeadersProc Fr_GetHeadersProc;
 static RatGetEnvelopeProc Fr_GetEnvelopeProc;
 static RatFetchTextProc Fr_FetchTextProc;
 static RatEnvelopeProc Fr_EnvelopeProc;
-static RatSetIndexProc Fr_SetIndexProc;
 static RatMsgDeleteProc Fr_MsgDeleteProc;
 static RatMakeChildrenProc Fr_MakeChildrenProc;
 static RatFetchBodyProc Fr_FetchBodyProc;
 static RatBodyDeleteProc Fr_BodyDeleteProc;
 static RatInfoProc Fr_GetInfoProc;
+static RatGetInternalDateProc Fr_GetInternalDateProc;
 
 
 /*
@@ -73,11 +74,11 @@ RatFrMessagesInit(MessageProcInfo *messageProcInfoPtr)
     messageProcInfoPtr->createBodyProc = Fr_CreateBodyProc;
     messageProcInfoPtr->fetchTextProc = Fr_FetchTextProc;
     messageProcInfoPtr->envelopeProc = Fr_EnvelopeProc;
-    messageProcInfoPtr->setIndexProc = Fr_SetIndexProc;
     messageProcInfoPtr->msgDeleteProc = Fr_MsgDeleteProc;
     messageProcInfoPtr->makeChildrenProc = Fr_MakeChildrenProc;
     messageProcInfoPtr->fetchBodyProc = Fr_FetchBodyProc;
     messageProcInfoPtr->bodyDeleteProc = Fr_BodyDeleteProc;
+    messageProcInfoPtr->getInternalDateProc = Fr_GetInternalDateProc;
 }
 
 
@@ -102,11 +103,10 @@ char*
 RatFrMessageCreate(Tcl_Interp *interp, char *data, int length,
 		   MessageInfo **msgPtrPtr)
 {
-    FrMessageInfo *msgFrPtr=(FrMessageInfo*)ckalloc(sizeof(FrMessageInfo));
+    FrMessageInfo *frMsgPtr=(FrMessageInfo*)ckalloc(sizeof(FrMessageInfo));
     MessageInfo *msgPtr=(MessageInfo*)ckalloc(sizeof(MessageInfo));
-    char *name = (char*)malloc(16);
-    char *bodyData;
-    int headerLength;
+    char *msgData, *cPtr;
+    int headerLength, j, fromLength;
 
     for (headerLength = 0; data[headerLength]; headerLength++) {
 	if (data[headerLength] == '\n' && data[headerLength+1] == '\n') {
@@ -120,31 +120,42 @@ RatFrMessageCreate(Tcl_Interp *interp, char *data, int length,
 	}
     }
 
-    bodyData = (char*)malloc(length+1);
-    memcpy(bodyData, data, length);
-    bodyData[length] = '\0';
+    msgData = (char*)ckalloc(length+1);
+    memcpy(msgData, data, length);
+    msgData[length] = '\0';
 
     msgPtr->folderInfoPtr = NULL;
     msgPtr->type = RAT_FREE_MESSAGE;
     msgPtr->bodyInfoPtr = NULL;
-    msgPtr->name = name;
     msgPtr->msgNo = 0;
     msgPtr->fromMe = RAT_ISME_UNKOWN;
     msgPtr->toMe = RAT_ISME_UNKOWN;
-    msgPtr->clientData = (ClientData)msgFrPtr;
-    msgFrPtr->bodyData = bodyData;
-    msgFrPtr->messagePtr = RatParseMsg(interp, bodyData);
-    msgFrPtr->headers = (char*)ckalloc(headerLength+1);
-    strncpy(msgFrPtr->headers, data, headerLength);
-    msgFrPtr->headers[headerLength] = '\0';
+    msgPtr->clientData = (ClientData)frMsgPtr;
+    for (j=0; j<sizeof(msgPtr->info)/sizeof(*msgPtr->info); j++) {
+	msgPtr->info[j] = NULL;
+    }
+    frMsgPtr->msgData = msgData;
+    frMsgPtr->messagePtr = RatParseMsg(interp, msgData);
+    frMsgPtr->bodyData = frMsgPtr->messagePtr->text.text.data +
+			 frMsgPtr->messagePtr->text.offset;
+    frMsgPtr->headers = (char*)ckalloc(headerLength+1);
+    RatStrNCpy(frMsgPtr->headers, data, headerLength+1);
+    if (!strncmp("From ", data, 5) && (cPtr = strchr(data, '\n'))) {
+	fromLength = cPtr-data;
+	frMsgPtr->from = (char*)ckalloc(fromLength+1);
+	RatStrNCpy(frMsgPtr->from, frMsgPtr->headers, fromLength);
+    } else {
+	frMsgPtr->from = NULL;
+    }
 
     if (msgPtrPtr) {
 	*msgPtrPtr = msgPtr;
     }
 
-    sprintf(name, "RatFrMsg%d", numFrMessages++);
-    Tcl_CreateCommand(interp, name, RatMessageCmd, (ClientData) msgPtr, NULL);
-    return name;
+    sprintf(msgPtr->name, "RatFrMsg%d", numFrMessages++);
+    Tcl_CreateObjCommand(interp, msgPtr->name, RatMessageCmd,
+	    (ClientData)msgPtr, NULL);
+    return msgPtr->name;
 }
 
 
@@ -194,10 +205,14 @@ Fr_GetEnvelopeProc(Tcl_Interp *interp, MessageInfo *msgPtr)
 	adrPtr = frMsgPtr->messagePtr->env->from;
     }
     if (!strcmp(currentHost, adrPtr->host)) {
-	sprintf(buf, "From %s", adrPtr->mailbox);
+	snprintf(buf, sizeof(buf), "From %s", adrPtr->mailbox);
     } else {
 	sprintf(buf, "From ");
-	rfc822_address(buf+5, adrPtr);
+	if (RatAddressSize(adrPtr, 0) > sizeof(buf)-32) {
+	    sprintf(buf+5, "ridiculously@long.address");
+	} else {
+	    rfc822_address(buf+5, adrPtr);
+	}
     }
     if (T == mail_parse_date(&elt, frMsgPtr->messagePtr->env->date)) {
 	tm.tm_sec = elt.seconds;
@@ -214,9 +229,10 @@ Fr_GetEnvelopeProc(Tcl_Interp *interp, MessageInfo *msgPtr)
 	date = 0;
     }
     tmPtr = gmtime(&date);
-    sprintf(buf + strlen(buf), " %s %s %2d %02d:%02d GMT 19%02d\n",
-	    dayName[tmPtr->tm_wday], monthName[tmPtr->tm_mon],
-	    tmPtr->tm_mday, tmPtr->tm_hour, tmPtr->tm_min, tmPtr->tm_year);
+    snprintf(buf + strlen(buf), sizeof(buf)-strlen(buf),
+	    " %s %s %2d %02d:%02d GMT %04d\n", dayName[tmPtr->tm_wday],
+	    monthName[tmPtr->tm_mon], tmPtr->tm_mday, tmPtr->tm_hour,
+	    tmPtr->tm_min, tmPtr->tm_year+1900);
     return buf;
 }
 
@@ -235,14 +251,12 @@ BodyInfo*
 Fr_CreateBodyProc(Tcl_Interp *interp, MessageInfo *msgPtr)
 {
     FrMessageInfo *frMsgPtr = (FrMessageInfo*)msgPtr->clientData;
-    FrBodyInfo *frBodyInfoPtr = (FrBodyInfo*)malloc(sizeof(FrBodyInfo));
+    FrBodyInfo *frBodyInfoPtr = (FrBodyInfo*)ckalloc(sizeof(FrBodyInfo));
     msgPtr->bodyInfoPtr = CreateBodyInfo(msgPtr);
 
     msgPtr->bodyInfoPtr->bodyPtr = frMsgPtr->messagePtr->body;
     msgPtr->bodyInfoPtr->clientData = (ClientData)frBodyInfoPtr;
-    frBodyInfoPtr->text = frMsgPtr->messagePtr->text.text.data +
-			  frMsgPtr->messagePtr->text.offset;
-    frBodyInfoPtr->mark = frBodyInfoPtr->text;
+    frBodyInfoPtr->text = frMsgPtr->bodyData;
     return msgPtr->bodyInfoPtr;
 }
 
@@ -261,8 +275,7 @@ static char*
 Fr_FetchTextProc(Tcl_Interp *interp, MessageInfo *msgPtr)
 {
     FrMessageInfo *frMsgPtr = (FrMessageInfo*)msgPtr->clientData;
-    return frMsgPtr->messagePtr->text.text.data +
-	   frMsgPtr->messagePtr->text.offset;
+    return (char*)frMsgPtr->bodyData;
 }
 
 
@@ -281,24 +294,6 @@ Fr_EnvelopeProc(MessageInfo *msgPtr)
 {
     return ((FrMessageInfo*)msgPtr->clientData)->messagePtr->env;
 }
-
-
-/*
- *----------------------------------------------------------------------
- *
- * Fr_SetIndexProc --
- *
- *      See ratFolder.h
- *
- *----------------------------------------------------------------------
- */
-
-static void
-Fr_SetIndexProc(MessageInfo *msgPtr, int index)
-{
-     return;
-}
-
 
 /*
  *----------------------------------------------------------------------
@@ -318,7 +313,7 @@ Fr_MsgDeleteProc(MessageInfo *msgPtr)
     mail_free_body(&frMsgPtr->messagePtr->body);
     ckfree(frMsgPtr->messagePtr);
     ckfree(frMsgPtr->headers);
-    ckfree(frMsgPtr->bodyData);
+    ckfree(frMsgPtr->msgData);
     ckfree(frMsgPtr);
 }
 
@@ -336,7 +331,7 @@ Fr_MsgDeleteProc(MessageInfo *msgPtr)
 static void
 Fr_MakeChildrenProc(Tcl_Interp *interp, BodyInfo *bodyInfoPtr)
 {
-    FrBodyInfo *frBodyInfoPtr = (FrBodyInfo*)bodyInfoPtr->clientData;
+    FrMessageInfo *frMsgPtr = (FrMessageInfo*)bodyInfoPtr->msgPtr->clientData;
     BODY *bodyPtr = bodyInfoPtr->bodyPtr;
     BodyInfo *partInfoPtr, **partInfoPtrPtr;
     FrBodyInfo *frPartInfoPtr;
@@ -352,13 +347,8 @@ Fr_MakeChildrenProc(Tcl_Interp *interp, BodyInfo *bodyInfoPtr)
 	    partInfoPtr->bodyPtr = &partPtr->body;
 	    partInfoPtrPtr = &partInfoPtr->nextPtr;
 	    partInfoPtr->clientData = (ClientData)frPartInfoPtr;
-	    frPartInfoPtr->text = frBodyInfoPtr->mark +
+	    frPartInfoPtr->text = frMsgPtr->bodyData +
 				  partPtr->body.contents.offset;
-	    if (TYPEMULTIPART == partPtr->body.type) {
-		frPartInfoPtr->mark = frBodyInfoPtr->text;
-	    } else {
-		frPartInfoPtr->mark = frPartInfoPtr->text;
-	    }
 	}
     }
 }
@@ -384,7 +374,7 @@ Fr_FetchBodyProc(BodyInfo *bodyInfoPtr, unsigned long *lengthPtr)
 	return Tcl_DStringValue(bodyInfoPtr->decodedTextPtr);
     }
     *lengthPtr = bodyInfoPtr->bodyPtr->contents.text.size;
-    return frBodyInfoPtr->text;
+    return (char*)frBodyInfoPtr->text;
 }
 
 
@@ -405,6 +395,32 @@ Fr_BodyDeleteProc(BodyInfo *bodyInfoPtr)
     ckfree(frBodyInfoPtr);
 }
 
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Fr_GetInternalDateProc --
+ *
+ *      See ratFolder.h
+ *
+ *----------------------------------------------------------------------
+ */
+
+static MESSAGECACHE*
+Fr_GetInternalDateProc(Tcl_Interp *interp, MessageInfo *msgPtr)
+{
+    FrMessageInfo *frMsgPtr = (FrMessageInfo*)msgPtr->clientData;
+    static MESSAGECACHE elt;
+
+    if (frMsgPtr->from) {
+	return RatParseFrom(frMsgPtr->from);
+    } else {
+	mail_parse_date(&elt, frMsgPtr->messagePtr->env->date);
+	return &elt;
+    }
+}
+
 
 /*
  *----------------------------------------------------------------------
@@ -416,7 +432,7 @@ Fr_BodyDeleteProc(BodyInfo *bodyInfoPtr)
  *----------------------------------------------------------------------
  */
 
-static char*
+static Tcl_Obj*
 Fr_GetInfoProc(Tcl_Interp *interp, ClientData clientData,
 	RatFolderInfoType type, int index)
 {
@@ -424,11 +440,17 @@ Fr_GetInfoProc(Tcl_Interp *interp, ClientData clientData,
     MessageInfo *msgPtr = (MessageInfo*)clientData;
     FrMessageInfo *frMsgPtr = (FrMessageInfo*)msgPtr->clientData;
     MESSAGECACHE elt;
+    Tcl_Obj *oPtr = NULL;
     char *cPtr;
     int i;
 
+    if (msgPtr->info[type]) {
+	return msgPtr->info[type];
+    }
+
     switch (type) {
 	case RAT_FOLDER_SUBJECT:	/* fallthrough */
+	case RAT_FOLDER_CANONSUBJECT:	/* fallthrough */
 	case RAT_FOLDER_NAME:		/* fallthrough */
 	case RAT_FOLDER_MAIL_REAL:	/* fallthrough */
 	case RAT_FOLDER_MAIL:		/* fallthrough */
@@ -440,6 +462,8 @@ Fr_GetInfoProc(Tcl_Interp *interp, ClientData clientData,
 	case RAT_FOLDER_SENDER:		/* fallthrough */
 	case RAT_FOLDER_CC:		/* fallthrough */
 	case RAT_FOLDER_REPLY_TO:	/* fallthrough */
+	case RAT_FOLDER_MSGID:		/* fallthrough */
+	case RAT_FOLDER_REF:		/* fallthrough */
 	case RAT_FOLDER_PARAMETERS:
 	    return RatGetMsgInfo(interp, type, msgPtr,frMsgPtr->messagePtr->env,
 		    frMsgPtr->messagePtr->body, NULL, 0);
@@ -503,31 +527,58 @@ Fr_GetInfoProc(Tcl_Interp *interp, ClientData clientData,
 			buf[i++] = ' ';
 		    }
 		    buf[i] = '\0';
-		    return buf;
+		    oPtr = Tcl_NewStringObj(buf, -1);
+		    break;
 		}
 	    } while ((cPtr = strchr(cPtr, '\n')) && cPtr++ && *cPtr);
-	    return "";
+	    break;
 	case RAT_FOLDER_FLAGS:
 	    cPtr = frMsgPtr->headers;
 	    buf[0] = '\0';
 	    do {
 		if (!strncasecmp(cPtr, "status:", 7)) {
-		    for (i=7; cPtr[i]; i++) {
+		    for (i=7; cPtr[i] != '\n' && cPtr[i]; i++) {
 			switch (cPtr[i]) {
 			case 'R': strcat(buf, " " RAT_SEEN_STR); break;
 			case 'D': strcat(buf, " " RAT_DELETED_STR); break;
 			case 'F': strcat(buf, " " RAT_FLAGGED_STR); break;
 			case 'A': strcat(buf, " " RAT_ANSWERED_STR); break;
+			case 'T': strcat(buf, " " RAT_DRAFT_STR); break;
 			}
 		    }
-		    return buf;
+		    if (*buf) {
+			oPtr = Tcl_NewStringObj(buf+1, -1);
+		    } else {
+			oPtr = Tcl_NewObj();
+		    }
+		    break;
 		}
 	    } while ((cPtr = strchr(cPtr, '\n')) && cPtr++ && *cPtr);
-	    return "";
+	    break;
+	case RAT_FOLDER_UNIXFLAGS:
+	    cPtr = frMsgPtr->headers;
+	    buf[0] = '\0';
+	    do {
+		if (!strncasecmp(cPtr, "status:", 7)) {
+		    for (cPtr += 7; isspace(*cPtr); cPtr++);
+		    oPtr = Tcl_NewStringObj(cPtr, -1);
+		    break;
+		}
+	    } while ((cPtr = strchr(cPtr, '\n')) && cPtr++ && *cPtr);
+	    break;
 	case RAT_FOLDER_INDEX:
-	    return "1";
+	    oPtr = Tcl_NewIntObj(1);
+	    break;
+	case RAT_FOLDER_THREADING:
+	    oPtr = Tcl_NewObj();
+	    break;
 	case RAT_FOLDER_END:
 	    break;
     }
-    return NULL;
+    if (!oPtr) {
+	oPtr = Tcl_NewObj();
+    }
+    msgPtr->info[type] = oPtr;
+    Tcl_IncrRefCount(oPtr);
+    return oPtr;
 }
