@@ -4,8 +4,8 @@
  *	Manages different holds of messages.
  *
  *
- * TkRat software and its included text is Copyright 1996,1997,1998
- * by Martin Forssn
+ * TkRat software and its included text is Copyright 1996-2000 by
+ * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
  * COPYRIGHT, included with this distribution.
@@ -13,7 +13,6 @@
 
 #include <pwd.h>
 #include <signal.h>
-#include <tk.h>
 #include "rat.h"
 #include <locale.h>
 
@@ -25,8 +24,13 @@ static char *holdMessageParts[] = {"remail", "return_path", "date", "from",
 	"reply_to", "subject", "to", "cc", "bcc", "in_reply_to", "message_id",
 	"save_to", "request_dsn", (char *) NULL};
 static char *holdBodyParts[] = {"type", "subtype", "encoding", "parameter",
-	"id", "description", "filename", "removeFile", "pgp_sign",
-	"pgp_encrypt", (char *) NULL};
+	"id", "description", "charset", "filename", "removeFile", "pgp_sign",
+	"pgp_encrypt", "disp_type", "disp_parm", (char *) NULL};
+
+/*
+ * Keeps track of number of held and deferred messages
+ */
+static int numHeld, numDeferred;
 
 /*
  * Save bodyparts during hold
@@ -115,7 +119,7 @@ RatHold(ClientData dummy, Tcl_Interp *interp, int argc, char *argv[])
 	    ret = TCL_ERROR;
 	} else {
 	    eId = RatLindex(interp, Tcl_DStringValue(&fileList), index);
-	    sprintf(buf, "%s/%s", holdDir, eId);
+	    snprintf(buf, sizeof(buf), "%s/%s", holdDir, eId);
 	    ret = RatHoldExtract(interp, buf, NULL, NULL);
 	}
 
@@ -182,7 +186,7 @@ RatHoldBody(Tcl_Interp *interp, FILE *fPtr, char *baseName, char *handler,
 		TCL_GLOBAL_ONLY))) {
 	    if ((newSize = Tcl_ScanElement(value, &flags)) > *listValueSize) {
 		*listValueSize = newSize+1;
-		*listValue = (char*) REALLOC(*listValue, *listValueSize);
+		*listValue = (char*) ckrealloc(*listValue, *listValueSize);
 	    }
 	    Tcl_ConvertElement(value, *listValue, flags);
 	    fprintf(fPtr, "set hold${holdId}(%s) %s\n", holdBodyParts[i],
@@ -226,7 +230,7 @@ RatHoldBody(Tcl_Interp *interp, FILE *fPtr, char *baseName, char *handler,
  *	will be created if it doesn't exist.
  *
  * Results:
- *      A standard Tcl result. interp->result will contain the name of the
+ *      A standard Tcl result. the result area will contain the name of the
  *	newly inserted entry (if everything went ok).
  *
  * Side effects:
@@ -248,15 +252,19 @@ RatHoldInsert(Tcl_Interp *interp, char *dir, char *handler, char *description)
 
     i = 0;
     do {
-	sprintf(baseName, "%s/%s:%x_%xM",
+	snprintf(baseName, sizeof(baseName), "%s/%s_%x_%xM",
 		dir, currentHost, (unsigned int)getpid(), i++);
     } while(!stat(baseName, &sbuf));
 
     /*
      * Write description file
      */
-    sprintf(buf, "%s.desc", baseName);
-    fPtr = fopen(buf, "w");
+    snprintf(buf, sizeof(buf), "%s.desc", baseName);
+    if (NULL == (fPtr = fopen(buf, "w"))) {
+	Tcl_AppendResult(interp, "error creating file \"", buf,
+		"\": ", Tcl_PosixError(interp), (char *) NULL);
+	return TCL_ERROR;
+    }
     (void)fprintf(fPtr, "%s\n", description);
     (void)fclose(fPtr);
 
@@ -274,7 +282,7 @@ RatHoldInsert(Tcl_Interp *interp, char *dir, char *handler, char *description)
 		TCL_GLOBAL_ONLY))) {
 	    if ((newSize = Tcl_ScanElement(value, &flags)) > listValueSize){
 		listValueSize = newSize+1;
-		listValue = (char*) REALLOC(listValue, listValueSize);
+		listValue = (char*) ckrealloc(listValue, listValueSize);
 	    }
 	    Tcl_ConvertElement(value, listValue, flags);
 	    fprintf(fPtr, "set hold${holdId}(%s) %s\n", holdMessageParts[i],
@@ -282,14 +290,15 @@ RatHoldInsert(Tcl_Interp *interp, char *dir, char *handler, char *description)
 	}
     }
 
-    sprintf(buf, "%s tag ranges noWrap", Tcl_GetVar2(interp, handler,
-	    "composeBody", TCL_GLOBAL_ONLY));
+    snprintf(buf, sizeof(buf), "%s tag ranges noWrap",
+	    Tcl_GetVar2(interp, handler, "composeBody", TCL_GLOBAL_ONLY));
     Tcl_Eval(interp, buf);
-    if ((newSize = Tcl_ScanElement(interp->result,&flags)) > listValueSize){
+    if ((newSize = Tcl_ScanElement(Tcl_GetStringResult(interp),
+	    &flags)) > listValueSize){
 	listValueSize = newSize+1;
-	listValue = (char*) REALLOC(listValue, listValueSize);
+	listValue = (char*) ckrealloc(listValue, listValueSize);
     }
-    Tcl_ConvertElement(interp->result, listValue, flags);
+    Tcl_ConvertElement(Tcl_GetStringResult(interp), listValue, flags);
     fprintf(fPtr, "set hold${holdId}(tag_range) %s\n", listValue);
 
     if ((value = Tcl_GetVar2(interp, handler, "body", TCL_GLOBAL_ONLY))) {
@@ -309,7 +318,7 @@ RatHoldInsert(Tcl_Interp *interp, char *dir, char *handler, char *description)
 	dirPtr = opendir(dir);
 	while (0 != (direntPtr = readdir(dirPtr))) {
 	    if (!strncmp(direntPtr->d_name, cPtr, strlen(cPtr))) {
-		sprintf(buf, "%s/%s", baseName, direntPtr->d_name);
+		snprintf(buf, sizeof(buf),"%s/%s", baseName, direntPtr->d_name);
 		(void)unlink(buf);
 	    }
 	}
@@ -320,6 +329,7 @@ RatHoldInsert(Tcl_Interp *interp, char *dir, char *handler, char *description)
 	return TCL_ERROR;
     }
     Tcl_SetResult(interp, baseName, TCL_VOLATILE);
+    RatHoldUpdateVars(interp, dir, 1);
     return TCL_OK;
 }
 
@@ -347,10 +357,12 @@ RatHoldList(Tcl_Interp *interp, char *dir, Tcl_DString *fileListPtr)
     char buf[1024];
     DIR *dirPtr;
     FILE *fPtr;
+    Tcl_Obj *oPtr = Tcl_NewObj();
     int l;
 
     if (NULL == (dirPtr = opendir(dir))) {
-	sprintf(buf, "Failed to open %s: %s", dir, Tcl_PosixError(interp));
+	snprintf(buf, sizeof(buf), "Failed to open %s: %s",
+		dir, Tcl_PosixError(interp));
 	Tcl_SetResult(interp, buf, TCL_VOLATILE);
 	return TCL_ERROR;
     }
@@ -360,18 +372,21 @@ RatHoldList(Tcl_Interp *interp, char *dir, Tcl_DString *fileListPtr)
 	    && 'e' == direntPtr->d_name[l-3]
 	    && 's' == direntPtr->d_name[l-2]
 	    && 'c' == direntPtr->d_name[l-1]) {
-	    sprintf(buf, "%s/%s", dir, direntPtr->d_name);
+	    snprintf(buf, sizeof(buf), "%s/%s", dir, direntPtr->d_name);
 	    fPtr = fopen(buf, "r");
 	    fgets(buf, sizeof(buf), fPtr);
 	    fclose(fPtr);
 	    buf[strlen(buf)-1] = '\0';
-	    Tcl_AppendElement(interp, buf);
-	    sprintf(buf, direntPtr->d_name);
-	    buf[strlen(buf)-5] = '\0';
-	    Tcl_DStringAppendElement(fileListPtr, buf);
+	    Tcl_ListObjAppendElement(interp, oPtr, Tcl_NewStringObj(buf, -1));
+	    snprintf(buf, sizeof(buf), direntPtr->d_name);
+	    if (fileListPtr) {
+		buf[strlen(buf)-5] = '\0';
+		Tcl_DStringAppendElement(fileListPtr, buf);
+	    }
 	}
     }
     closedir(dirPtr);
+    Tcl_SetObjResult(interp, oPtr);
     return TCL_OK;
 }
 
@@ -384,7 +399,7 @@ RatHoldList(Tcl_Interp *interp, char *dir, Tcl_DString *fileListPtr)
  *
  * Results:
  *      A standard Tcl result. The handler of the new message will be left
- *	in interp->result.
+ *	in the result area.
  *
  * Side effects:
  *	The content of holdId will be modified.
@@ -398,11 +413,24 @@ RatHoldExtract(Tcl_Interp *interp, char *prefix, Tcl_DString *usedVariablesPtr,
 	Tcl_DString *filesPtr)
 {
     static int holdId = 0;
-    char buf[1024];
+    Tcl_Channel ch;
+    char buf[1024], *cPtr;
     int i, oldId;
+    Tcl_Obj *oPtr = Tcl_NewObj();
 
     /*
-     * Now we should read the file, first we set the holdId variable
+     * We start by reading the file. This is complicated by the fact that
+     * the file is encoded in utf-8.
+     */
+    ch = Tcl_OpenFileChannel(interp, prefix, "r", 0);
+    Tcl_SetChannelOption(interp, ch, "-encoding", "utf-8");
+    i = Tcl_Seek(ch, 0, SEEK_END);
+    Tcl_Seek(ch, 0, SEEK_SET);
+    Tcl_ReadChars(ch, oPtr, i, 0);
+    Tcl_Close(interp, ch);
+
+    /*
+     * Now we should eval the data, first we set the holdId variable
      * so that the file can generate unique handlers, after the read
      * we remember a new value of holdId. Then we get the right entry
      * in the list of files.
@@ -410,9 +438,9 @@ RatHoldExtract(Tcl_Interp *interp, char *prefix, Tcl_DString *usedVariablesPtr,
     oldId = holdId;
     sprintf(buf, "%d", holdId);
     Tcl_SetVar(interp, "holdId", buf, 0);
-    Tcl_EvalFile(interp, prefix);
-    Tcl_ResetResult(interp);
-    sprintf(interp->result, "hold%d", holdId);
+    Tcl_EvalObjEx(interp, oPtr, TCL_EVAL_DIRECT);
+    sprintf(buf, "hold%d", holdId);
+    Tcl_SetResult(interp, buf, TCL_VOLATILE);
     Tcl_GetInt(interp, Tcl_GetVar(interp, "holdId", 0), &holdId);
     if (usedVariablesPtr) {
 	for (i=oldId; i<holdId; i++) {
@@ -420,7 +448,7 @@ RatHoldExtract(Tcl_Interp *interp, char *prefix, Tcl_DString *usedVariablesPtr,
 	    Tcl_DStringAppendElement(usedVariablesPtr, buf);
 	}
     }
-    sprintf(buf, "%s.desc", prefix);
+    snprintf(buf, sizeof(buf), "%s.desc", prefix);
     if (filesPtr) {
 	Tcl_DStringAppendElement(filesPtr, prefix);
 	Tcl_DStringAppendElement(filesPtr, buf);
@@ -429,5 +457,90 @@ RatHoldExtract(Tcl_Interp *interp, char *prefix, Tcl_DString *usedVariablesPtr,
 	unlink(buf);
     }
     fflush(stderr);
+
+    RatStrNCpy(buf, prefix, sizeof(buf));
+    if ((cPtr = strrchr(buf, '/'))) *cPtr = '\0';
+    RatHoldUpdateVars(interp, buf, -1);
     return TCL_OK;
 }
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * RatHoldInitVars --
+ *
+ *	Initialize some hold variables
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *	Some global variables are initialized
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+RatHoldInitVars(Tcl_Interp *interp)
+{
+    char *dir;
+    Tcl_DString tildeBuffer;
+
+    Tcl_DStringInit(&tildeBuffer);
+    dir = Tcl_GetVar2(interp, "option", "send_cache", TCL_GLOBAL_ONLY);
+    dir = Tcl_TranslateFileName(interp, dir, &tildeBuffer);
+    RatHoldList(interp, dir, NULL);
+    Tcl_ListObjLength(interp, Tcl_GetObjResult(interp), &numDeferred);
+
+    dir = Tcl_GetVar2(interp, "option", "hold_dir", TCL_GLOBAL_ONLY);
+    dir = Tcl_TranslateFileName(interp, dir, &tildeBuffer);
+    if (TCL_OK == RatHoldList(interp, dir, NULL)) {
+	Tcl_ListObjLength(interp, Tcl_GetObjResult(interp), &numHeld);
+    } else {
+	numHeld = 0;
+    }
+    Tcl_DStringFree(&tildeBuffer);
+
+    Tcl_SetVar2Ex(interp, "numDeferred", NULL, Tcl_NewIntObj(numDeferred),
+	    TCL_GLOBAL_ONLY);
+    Tcl_SetVar2Ex(interp, "numHeld", NULL, Tcl_NewIntObj(numHeld),
+	    TCL_GLOBAL_ONLY);
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * RatHoldUpdateVars --
+ *
+ *	Update the hold variables
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *	Some global variables may be modified
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+RatHoldUpdateVars(Tcl_Interp *interp, char *dir, int diff)
+{
+    char *senddir, *varname;
+    Tcl_DString tildeBuffer;
+    int *intvar;
+
+    Tcl_DStringInit(&tildeBuffer);
+    senddir = Tcl_GetVar2(interp, "option", "send_cache", TCL_GLOBAL_ONLY);
+    senddir = Tcl_TranslateFileName(interp, senddir, &tildeBuffer);
+    if (!strcmp(dir, senddir)) {
+	varname = "numDeferred";
+	intvar = &numDeferred;
+    } else {
+	varname = "numHeld";
+	intvar = &numHeld;
+    }
+
+    *intvar += diff;
+    Tcl_SetVar2Ex(interp, varname, NULL,Tcl_NewIntObj(*intvar),TCL_GLOBAL_ONLY);
+}
