@@ -3,8 +3,8 @@
  *
  *	This file contains basic support for sending messages via SMTP.
  *
- * TkRat software and its included text is Copyright 1996,1997,1998
- * by Martin Forssn
+ * TkRat software and its included text is Copyright 1996-2000 by
+ * Martin Forssn
  *
  * The full text of the legal notices is contained in the file called
  * COPYRIGHT, included with this distribution.
@@ -49,6 +49,7 @@ static int tableAllocated = 0;
 static void MailcapReload(Tcl_Interp *interp);
 static char *ExpandString(Tcl_Interp *interp, BodyInfo *bodyInfoPtr, char *s,
 	char **filePtr);
+static void FixQuotes(char *s);
 
 
 /*
@@ -83,7 +84,7 @@ MailcapReload(Tcl_Interp *interp)
      * Free old data
      */
     for (i=0; i<numTextBlocks; i++) {
-	free(textBlock[i]);
+	ckfree(textBlock[i]);
     }
     numTextBlocks = tableSize = 0;
 
@@ -123,9 +124,10 @@ MailcapReload(Tcl_Interp *interp)
 	}
 	if (allocTextBlocks == numTextBlocks) {
 	    allocTextBlocks += 5;
-	    textBlock = REALLOC(textBlock, allocTextBlocks*sizeof(char*));
+	    textBlock =
+		    (char**)ckrealloc(textBlock, allocTextBlocks*sizeof(char*));
 	}
-	data = textBlock[numTextBlocks++] = (char*)malloc(sbuf.st_size+1);
+	data = textBlock[numTextBlocks++] = (char*)ckalloc(sbuf.st_size+1);
 	read(fd, data, sbuf.st_size);
 	close(fd);
 	data[sbuf.st_size] = '\0';
@@ -143,7 +145,7 @@ MailcapReload(Tcl_Interp *interp)
 	 *   continue to copy the next line etc.
 	 */
 	for (i = line = 0, dstPtr = data; i < sbuf.st_size; i++) {
-	    while (data[i] && isspace(data[i])) {
+	    while (data[i] && isspace((unsigned char)data[i])) {
 	        if ('\n' == data[i]) {
 		    line++;
 		}
@@ -183,14 +185,13 @@ MailcapReload(Tcl_Interp *interp)
 	     */
 	    if (tableSize == tableAllocated) {
 		tableAllocated = tableSize+64;
-		tablePtr = (MailcapEntry*)REALLOC(tablePtr,
+		tablePtr = (MailcapEntry*)ckrealloc(tablePtr,
 			tableAllocated*sizeof(MailcapEntry));
 	    }
 	    tablePtr[tableSize].type = cPtr;
 	    for (;'/' != *cPtr && ';' != *cPtr && *cPtr; cPtr++);
 	    if (!*cPtr) {
-		sprintf(buf, "Syntax error in %s at line %d", s, line);
-		RatLog(interp, RAT_ERROR, buf, 0);
+		RatLogF(interp, RAT_ERROR, "syntax_error", RATLOG_TIME, s,line);
 		continue;
 	    }
 	    if (';' == *cPtr) {
@@ -201,19 +202,19 @@ MailcapReload(Tcl_Interp *interp)
 		tablePtr[tableSize].subtype = cPtr;
 		for (;';' != *cPtr && *cPtr; cPtr++);
 		if (!*cPtr) {
-		    sprintf(buf, "Syntax error in %s at line %d", s, line);
-		    RatLog(interp, RAT_ERROR, buf, 0);
+		    RatLogF(interp, RAT_ERROR, "syntax_error", RATLOG_TIME,
+			    s, line);
 		    continue;
 		}
-		for (tPtr = cPtr-1; *tPtr && isspace(*tPtr); tPtr--) {
+		for (tPtr = cPtr-1;
+			*tPtr && isspace((unsigned char)*tPtr); tPtr--) {
 		    *tPtr = '\0';
 		}
 		*cPtr++ = '\0';
 	    }
-	    for (;isspace(*cPtr) && *cPtr; cPtr++);
+	    for (;isspace((unsigned char)*cPtr) && *cPtr; cPtr++);
 	    if (!*cPtr) {
-		sprintf(buf, "Syntax error in %s at line %d", s, line);
-		RatLog(interp, RAT_ERROR, buf, 0);
+		RatLogF(interp, RAT_ERROR, "syntax_error", RATLOG_TIME, s,line);
 		continue;
 	    }
 	    tablePtr[tableSize].view = cPtr;
@@ -230,7 +231,7 @@ MailcapReload(Tcl_Interp *interp)
 	    while (*cPtr) {
 		*cPtr++ = '\0';
 		cPtrPtr = NULL;
-		for (;isspace(*cPtr) && *cPtr; cPtr++);
+		for (;isspace((unsigned char)*cPtr) && *cPtr; cPtr++);
 		if (!strncasecmp(cPtr, "test", 4)) {
 		    cPtrPtr = &tablePtr[tableSize].test;
 		} else if (!strncasecmp(cPtr, "composetyped", 12)) {
@@ -250,27 +251,45 @@ MailcapReload(Tcl_Interp *interp)
 		} else if (!strncasecmp(cPtr, "copiousoutput", 13)) {
 		    tablePtr[tableSize].copiousoutput = 1;
 		}
-		for (;isalpha(*cPtr) && *cPtr; cPtr++);
-		for (;isspace(*cPtr) && *cPtr; cPtr++);
+		for (;isalpha((unsigned char)*cPtr) && *cPtr; cPtr++);
+		for (;isspace((unsigned char)*cPtr) && *cPtr; cPtr++);
 		if (!cPtrPtr) {
 		    continue;
 		}
 		if ('=' != *cPtr) {
-		     sprintf(buf, "Syntax error in %s at line %d", s, line);
-		     RatLog(interp, RAT_ERROR, buf, 0);
+		     snprintf(buf, sizeof(buf),
+			    "Syntax error in %s at line %d", s, line);
 		     for (; *cPtr; cPtr++);
 		     continue;
 		}
-		for (cPtr++;isspace(*cPtr) && *cPtr; cPtr++);
+		for (cPtr++;isspace((unsigned char)*cPtr) && *cPtr; cPtr++);
 		if (!*cPtr) {
-		     sprintf(buf, "Syntax error in %s at line %d", s, line);
-		     RatLog(interp, RAT_ERROR, buf, 0);
+		     snprintf(buf, sizeof(buf),
+			    "Syntax error in %s at line %d", s, line);
 		     for (; *cPtr; cPtr++);
 		     continue;
 		}
 		*cPtrPtr = cPtr;
 		for (;*cPtr && (';' != *cPtr || '\\' == *(cPtr-1)); cPtr++);
 	    }
+	    if (tablePtr[tableSize].view) {
+		FixQuotes(tablePtr[tableSize].view);
+	    }
+	    if (tablePtr[tableSize].test) {
+		FixQuotes(tablePtr[tableSize].test);
+	    }
+	    if (tablePtr[tableSize].compose) {
+		FixQuotes(tablePtr[tableSize].compose);
+	    }
+	    if (tablePtr[tableSize].composetyped) {
+		FixQuotes(tablePtr[tableSize].composetyped);
+	    }
+	    if (tablePtr[tableSize].edit) {
+		FixQuotes(tablePtr[tableSize].edit);
+	    }
+	    if (tablePtr[tableSize].print) {
+		FixQuotes(tablePtr[tableSize].print);
+	    }
 	    tableSize++;
 	}
     }
@@ -343,7 +362,7 @@ ExpandString(Tcl_Interp *interp, BodyInfo *bodyInfoPtr, char *s, char **filePtr)
 		if (0 == Tcl_DStringLength(&file)) {
 		    Tcl_DStringAppend(&file, "/tmp/rat.", -1);
 		    RatGenId(NULL, interp, 0, NULL);
-		    Tcl_DStringAppend(&file, interp->result, -1);
+		    Tcl_DStringAppend(&file, Tcl_GetStringResult(interp), -1);
 		    *filePtr = Tcl_DStringValue(&file);
 		}
 		Tcl_DStringAppend(&ds, Tcl_DStringValue(&file), -1);
@@ -434,7 +453,7 @@ RatMcapFindCmd(Tcl_Interp *interp, BodyInfo *bodyInfoPtr)
     Tcl_DString ds;
     char *cmd, *file;
     Tcl_Channel channel;
-    int i;
+    int i, perm;
 
     /*
      * We start by making sure that the mailcap files have been loaded
@@ -461,7 +480,9 @@ RatMcapFindCmd(Tcl_Interp *interp, BodyInfo *bodyInfoPtr)
 		continue;
 	    }
 	    if (file) {
-		channel = Tcl_OpenFileChannel(interp, file, "w", 0600);
+		Tcl_GetInt(interp, Tcl_GetVar2(interp, "option", "permissions",
+			TCL_GLOBAL_ONLY), &perm);
+		channel = Tcl_OpenFileChannel(interp, file, "w", perm);
 		RatBodySave(interp, channel, bodyInfoPtr, 0, 1);
 		Tcl_Close(interp, channel);
 	    }
@@ -516,3 +537,56 @@ RatMailcapReload(ClientData dummy, Tcl_Interp *interp, int argc, char *argv[])
     MailcapReload(interp);
     return TCL_OK;
 }
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * FixQuotes --
+ *
+ *      Change pairs of single quotes to braces
+ *
+ * Results:
+ *	None
+ *
+ * Side effects:
+ *	May modify the passed string
+ *
+ *
+ *----------------------------------------------------------------------
+ */
+
+static void
+FixQuotes(char *line)
+{
+    char *c, *s, *e;
+
+    c = line;
+    while(*c) {
+	/*
+	 * Find start
+	 */
+	do {
+	    if (NULL == (s = strchr(c, '\''))) {
+		return;
+	    }
+	    c = s+1;
+	} while (s == line || !isspace(*(s-1)));
+
+	/*
+	 * Find end
+	 */
+	do {
+	    if (NULL == (e = strchr(c, '\''))) {
+		return;
+	    }
+	    c = e+1;
+	} while ('\\' != *(e-1) && e[1] && !isspace(e[1]));
+
+	/*
+	 * Convert to braces
+	 */
+	*s = '{';
+	*e = '}';
+    }
+}
