@@ -3,8 +3,8 @@
  *
  *	This file contains compatibility functions.
  *
- * TkRat software and its included text is Copyright 1996,1997,1998
- * by Martin Forssn
+ * TkRat software and its included text is Copyright 1996-2000 by
+ * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
  * COPYRIGHT, included with this distribution.
@@ -89,15 +89,15 @@ RatPGPPhrase(Tcl_Interp *interp)
 	return cpystr(pgpPass);
     }
 
-    strcpy(buf, "RatGetPGPPassPhrase");
+    RatStrNCpy(buf, "RatGetPGPPassPhrase", sizeof(buf));
     Tcl_Eval(interp, buf);
-    Tcl_SplitList(interp, interp->result, &argc, &argv);
+    Tcl_SplitList(interp, Tcl_GetStringResult(interp), &argc, &argv);
     if (!strcmp("ok", argv[0])) {
 	Tcl_GetBoolean(interp,
 		   Tcl_GetVar2(interp, "option", "cache_pgp", TCL_GLOBAL_ONLY),
 		   &doCache);
 	if (doCache) {
-	    strcpy(pgpPass, argv[1]);
+	    RatStrNCpy(pgpPass, argv[1], sizeof(pgpPass));
 	    pgpPassValid = 1;
 	    if (timeout) {
 		pgpPassToken = Tcl_CreateTimerHandler(timeout*1000,
@@ -107,10 +107,10 @@ RatPGPPhrase(Tcl_Interp *interp)
 	    }
 	}
 	result = cpystr(argv[1]);
-	free(argv);
+	ckfree(argv);
 	return result;
     } else {
-	free(argv);
+	ckfree(argv);
 	return NULL;
     }
 }
@@ -145,7 +145,7 @@ RatSenderPGPPhrase(Tcl_Interp *interp)
 
     if (!strncmp("PHRASE ", result, 7)) {
 	if (argv) {
-	    free(argv);
+	    ckfree(argv);
 	}
 	Tcl_SplitList(interp, result, &argc, &argv);
 	memset(result, '\0', strlen(result));
@@ -179,13 +179,12 @@ RatPGPBodyCheck(Tcl_Interp *interp, MessageProcInfo *procInfo,
 		BodyInfo **bodyInfoPtrPtr)
 {
     PARAMETER *parPtr;
-    int alg, prot, enc, enabled;
+    int alg, prot, enc;
     unsigned long length;
-    char *text, *start, *end, *middle;
+    char *text, *start, *end, *middle, *version;
 
-    Tcl_GetBoolean(interp, Tcl_GetVar2(interp, "option",
-	    "pgp_enable", TCL_GLOBAL_ONLY), &enabled);
-    if (!enabled) {
+    version = Tcl_GetVar2(interp, "option", "pgp_version", TCL_GLOBAL_ONLY);
+    if (!version || !strcmp("0", version)) {
 	return;
     }
 
@@ -215,7 +214,8 @@ RatPGPBodyCheck(Tcl_Interp *interp, MessageProcInfo *procInfo,
 	for (parPtr = (*bodyInfoPtrPtr)->bodyPtr->parameter; parPtr;
 		parPtr = parPtr->next) {
 	    if (!strcasecmp(parPtr->attribute, "micalg")
-		    && !strcasecmp(parPtr->value, "pgp-md5")) {
+		&& (!strcasecmp(parPtr->value, "pgp-md5")
+		    || !strcasecmp(parPtr->value, "pgp-sha1"))) {
 		alg = 1;
 	    } else if (!strcasecmp(parPtr->attribute, "protocol")
 		    && !strcasecmp(parPtr->value, "application/pgp-signature")){
@@ -237,14 +237,14 @@ RatPGPBodyCheck(Tcl_Interp *interp, MessageProcInfo *procInfo,
 	    && !strcasecmp("pgp", (*bodyInfoPtrPtr)->bodyPtr->subtype))) {
 	text = (*procInfo[(*bodyInfoPtrPtr)->type].fetchBodyProc)
 		(*bodyInfoPtrPtr, &length);
-	if (((start = RatPGPStrFind(text, length, "BEGIN PGP SIGNED", 1))
+	if (text && (((start = RatPGPStrFind(text, length,"BEGIN PGP SIGNED",1))
 		&& (middle = RatPGPStrFind(start, length - (start-text),
 			"BEGIN PGP SIGNATURE",1))
 		&& (end = RatPGPStrFind(middle, length - (middle-text),
 			"END PGP",1)))
 		|| ((start = RatPGPStrFind(text, length, "BEGIN PGP MESSAGE",1))
 		&& (end = RatPGPStrFind(start, length - (start-text),
-			"END PGP",1)))) {
+			"END PGP",1))))) {
 	    RatPGPHandleOld(interp, *bodyInfoPtrPtr, text, start, end+1);
 	}
     }
@@ -255,7 +255,7 @@ RatPGPBodyCheck(Tcl_Interp *interp, MessageProcInfo *procInfo,
  *
  * RatPGPCmd --
  *
- *      VHandle ratPGP command.
+ *      Handle ratPGP command.
  *
  * Results:
  *	A standard tcl result.
