@@ -3,8 +3,8 @@
  *
  *      Declarations of functions used in the Std folder and messages
  *
- * TkRat software and its included text is Copyright 1996,1997,1998
- * by Martin Forssn
+ * TkRat software and its included text is Copyright 1996-2000 by
+ * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
  * COPYRIGHT, included with this distribution.
@@ -26,7 +26,7 @@ extern char *RatPGPStrFind(char *haystack, long straws, char *needle,
 /* ratPGPprog.c */
 extern BODY *RatPGPEncrypt(Tcl_Interp *interp, ENVELOPE *env, BODY *body,
 			   int sign);
-extern BODY *RatPGPSign(Tcl_Interp *interp, BODY *body);
+extern BODY *RatPGPSign(Tcl_Interp *interp, ENVELOPE *env, BODY *body);
 extern void RatPGPChecksig(Tcl_Interp *interp, MessageProcInfo *procInfo,
 			   BodyInfo *bodyInfoPtr);
 extern void RatPGPDecrypt(Tcl_Interp *interp, MessageProcInfo *procInfo,
