@@ -3,8 +3,8 @@
  *
  *	This file contains basic support for sending messages via SMTP.
  *
- * TkRat software and its included text is Copyright 1996,1997,1998
- * by Martin Forssn
+ * TkRat software and its included text is Copyright 1996-2000 by
+ * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
  * COPYRIGHT, included with this distribution.
@@ -28,7 +28,7 @@ typedef struct SMTPChannelPriv {
 static char *RatTimedGets(Tcl_Interp *interp, Tcl_Channel channel, int timeout);
 static int RatSendCommand(Tcl_Interp *interp, Tcl_Channel channel, char *cmd);
 static int RatSendRcpt(Tcl_Interp *interp, Tcl_Channel channel,
-	ADDRESS *adrPtr, DSNhandle handle, int *size, int verbose);
+	ADDRESS *adrPtr, DSNhandle handle, int verbose);
 
 
 /*
@@ -84,7 +84,7 @@ RatTimedGets(Tcl_Interp *interp, Tcl_Channel channel, int timeout)
  *
  * Results:
  *	A standard Tcl result and an eventual error messages in 
- *	interp->result.
+ *	the result area.
  *
  * Side effects:
  *	None.
@@ -142,21 +142,26 @@ RatSendCommand(Tcl_Interp *interp, Tcl_Channel channel, char *cmd)
 
 static int
 RatSendRcpt(Tcl_Interp *interp, Tcl_Channel channel, ADDRESS *adrPtr,
-	DSNhandle handle, int *size, int verbose)
+	DSNhandle handle, int verbose)
 {
-    char buf[1024], adr[1024], msg[1024];
+    char buf[2048], adr[1024];
     unsigned char *cPtr;
     int failures = 0, i;
 
     for (; adrPtr; adrPtr = adrPtr->next) {
+	if (RatAddressSize(adrPtr, 0) > sizeof(adr)) {
+	    RatLogF(interp, RAT_WARN, "ridiculously_long", RATLOG_TIME);
+	    failures++;
+	}
 	adr[0] = '\0';
 	rfc822_address(adr, adrPtr);
-	*size += strlen(adr)+8;
-	sprintf(buf, "RCPT TO:<%s>", adr);
+	snprintf(buf, sizeof(buf), "RCPT TO:<%s>", adr);
 	if (handle) {
 	    RatDSNAddRecipient(interp, handle,  adr);
-	    sprintf(buf+strlen(buf), " NOTIFY=SUCCESS,FAILURE,DELAY");
-	    sprintf(buf+strlen(buf), " ORCPT=rfc822;");
+	    snprintf(buf+strlen(buf), sizeof(buf)-strlen(buf),
+		    " NOTIFY=SUCCESS,FAILURE,DELAY");
+	    snprintf(buf+strlen(buf), sizeof(buf)-strlen(buf),
+		    " ORCPT=rfc822;");
 	    for (i = strlen(buf),cPtr = (unsigned char*)adr; *cPtr; cPtr++) {
 		if (*cPtr < 33 || *cPtr > 126 || *cPtr == '+' || *cPtr == '=') {
 		    buf[i++] = '+';
@@ -169,10 +174,7 @@ RatSendRcpt(Tcl_Interp *interp, Tcl_Channel channel, ADDRESS *adrPtr,
 	    buf[i] = '\0';
 	}
 	if (3 == verbose) {
-	    sprintf(msg, "%s %s...", Tcl_GetVar2(interp, "t", "send_rcpt",
-		    TCL_GLOBAL_ONLY), adr);
-	    RatLog(interp, RAT_PARSE, msg, 1);
-	    
+	    RatLogF(interp, RAT_PARSE, "send_rcpt", RATLOG_EXPLICIT, adr);
 	}
 	if (TCL_OK != RatSendCommand(interp, channel, buf)) {
 	    failures++;
@@ -202,10 +204,10 @@ SMTPChannel
 RatSMTPOpen (Tcl_Interp *interp, char *host, int verbose)
 {
     SMTPChannelPriv *chPtr;
-    char *reply, buf[1024], *msg, *cPtr;
+    char *reply, buf[1024], *cPtr;
     int port, timeout;
 
-    strcpy(buf, host);
+    RatStrNCpy(buf, host, sizeof(buf));
     if ((cPtr = strchr(buf, ':'))) {
 	*cPtr++ = '\0';
 	port = atoi(cPtr);
@@ -214,8 +216,7 @@ RatSMTPOpen (Tcl_Interp *interp, char *host, int verbose)
     }
 
     if (verbose > 1) {
-	msg = Tcl_GetVar2(interp, "t", "opening_connection", TCL_GLOBAL_ONLY);
-	RatLog(interp, RAT_PARSE, msg, 1);
+	RatLogF(interp, RAT_PARSE, "opening_connection", RATLOG_EXPLICIT);
     }
     chPtr = (SMTPChannelPriv*)ckalloc(sizeof(SMTPChannelPriv));
     chPtr->mime = chPtr->dsn = 0;
@@ -237,8 +238,7 @@ RatSMTPOpen (Tcl_Interp *interp, char *host, int verbose)
      * Get initial greeting
      */
     if (verbose > 1) {
-	msg = Tcl_GetVar2(interp, "t", "wait_greeting", TCL_GLOBAL_ONLY);
-	RatLog(interp, RAT_PARSE, msg, 1);
+	RatLogF(interp, RAT_PARSE, "wait_greeting", RATLOG_EXPLICIT);
     }
     Tcl_GetInt(interp, Tcl_GetVar2(interp, "option", "smtp_timeout",
 	    TCL_GLOBAL_ONLY), &timeout);
@@ -255,18 +255,17 @@ RatSMTPOpen (Tcl_Interp *interp, char *host, int verbose)
      * Send EHLO (HELO) and get capabilities
      */
     if (verbose > 1) {
-	msg = Tcl_GetVar2(interp, "t", "get_capabilities", TCL_GLOBAL_ONLY);
-	RatLog(interp, RAT_PARSE, msg, 1);
+	RatLogF(interp, RAT_PARSE, "get_capabilities", RATLOG_EXPLICIT);
     }
-    sprintf(buf, "EHLO %s\r\n", currentHost);
+    snprintf(buf, sizeof(buf), "EHLO %s\r\n", currentHost);
     Tcl_Write(chPtr->channel, buf, -1);
     reply = RatTimedGets(interp, chPtr->channel, timeout);
     if (!reply || '2' != reply[0]) {
-	sprintf(buf, "HELO %s.%s\r\n", currentHost, currentHost);
+	snprintf(buf, sizeof(buf), "HELO %s.%s\r\n", currentHost, currentHost);
 	Tcl_Write(chPtr->channel, buf, -1);
 	reply = RatTimedGets(interp, chPtr->channel, timeout);
     }
-    while (reply && strncmp("250 ", reply, 4)) {
+    while (reply) {
 	if (!reply) {
 	    Tcl_Close(interp, chPtr->channel);
 	    ckfree(chPtr);
@@ -277,11 +276,14 @@ RatSMTPOpen (Tcl_Interp *interp, char *host, int verbose)
 	} else if (!strncmp("DSN", &reply[4], 3)) {
 	    chPtr->dsn = 1;
 	}
+	if (!strncmp("250 ", reply, 4)) {
+	    break;
+	}
 	reply = RatTimedGets(interp, chPtr->channel, timeout);
     }
 
     if (verbose > 1) {
-	RatLog(interp, RAT_PARSE, "", 1);
+	RatLog(interp, RAT_PARSE, "", RATLOG_EXPLICIT);
     }
     return (SMTPChannel)chPtr;
 }
@@ -307,16 +309,14 @@ void
 RatSMTPClose (Tcl_Interp *interp, SMTPChannel channel, int verbose)
 {
     SMTPChannelPriv *chPtr = (SMTPChannelPriv*)channel;
-    char *msg;
 
     if (verbose > 1) {
-	msg = Tcl_GetVar2(interp, "t", "closing", TCL_GLOBAL_ONLY);
-	RatLog(interp, RAT_PARSE, msg, 1);
+	RatLogF(interp, RAT_PARSE, "closing", RATLOG_EXPLICIT);
     }
     Tcl_Write(chPtr->channel, "QUIT\r\n", -1);
     Tcl_Close(interp, chPtr->channel);
     if (verbose > 1) {
-	RatLog(interp, RAT_PARSE, "", 1);
+	RatLog(interp, RAT_PARSE, "", RATLOG_EXPLICIT);
     }
     ckfree(chPtr);
 }
@@ -330,7 +330,7 @@ RatSMTPClose (Tcl_Interp *interp, SMTPChannel channel, int verbose)
  *
  * Results:
  *	A standard Tcl result and an eventual error messages in 
- *	interp->result.
+ *	the result area.
  *
  * Side effects:
  *	The DSN structures may be updated.
@@ -344,8 +344,8 @@ RatSMTPSend (Tcl_Interp *interp, SMTPChannel channel, ENVELOPE *envPtr,
 	BODY *bodyPtr, int doDSN, int verbose)
 {
     SMTPChannelPriv *chPtr = (SMTPChannelPriv*)channel;
-    char buf[1024], *header, *msg;
-    int failures = 0, size = 4096;
+    char buf[1024], *header;
+    int failures = 0;
     DSNhandle handle = NULL;
 
     /*
@@ -363,7 +363,7 @@ RatSMTPSend (Tcl_Interp *interp, SMTPChannel channel, ENVELOPE *envPtr,
      * Check if we should request DSN
      */
     if (doDSN && !chPtr->dsn) {
-	RatLog(interp, RAT_WARN, "DSN not avaliable", 0);
+	RatLogF(interp, RAT_WARN, "no_dsn", RATLOG_TIME);
 	doDSN = 0;
     }
 
@@ -372,42 +372,34 @@ RatSMTPSend (Tcl_Interp *interp, SMTPChannel channel, ENVELOPE *envPtr,
      */
     if (verbose > 1) {
 	if (verbose == 2) {
-	    msg = Tcl_GetVar2(interp, "t", "send_envelope", TCL_GLOBAL_ONLY);
+	    RatLogF(interp, RAT_PARSE, "send_envelope", RATLOG_EXPLICIT);
 	} else {
-	    msg = Tcl_GetVar2(interp, "t", "send_from", TCL_GLOBAL_ONLY);
+	    RatLogF(interp, RAT_PARSE, "send_from", RATLOG_EXPLICIT);
 	}
-	RatLog(interp, RAT_PARSE, msg, 1);
     }
-    sprintf(buf, "MAIL FROM:<");
+    if (RatAddressSize(envPtr->return_path, 0) > sizeof(buf)-128) {
+	RatLogF(interp, RAT_WARN, "ridiculously_long", RATLOG_TIME);
+	return TCL_ERROR;
+    }
+    snprintf(buf, sizeof(buf), "MAIL FROM:<");
     rfc822_address(buf, envPtr->return_path);
     strcat(buf, ">");
     if (chPtr->mime) {
 	strcat(buf, " BODY=8BITMIME");
     }
-    size += strlen(buf);
-    size += (envPtr->remail?strlen(envPtr->remail):0);
-    size += (envPtr->date?strlen(envPtr->date):0);
-    size += (envPtr->subject?strlen(envPtr->subject):0);
-    size += (envPtr->in_reply_to?strlen(envPtr->in_reply_to):0);
-    size += (envPtr->message_id?strlen(envPtr->message_id):0);
-    size += (envPtr->newsgroups?strlen(envPtr->newsgroups):0);
-    size += (envPtr->followup_to?strlen(envPtr->followup_to):0);
-    size += (envPtr->references?strlen(envPtr->references):0);
     if (doDSN) {
 	RatGenId(NULL, interp, 0, NULL);
-	handle = RatDSNStartMessage(interp, interp->result, envPtr->subject);
+	handle = RatDSNStartMessage(interp, Tcl_GetStringResult(interp),
+		envPtr->subject);
 	strcat(buf, " ENVID=");
-	strcat(buf, interp->result);
+	strcat(buf, Tcl_GetStringResult(interp));
     }
     if (TCL_OK != RatSendCommand(interp, chPtr->channel, buf)) {
 	return TCL_ERROR;
     }
-    failures += RatSendRcpt(interp, chPtr->channel, envPtr->to, handle, &size,
-	    verbose);
-    failures += RatSendRcpt(interp, chPtr->channel, envPtr->cc, handle, &size,
-	    verbose);
-    failures += RatSendRcpt(interp, chPtr->channel, envPtr->bcc, handle, &size,
-	    verbose);
+    failures += RatSendRcpt(interp, chPtr->channel, envPtr->to, handle,verbose);
+    failures += RatSendRcpt(interp, chPtr->channel, envPtr->cc, handle,verbose);
+    failures += RatSendRcpt(interp, chPtr->channel, envPtr->bcc,handle,verbose);
     if (failures) {
 	goto error;
     }
@@ -416,19 +408,17 @@ RatSMTPSend (Tcl_Interp *interp, SMTPChannel channel, ENVELOPE *envPtr,
      * Send message data
      */
     if (verbose > 1) {
-	msg = Tcl_GetVar2(interp, "t", "send_data", TCL_GLOBAL_ONLY);
-	RatLog(interp, RAT_PARSE, msg, 1);
+	RatLogF(interp, RAT_PARSE, "send_data", RATLOG_EXPLICIT);
     }
     if (TCL_OK != RatSendCommand(interp, chPtr->channel, "DATA")) {
 	goto error;
     }
-    header = (char*)malloc(size);
+    header = (char*)ckalloc(RatHeaderSize(envPtr, bodyPtr));
     rfc822_output(header, envPtr, bodyPtr, RatTclPutsSMTP, chPtr->channel,
 	    chPtr->mime);
-    free(header);
+    ckfree(header);
     if (verbose > 1) {
-	msg = Tcl_GetVar2(interp, "t", "wait_ack", TCL_GLOBAL_ONLY);
-	RatLog(interp, RAT_PARSE, msg, 1);
+	RatLogF(interp, RAT_PARSE, "wait_ack", RATLOG_EXPLICIT);
     }
     if (TCL_OK != RatSendCommand(interp, chPtr->channel, ".")) {
 	goto error;
@@ -453,7 +443,7 @@ error:
  *      Check if a host supports DSN.
  *
  * Results:
- *	TCL_OK and interp->result will contain "1" if the host supports
+ *	TCL_OK and the result area will contain "1" if the host supports
  *	DSN and "0" otherwise.
  *
  * Side effects:
@@ -487,10 +477,9 @@ RatSMTPSupportDSN(ClientData dummy, Tcl_Interp *interp, int argc, char **argv)
 	result = 0;
     }
     if (verbose) {
-	RatLog(interp, RAT_PARSE, "", 1);
+	RatLog(interp, RAT_PARSE, "", RATLOG_EXPLICIT);
     }
-    Tcl_ResetResult(interp);
-    sprintf(interp->result, "%d", result);
+    Tcl_SetObjResult(interp, Tcl_NewIntObj(result));
     return TCL_OK;
 }
 
@@ -518,7 +507,7 @@ RatTclPutsSMTP(void *stream_x, char *string)
     char *cPtr, *srcPtr;
 
     if ('.' == string[0]) {
-	Tcl_Write(channel, ".", -1);
+	Tcl_Write(channel, ".", 1);
     }
 
     for (srcPtr = string; *srcPtr;) {
@@ -532,7 +521,7 @@ RatTclPutsSMTP(void *stream_x, char *string)
 	}
 	if (cPtr[2]) {
 	    if (-1 == Tcl_Write(channel, srcPtr, cPtr-srcPtr+3)
-		    || -1 == Tcl_Write(channel, ".", -1)) {
+		    || -1 == Tcl_Write(channel, ".", 1)) {
 		return 0;
 	    }
 	    srcPtr = cPtr+3;
@@ -546,3 +535,125 @@ RatTclPutsSMTP(void *stream_x, char *string)
 	return 1;
     }
 }
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * RatTclPutsSendmail --
+ *
+ *	Like RatTclPutsSMTP but also escapes changes line endings to
+ *	put \n
+ *
+ * Results:
+ *      Always returns 1L.
+ *
+ * Side effects:
+ *      None.
+ *
+ *
+ *----------------------------------------------------------------------
+ */
+
+long
+RatTclPutsSendmail(void *stream_x, char *string)
+{
+    Tcl_Channel channel = (Tcl_Channel)stream_x;
+    char *cPtr, *srcPtr;
+    int add;
+
+    for (srcPtr = string; *srcPtr;) {
+	if (!srcPtr[0] || !srcPtr[1]) {
+	    break;
+	}
+	add = 1;
+	for (cPtr = srcPtr; cPtr[1]; cPtr++) {
+	    if ('\r' == cPtr[0] && '\n' == cPtr[1]) {
+		cPtr--;
+		add = 2;
+		break;
+	    }
+	}
+	if (-1 == Tcl_Write(channel, srcPtr, (cPtr+1)-srcPtr)) {
+	    return 0;
+	}
+	srcPtr = cPtr+add;
+    }
+    if (*srcPtr && -1 == Tcl_Write(channel, srcPtr, -1)) {
+	return 0;
+    } else {
+	return 1;
+    }
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * RatHeaderSize --
+ *
+ *	Calculate size of header
+ *
+ * Results:
+ *      Maximum size of header
+ *
+ * Side effects:
+ *      None.
+ *
+ *
+ *----------------------------------------------------------------------
+ */
+
+static int RatHeaderLineSize(char *name, ENVELOPE *env, char *text);
+static int RatHeaderAddressSize(char *name, ENVELOPE *env, ADDRESS *adr);
+
+static int
+RatHeaderLineSize(char *name, ENVELOPE *env, char *text)
+{
+    if (text) {
+	return (env->remail ? 7 : 0) + strlen(name) + 2 + strlen(text) + 2;
+    } else {
+	return 0;
+    }
+}
+
+static int
+RatHeaderAddressSize(char *name, ENVELOPE *env, ADDRESS *adr)
+{
+    if (adr) {
+	return (env->remail?7:0) + strlen(name) + 2 + RatAddressSize(adr, 1)+2;
+    } else {
+	return 0;
+    }
+}
+
+size_t
+RatHeaderSize(ENVELOPE *env,BODY *body)
+{
+    size_t len = 0;
+
+    if (env->remail) len += strlen(env->remail);
+    len += RatHeaderLineSize("Newsgroups", env, env->newsgroups);
+    len += RatHeaderLineSize("Date", env, env->date); 
+    len += RatHeaderAddressSize("From", env, env->from);
+    len += RatHeaderAddressSize("Sender", env, env->sender);
+    len += RatHeaderAddressSize("Reply-To", env, env->reply_to);
+    len += RatHeaderLineSize("Subject", env, env->subject);
+    if (env->bcc && !(env->to || env->cc)) {
+	len += strlen("To: undisclosed recipients: ;\015\012");
+    }
+    len += RatHeaderAddressSize("To", env, env->to);
+    len += RatHeaderAddressSize("cc", env, env->cc);
+    len += RatHeaderLineSize("In-Reply-To", env, env->in_reply_to);
+    len += RatHeaderLineSize("Message-ID", env, env->message_id);
+    len += RatHeaderLineSize("Followup-to", env, env->followup_to);
+    len += RatHeaderLineSize("References", env, env->references);
+    if (body && !env->remail) {   /* not if remail or no body structure */
+	/*
+	 * TODO: Fix this correctly
+	 * Here we assume that the body headers will never become longer
+	 * than 8192 bytes
+	 */
+        len += 8192;
+    } 
+    len += 2;
+    return len;
+}
