@@ -4,14 +4,13 @@
  *	This is the subprocess which handles the actual sending of messages.
  *
  *
- * TkRat software and its included text is Copyright 1996,1997,1998
- * by Martin Forssn
+ * TkRat software and its included text is Copyright 1996-2000 by
+ * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
  * COPYRIGHT, included with this distribution.
  */
 
-#include <tk.h>
 #include "ratFolder.h"
 #include "ratPGP.h"
 
@@ -30,10 +29,12 @@ static SMTPChannel smtpChannel = NULL;
  * Create bodypart s procedure
  */
 static BODY *RatCreateBody(Tcl_Interp *interp, char *handler, ENVELOPE *env,
-	int *errorFlag, char *charset, Tcl_DString *files);
+	int *errorFlag, Tcl_DString *files);
 static int RatSenderSend(Tcl_Interp *interp, char *prefix,
-	Tcl_DString *usedVariables, Tcl_DString *files);
+	Tcl_DString *usedVariables, Tcl_DString *files, int *hardError);
 static void RatSenderStanddown(Tcl_Interp *interp);
+static int RatParseParameter(Tcl_Interp *interp, const char *src,
+	PARAMETER **dstPtrPtr);
 
 
 /*
@@ -59,7 +60,7 @@ RatSender(Tcl_Interp *interp)
     Tcl_DString usedVariables, files, result;
     char *buf, **argv, **sendlistArgv;
     CmdList *cmdPtr;
-    int sendlistArgc, argc, i, s, buflen;
+    int sendlistArgc, argc, i, s, buflen, hardError = 0;
 
     /*
      * Clear cached passwords
@@ -70,15 +71,15 @@ RatSender(Tcl_Interp *interp)
     Tcl_DStringInit(&files);
     Tcl_DStringInit(&result);
     buflen = 1024;
-    buf = (char*)malloc(buflen);
+    buf = (char*)ckalloc(buflen);
 
     while (1) {
 	if (cmdList) {
 	    cmdPtr = cmdList;
-	    strcpy(buf, cmdList->cmd);
+	    RatStrNCpy(buf, cmdList->cmd, buflen);
 	    cmdList = cmdList->next;
-	    free(cmdPtr->cmd);
-	    free(cmdPtr);
+	    ckfree(cmdPtr->cmd);
+	    ckfree(cmdPtr);
 	} else {
 	    i = 0;
 	    while (buf[buflen-2] = '\0',
@@ -87,7 +88,7 @@ RatSender(Tcl_Interp *interp)
 		    && buf[buflen-2] != '\n') {
 		i = buflen-1;
 		buflen += 1024;
-		buf = REALLOC(buf, buflen);
+		buf = ckrealloc(buf, buflen);
 	    }
 	    if (feof(stdin)) {
 		exit(0);
@@ -96,14 +97,13 @@ RatSender(Tcl_Interp *interp)
 	
 	if (!strncmp(buf, "SEND", 4)) {
 	    (void)Tcl_SplitList(interp, buf, &sendlistArgc, &sendlistArgv);
-	    for (s=1; s<sendlistArgc; s++) {
+	    for (s=1; s<sendlistArgc && !hardError; s++) {
 		(void)Tcl_SplitList(interp, sendlistArgv[s], &argc, &argv);
 		if (TCL_OK == RatSenderSend(interp, argv[1], &usedVariables,
-			&files)){
+			&files, &hardError)){
 		    Tcl_DStringAppendElement(&result, "SENT");
 		    Tcl_DStringAppendElement(&result, argv[0]);
-		    Tcl_DStringAppendElement(&result, argv[2]);
-		    free(argv);
+		    ckfree(argv);
 		    Tcl_SplitList(interp, Tcl_DStringValue(&files),&argc,&argv);
 		    for (i=0; i<argc; i++) {
 			(void)unlink(argv[i]);
@@ -112,16 +112,23 @@ RatSender(Tcl_Interp *interp)
 		    Tcl_DStringAppendElement(&result, "FAILED");
 		    Tcl_DStringAppendElement(&result, argv[0]);
 		    Tcl_DStringAppendElement(&result, argv[1]);
-		    Tcl_DStringAppendElement(&result, argv[2]);
-		    Tcl_DStringAppendElement(&result, interp->result);
+		    Tcl_DStringAppendElement(&result,
+			    Tcl_GetStringResult(interp));
+		    sprintf(buf, "%d", hardError);
+		    Tcl_DStringAppendElement(&result, buf);
 		}
-		free(argv);
+		ckfree(argv);
 		(void)Tcl_SplitList(interp, Tcl_DStringValue(&usedVariables),
 				    &argc, &argv);
 		for (i=0; i<argc; i++) {
 		    (void)Tcl_UnsetVar(interp, argv[i], TCL_GLOBAL_ONLY);
 		}
-		free(argv);
+		ckfree(argv);
+		for (i=Tcl_DStringLength(&result)-1; i>=0; i--) {
+		    if ('\n' == Tcl_DStringValue(&result)[i]) {
+			Tcl_DStringValue(&result)[i] = ' ';
+		    }
+		}
 		fwrite(Tcl_DStringValue(&result), Tcl_DStringLength(&result)+1,
 		       1, stdout);
 		fflush(stdout);
@@ -129,8 +136,10 @@ RatSender(Tcl_Interp *interp)
 		Tcl_DStringSetLength(&usedVariables, 0);
 		Tcl_DStringSetLength(&files, 0);
 	    }
-	    free(sendlistArgv);
+	    ckfree(sendlistArgv);
 	    RatSenderStanddown(interp);
+	} else if (!strncmp(buf, "RSET", 4)) {
+	    hardError = 0;
 	} else {
 	    exit(0);
 	}
@@ -159,11 +168,11 @@ RatSender(Tcl_Interp *interp)
 
 static int
 RatSenderSend(Tcl_Interp *interp, char *prefix, Tcl_DString *usedVariablesPtr,
-	Tcl_DString *filesPtr)
+	Tcl_DString *filesPtr, int *hardError)
 {
-    char *charset = Tcl_GetVar2(interp, "option", "charset", TCL_GLOBAL_ONLY);
     int listArgc, i, errorFlag = 0, requestDSN, useFrom, verbose;
-    char *tmp, buf[10000], **listArgv, *handler, *saveTo;
+    char *tmp, buf[1024], **listArgv, *handler, *saveTo, *header = NULL;
+    Tcl_Obj *oPtr;
     ENVELOPE *env;
     BODY *body;
 
@@ -173,7 +182,7 @@ RatSenderSend(Tcl_Interp *interp, char *prefix, Tcl_DString *usedVariablesPtr,
     if (TCL_OK != RatHoldExtract(interp, prefix, usedVariablesPtr, filesPtr)) {
 	return TCL_ERROR;
     }
-    handler = cpystr(interp->result);
+    handler = cpystr(Tcl_GetStringResult(interp));
 
     /*
      * Construct the headers
@@ -185,6 +194,7 @@ RatSenderSend(Tcl_Interp *interp, char *prefix, Tcl_DString *usedVariablesPtr,
     tmp = Tcl_GetVar2(interp, handler, "to", TCL_GLOBAL_ONLY);
     env = mail_newenvelope ();
     rfc822_parse_adrlist(&env->to, tmp, currentHost);
+    RatEncodeAddresses(interp, env->to);
     env->remail = cpystr(Tcl_GetVar2(interp,handler,"remail",TCL_GLOBAL_ONLY));
     if ((tmp = Tcl_GetVar2(interp, handler, "return_path", TCL_GLOBAL_ONLY)) 
 	    && !RatIsEmpty(tmp)) {
@@ -196,6 +206,7 @@ RatSenderSend(Tcl_Interp *interp, char *prefix, Tcl_DString *usedVariablesPtr,
 	env->return_path->mailbox = cpystr(currentMailboxName);
 	env->return_path->host = cpystr(currentHost);
     }
+    RatEncodeAddresses(interp, env->return_path);
     if (NULL==(env->date=Tcl_GetVar2(interp,handler,"date",TCL_GLOBAL_ONLY))) {
 	rfc822_date(buf);
 	env->date = buf;
@@ -211,18 +222,22 @@ RatSenderSend(Tcl_Interp *interp, char *prefix, Tcl_DString *usedVariablesPtr,
     }
     if (env->from) {
 	ADDRESS *adrPtr;
+	int cs;
 
 	for (adrPtr = env->from; adrPtr; adrPtr = adrPtr->next) {
 	    if (RatAddressIsMe(interp, adrPtr, 0)) {
 		break;
 	    }
 	}
-	if (!adrPtr) {
+	oPtr = Tcl_GetVar2Ex(interp, "option","create_sender",TCL_GLOBAL_ONLY);
+	Tcl_GetBooleanFromObj(interp, oPtr, &cs);
+	if (!adrPtr && cs) {
 	    env->sender = mail_newaddr();
 	    env->sender->personal =
 		    currentPersonalName ? cpystr(currentPersonalName) : NULL;
 	    env->sender->mailbox = cpystr(currentMailboxName);
 	    env->sender->host = cpystr(currentHost);
+	    RatEncodeAddresses(interp, env->sender);
 	} else {
 	    env->sender = NULL;
 	}
@@ -233,20 +248,25 @@ RatSenderSend(Tcl_Interp *interp, char *prefix, Tcl_DString *usedVariablesPtr,
 	env->from->mailbox = cpystr(currentMailboxName);
 	env->from->host = cpystr(currentHost);
     }
+    RatEncodeAddresses(interp, env->from);
     if ((tmp = Tcl_GetVar2(interp, handler, "reply_to", TCL_GLOBAL_ONLY))
 	    && !RatIsEmpty(tmp)) {
 	rfc822_parse_adrlist(&env->reply_to, tmp, currentHost);
+	RatEncodeAddresses(interp, env->reply_to);
+    }
+    oPtr = Tcl_GetVar2Ex(interp, handler, "subject", TCL_GLOBAL_ONLY);
+    if (oPtr) {
+	env->subject = (char*)RatEncodeHeaderLine(interp, oPtr, 9);
     }
-    env->subject = (char*)RatEncodeHeaderLine((unsigned char*)
-	    Tcl_GetVar2(interp, handler, "subject", TCL_GLOBAL_ONLY),
-	    charset, 9, 1);
     if ((tmp = Tcl_GetVar2(interp, handler, "cc", TCL_GLOBAL_ONLY))
 	    && !RatIsEmpty(tmp)) {
 	rfc822_parse_adrlist(&env->cc, tmp, currentHost);
+	RatEncodeAddresses(interp, env->cc);
     }
     if ((tmp = Tcl_GetVar2(interp, handler, "bcc", TCL_GLOBAL_ONLY))
 	    && !RatIsEmpty(tmp)) {
 	rfc822_parse_adrlist(&env->bcc, tmp, currentHost);
+	RatEncodeAddresses(interp, env->bcc);
     }
     env->in_reply_to = cpystr(Tcl_GetVar2(interp, handler, "in_reply_to",
 	    TCL_GLOBAL_ONLY));
@@ -258,7 +278,7 @@ RatSenderSend(Tcl_Interp *interp, char *prefix, Tcl_DString *usedVariablesPtr,
      * Construct the body
      */
     if ((tmp = Tcl_GetVar2(interp, handler, "body", TCL_GLOBAL_ONLY))) {
-	body = RatCreateBody(interp, tmp, env, &errorFlag, charset, filesPtr);
+	body = RatCreateBody(interp, tmp, env, &errorFlag, filesPtr);
     } else {
 	body = mail_newbody ();
     }
@@ -269,11 +289,11 @@ RatSenderSend(Tcl_Interp *interp, char *prefix, Tcl_DString *usedVariablesPtr,
     /*
      * Send the message
      */
+    header = (char*)ckalloc(RatHeaderSize(env, body));
     Tcl_GetInt(interp, Tcl_GetVar2(interp, "option", "smtp_verbose",
 	    TCL_GLOBAL_ONLY), &verbose);
     if (1 == verbose) {
-	tmp = Tcl_GetVar2(interp, "t", "sending_message", TCL_GLOBAL_ONLY);
-	RatLog(interp, RAT_PARSE, tmp, 1);
+	RatLogF(interp, RAT_PARSE, "sending_message", RATLOG_EXPLICIT);
     }
     tmp = Tcl_GetVar2(interp, "option", "sendprot", TCL_GLOBAL_ONLY);
     if (tmp && !strcmp(tmp, "smtp")) {
@@ -289,6 +309,7 @@ RatSenderSend(Tcl_Interp *interp, char *prefix, Tcl_DString *usedVariablesPtr,
 	    Tcl_SetResult(interp, 
 			  "Configuration error; no valid SMTP hosts given",
 		          TCL_STATIC);
+	    *hardError = 1;
 	    goto error;
 	}
 	for (i=0; !smtpChannel && i<listArgc; i++) {
@@ -297,6 +318,7 @@ RatSenderSend(Tcl_Interp *interp, char *prefix, Tcl_DString *usedVariablesPtr,
 	ckfree(listArgv);
 	if (!smtpChannel) {
 	    Tcl_SetResult(interp, "No valid SMTP hosts", TCL_STATIC);
+	    *hardError = 1;
 	    goto error;
 	}
 	if (TCL_OK != RatSMTPSend(interp, smtpChannel, env, body, requestDSN,
@@ -324,24 +346,44 @@ RatSenderSend(Tcl_Interp *interp, char *prefix, Tcl_DString *usedVariablesPtr,
 	if (NULL == (tmp = Tcl_GetVar2(interp,"option", "sendprog",
 		     TCL_GLOBAL_ONLY))) {
 	    Tcl_SetResult(interp, "Invalid send program", TCL_STATIC);
+	    *hardError = 1;
 	    goto error;
 	}
 	Tcl_DStringInit(&ds);
-	Tcl_DStringAppend(&ds, tmp, -1);
+	Tcl_DStringAppendElement(&ds, tmp);
+	Tcl_DStringAppendElement(&ds, "-i");
 	for (adrPtr = env->to; adrPtr; adrPtr = adrPtr->next) {
-	    buf[0] = '\0';
-	    rfc822_address(buf, adrPtr);
-	    Tcl_DStringAppendElement(&ds, buf);
+	    if (RatAddressSize(adrPtr, 0) > sizeof(buf)-1) {
+		Tcl_SetResult(interp, "Ridiculously long address", TCL_STATIC);
+		*hardError = 1;
+		goto error;
+	    } else {
+		buf[0] = '\0';
+		rfc822_address(buf, adrPtr);
+		Tcl_DStringAppendElement(&ds, buf);
+	    }
 	}
 	for (adrPtr = env->cc; adrPtr; adrPtr = adrPtr->next) {
-	    buf[0] = '\0';
-	    rfc822_address(buf, adrPtr);
-	    Tcl_DStringAppendElement(&ds, buf);
+	    if (RatAddressSize(adrPtr, 0) > sizeof(buf)-1) {
+		Tcl_SetResult(interp, "Ridiculously long address", TCL_STATIC);
+		*hardError = 1;
+		goto error;
+	    } else {
+		buf[0] = '\0';
+		rfc822_address(buf, adrPtr);
+		Tcl_DStringAppendElement(&ds, buf);
+	    }
 	}
 	for (adrPtr = env->bcc; adrPtr; adrPtr = adrPtr->next) {
-	    buf[0] = '\0';
-	    rfc822_address(buf, adrPtr);
-	    Tcl_DStringAppendElement(&ds, buf);
+	    if (RatAddressSize(adrPtr, 0) > sizeof(buf)-1) {
+		Tcl_SetResult(interp, "Ridiculously long address", TCL_STATIC);
+		*hardError = 1;
+		goto error;
+	    } else {
+		buf[0] = '\0';
+		rfc822_address(buf, adrPtr);
+		Tcl_DStringAppendElement(&ds, buf);
+	    }
 	}
 
 	Tcl_ResetResult(interp);
@@ -351,28 +393,30 @@ RatSenderSend(Tcl_Interp *interp, char *prefix, Tcl_DString *usedVariablesPtr,
 		    listArgv, TCL_STDIN|TCL_STDOUT|TCL_STDERR)))) {
 	    Tcl_DStringSetLength(&ds, 0);
 	    Tcl_DStringAppend(&ds, "Failed to run send program: ", -1);
-	    Tcl_DStringAppend(&ds, interp->result, -1);
+	    Tcl_DStringAppend(&ds, Tcl_GetStringResult(interp), -1);
 	    Tcl_DStringResult(interp, &ds);
 	    goto error;
 	} else {
 	    Tcl_DStringFree(&ds);
 	    ckfree(listArgv);
-	    rfc822_output(buf, env, body, RatTclPutsSMTP, channel, i);
+	    rfc822_output(header, env, body, RatTclPutsSendmail, channel, i);
 	    Tcl_Close(interp, channel);
 	}
     } else {
-	sprintf(buf, "Invalid send protocol '%s'", tmp ? tmp : "<NULL>");
+	snprintf(buf, sizeof(buf), "Invalid send protocol '%s'",
+		tmp ? tmp : "<NULL>");
 	Tcl_SetResult(interp, buf, TCL_VOLATILE);
+	*hardError = 1;
 	goto error;
     }
     if (verbose) {
-	RatLog(interp, RAT_PARSE, "", 1);
+	RatLog(interp, RAT_PARSE, "", RATLOG_EXPLICIT);
     }
 
     saveTo = Tcl_GetVar2(interp, handler, "save_to", TCL_GLOBAL_ONLY);
     if (saveTo && *saveTo) {
 	MESSAGECACHE elt;
-	Tcl_DString saveArg;
+	Tcl_DString saveArg, ds;
 	Tcl_Channel channel;
 	char saveFile[1024];
 	struct tm *tmPtr;
@@ -380,9 +424,11 @@ RatSenderSend(Tcl_Interp *interp, char *prefix, Tcl_DString *usedVariablesPtr,
 	int perm;
 
 	Tcl_DStringInit(&saveArg);
+	Tcl_DStringInit(&ds);
 	tmp = Tcl_GetVar2(interp, "option", "tmp", TCL_GLOBAL_ONLY);
 	RatGenId(NULL, interp, 0, NULL);
-	sprintf(saveFile, "%s/rat.%s", tmp, interp->result);
+	snprintf(saveFile, sizeof(saveFile), "%s/rat.%s",
+		tmp, Tcl_GetStringResult(interp));
 
 	Tcl_GetInt(interp, Tcl_GetVar2(interp, "option", "permissions",
 		TCL_GLOBAL_ONLY), &perm);
@@ -391,25 +437,41 @@ RatSenderSend(Tcl_Interp *interp, char *prefix, Tcl_DString *usedVariablesPtr,
 		    Tcl_PosixError(interp), (char*) NULL);
 	    goto error;
 	}
-	rfc822_output(buf, env, body, RatTclPuts, channel, 1);
+	rfc822_output(header, env, body, RatTclPuts, channel, 1);
 	Tcl_Close(interp, channel);
 
 	Tcl_DStringAppendElement(&saveArg, saveFile);
 	Tcl_DStringAppendElement(&saveArg, saveTo);
-	buf[0] = '\0';
-	rfc822_write_address(buf, env->to);
-	Tcl_DStringAppendElement(&saveArg, buf);
+	Tcl_DStringSetLength(&ds, RatAddressSize(env->to, 1));
+	Tcl_DStringValue(&ds)[0] = '\0';
+	rfc822_write_address(Tcl_DStringValue(&ds), env->to);
+	Tcl_DStringSetLength(&ds, strlen(Tcl_DStringValue(&ds)));
+	Tcl_DStringAppendElement(&saveArg,
+		RatDecodeHeader(interp, Tcl_DStringValue(&ds), 1));
 	if (env->from) {
-	    buf[0] = '\0';
-	    rfc822_write_address(buf, env->from);
+	    Tcl_DStringSetLength(&ds, RatAddressSize(env->from, 1));
+	    Tcl_DStringValue(&ds)[0] = '\0';
+	    rfc822_write_address(Tcl_DStringValue(&ds), env->from);
+	    Tcl_DStringSetLength(&ds, strlen(Tcl_DStringValue(&ds)));
 	} else {
-	    sprintf(buf, "%s@%s", currentMailboxName, currentHost);
+	    Tcl_DStringSetLength(&ds,
+		    strlen(currentMailboxName) + strlen(currentHost)+2);
+	    sprintf(Tcl_DStringValue(&ds), "%s@%s", currentMailboxName,
+		    currentHost);
 	}
-	Tcl_DStringAppendElement(&saveArg, buf);
-	buf[0] = '\0';
-	rfc822_write_address(buf, env->cc);
-	Tcl_DStringAppendElement(&saveArg, buf);
-	Tcl_DStringAppendElement(&saveArg, env->subject);
+	Tcl_DStringAppendElement(&saveArg,
+		RatDecodeHeader(interp, Tcl_DStringValue(&ds), 1));
+	Tcl_DStringSetLength(&ds, RatAddressSize(env->cc, 1));
+	Tcl_DStringValue(&ds)[0] = '\0';
+	rfc822_write_address(Tcl_DStringValue(&ds), env->cc);
+	Tcl_DStringSetLength(&ds, strlen(Tcl_DStringValue(&ds)));
+	Tcl_DStringAppendElement(&saveArg,
+		RatDecodeHeader(interp, Tcl_DStringValue(&ds), 1));
+	Tcl_DStringAppendElement(&saveArg, env->message_id);
+	Tcl_DStringAppendElement(&saveArg,
+		(env->in_reply_to ? env->in_reply_to : env->references));
+	Tcl_DStringAppendElement(&saveArg,
+		RatDecodeHeader(interp, env->subject, 0));
 	Tcl_DStringAppendElement(&saveArg, RAT_SEEN_STR);
 	now = time(NULL);
 	tmPtr = gmtime(&now);
@@ -431,11 +493,15 @@ RatSenderSend(Tcl_Interp *interp, char *prefix, Tcl_DString *usedVariablesPtr,
 
     mail_free_envelope(&env);
     mail_free_body(&body);
+    ckfree(header);
     return TCL_OK;
 
 error:
     if (verbose) {
-	RatLog(interp, RAT_PARSE, "", 1);
+	RatLog(interp, RAT_PARSE, "", RATLOG_EXPLICIT);
+    }
+    if (header) {
+	ckfree(header);
     }
     mail_free_envelope(&env);
     mail_free_body(&body);
@@ -483,8 +549,8 @@ RatSenderStanddown(Tcl_Interp *interp)
  *
  * Results:
  *      The return value is normally TCL_OK and the result can be found
- *      in interp->result. If something goes wrong TCL_ERROR is returned
- *      and an error message will be left in interp->result.
+ *      in the result area. If something goes wrong TCL_ERROR is returned
+ *      and an error message will be left in the result area.
  *
  * Side effects:
  *      None.
@@ -494,7 +560,7 @@ RatSenderStanddown(Tcl_Interp *interp)
  */
 static BODY*
 RatCreateBody(Tcl_Interp *interp, char *handler, ENVELOPE *env,
-	int *errorFlag, char *charset, Tcl_DString *filesPtr)
+	int *errorFlag, Tcl_DString *filesPtr)
 {
     BODY *body = mail_newbody();
     char *type;
@@ -503,7 +569,9 @@ RatCreateBody(Tcl_Interp *interp, char *handler, ENVELOPE *env,
     char buf[1024];
     char *filename = NULL;
     char *value;
+    char *children;
     int pgp_sign, pgp_encrypt;
+    Tcl_Obj *oPtr;
 
     if (NULL == (type = Tcl_GetVar2(interp, handler, "type",
 	    TCL_GLOBAL_ONLY))) {
@@ -530,11 +598,20 @@ RatCreateBody(Tcl_Interp *interp, char *handler, ENVELOPE *env,
 	else if (!strcasecmp(encoding, "quoted-printable"))
 					body->encoding = ENCQUOTEDPRINTABLE;
 	else {
-	    sprintf(buf, "Unkown encoding %s\n", encoding);
+	    snprintf(buf, sizeof(buf), "Unkown encoding %s\n", encoding);
 	    Tcl_SetResult(interp, buf, TCL_VOLATILE);
 	    (*errorFlag)++;
 	}
     }
+    /*
+     * Force all non-text and non-message bodyparts to binary
+     * (unless already encoded)
+     */
+    if (TYPETEXT != body->type && TYPEMESSAGE != body->type
+	&& (ENC7BIT == body->encoding || ENC8BIT == body->encoding)) {
+	body->encoding = ENCBINARY;
+    }
+
     if (NULL == (body->subtype = Tcl_GetVar2(interp, handler, "subtype",
 	    TCL_GLOBAL_ONLY))) {
 	Tcl_SetResult(interp,"Internal error, no bhandler(subtype)",TCL_STATIC);
@@ -543,54 +620,44 @@ RatCreateBody(Tcl_Interp *interp, char *handler, ENVELOPE *env,
     }
     body->subtype = cpystr(body->subtype);
     if ((parameter = Tcl_GetVar2(interp,handler,"parameter",TCL_GLOBAL_ONLY))) {
-	PARAMETER **parmPtrPtr = &body->parameter;
-	int parmListArgc, parmArgc, i;
-	char **parmListArgv, **parmArgv;
-	Tcl_SplitList(interp, parameter, &parmListArgc, &parmListArgv);
-	for (i=0; i<parmListArgc; i++) {
-	    if (TCL_ERROR == Tcl_SplitList(interp, parmListArgv[i], &parmArgc,
-		    &parmArgv) || 2 != parmArgc) {
-		sprintf(buf, "Illegal parameter: %s", parmListArgv[i]);
-		Tcl_SetResult(interp, buf, TCL_VOLATILE);
-		(*errorFlag)++;
-	    } else {
-		*parmPtrPtr = mail_newbody_parameter();
-		(*parmPtrPtr)->attribute = cpystr(parmArgv[0]);
-		(*parmPtrPtr)->value = cpystr(parmArgv[1]);
-		parmPtrPtr = &(*parmPtrPtr)->next;
-	    }
-	    ckfree(parmArgv);
-	}
-	ckfree(parmListArgv);
+	(*errorFlag) += RatParseParameter(interp, parameter, &body->parameter);
+    }
+    if ((parameter =
+	    Tcl_GetVar2(interp, handler, "disp_parm", TCL_GLOBAL_ONLY))) {
+	(*errorFlag) += RatParseParameter(interp, parameter,
+					  &body->disposition.parameter);
+    }
+    body->disposition.type = cpystr(Tcl_GetVar2(interp, handler,
+	    "disp_type", TCL_GLOBAL_ONLY));
+    if (body->disposition.type && !strlen(body->disposition.type)) {
+	body->disposition.type = NULL;
     }
     body->id = cpystr(Tcl_GetVar2(interp, handler, "id", TCL_GLOBAL_ONLY));
     if (body->id && !strlen(body->id)) {
 	body->id = NULL;
     }
-    body->description = (char*)RatEncodeHeaderLine((unsigned char*)
-	    Tcl_GetVar2(interp, handler, "description", TCL_GLOBAL_ONLY),
-	    charset, 13, 1);
+    oPtr = Tcl_GetVar2Ex(interp, handler, "description", TCL_GLOBAL_ONLY);
+    if (oPtr) {
+	body->description = (char*)RatEncodeHeaderLine(interp, oPtr, 13);
+    }
     if (body->description && !strlen(body->description)) {
 	body->description = NULL;
     }
 
-    if (TYPEMULTIPART == body->type) {
+    if (TYPEMULTIPART == body->type && NULL != (children =
+	    Tcl_GetVar2(interp, handler, "children", TCL_GLOBAL_ONLY))) {
 	PART **partPtrPtr = &body->nested.part;
 	int childrenArgc, i;
 	char **childrenArgv;
-	char *children = Tcl_GetVar2(interp, handler, "children",
-		TCL_GLOBAL_ONLY);
 	Tcl_SplitList(interp, children, &childrenArgc, &childrenArgv);
 	for (i=0; i<childrenArgc; i++) {
 	    *partPtrPtr = mail_newbody_part();
 	    (*partPtrPtr)->body = *RatCreateBody(interp, childrenArgv[i],
-		    env, errorFlag, charset, filesPtr);
+		    env, errorFlag, filesPtr);
 	    partPtrPtr = &(*partPtrPtr)->next;
 	}
     } else if (TYPEMESSAGE == body->type) {
-	struct stat sbuf;
-	char *message;
-	int fd;
+	unsigned char *message;
 
 	if (NULL == (filename = Tcl_GetVar2(interp, handler,"filename",
 		TCL_GLOBAL_ONLY))) {
@@ -599,65 +666,41 @@ RatCreateBody(Tcl_Interp *interp, char *handler, ENVELOPE *env,
 	    (*errorFlag)++;
 	    return body;
 	}
-	if (-1 == (fd = open(filename, O_RDONLY))) {
-	    sprintf(buf, "Failed to open file \"%s\": %s", filename,
-		    Tcl_PosixError(interp));
-	    Tcl_SetResult(interp, buf, TCL_VOLATILE);
+	message = Rat_ReadFile(interp, filename, &body->contents.text.size, 1);
+	if (NULL == message) {
 	    (*errorFlag)++;
 	    return body;
 	}
-	fstat(fd, &sbuf);
-	message = (char*)ckalloc(sbuf.st_size+1);
-	read(fd, message, sbuf.st_size);
-	message[sbuf.st_size] = '\0';
-	(void)close(fd);
-
 	body->nested.msg = RatParseMsg(interp, message);
-	body->contents.text.data = message;
-	body->contents.text.size = sbuf.st_size;
+	body->contents.text.data = (unsigned char*)message;
 
     } else {
-	FILE *fp;
-	struct stat statbuf;
-	int len = 0, allocated, ci;
-	char *data;
+	unsigned char *data;
 
-	if (NULL == (filename = Tcl_GetVar2(interp, handler,"filename",
+	if (NULL == (filename = Tcl_GetVar2(interp, handler, "filename",
 		TCL_GLOBAL_ONLY))) {
 	    Tcl_SetResult(interp, "Internal error, no bhandler(filename)",
 			  TCL_STATIC);
 	    (*errorFlag)++;
 	    return body;
 	}
-	if (NULL == (fp = fopen(filename, "r"))) {
-	    sprintf(buf, "Failed to open file \"%s\": %s", filename,
-		    Tcl_PosixError(interp));
-	    Tcl_SetResult(interp, buf, TCL_VOLATILE);
+	if (ENCBINARY != body->encoding && TYPETEXT == body->type) {
+	    data = Rat_ReadFile(interp, filename, &body->contents.text.size,1);
+	} else {
+	    data = Rat_ReadFile(interp, filename, &body->contents.text.size,0);
+	}
+	if (NULL == data) {
 	    (*errorFlag)++;
 	    return body;
 	}
-	fstat(fileno(fp), &statbuf);
-	allocated = (statbuf.st_size*105)/100;
-	data = (char*)ckalloc(allocated);
-	if (ENCBINARY != body->encoding) {
-	    while (EOF != (ci = getc(fp))) {
-		if (len >= allocated-2) {
-		    allocated += 1024;
-		    data = (char*)REALLOC(data, allocated);
-		}
-		if (ci == '\n') {
-		    data[len++] = '\r';
-		}
-		data[len++] = ci;
-	    }
-	} else {
-	    fread(data, statbuf.st_size, 1, fp);
-	    len = statbuf.st_size;
-	}
-	data[len] = '\0';
-	fclose(fp);
 	body->contents.text.data = data;
-	body->size.bytes = body->contents.text.size = len;
+	body->size.bytes = body->contents.text.size;
+	
+	if (TYPEMULTIPART == body->type) {
+	    body->type = TYPEAPPLICATION;
+	    ckfree(body->subtype);
+	    body->subtype = cpystr("octet-stream");
+	}
     }
     if (filename) {
 	int removeFile = 0;
@@ -685,7 +728,7 @@ RatCreateBody(Tcl_Interp *interp, char *handler, ENVELOPE *env,
 	if (pgp_encrypt) {
 	    body = RatPGPEncrypt(interp, env, body, pgp_sign);
 	} else {
-	    body = RatPGPSign(interp, body);
+	    body = RatPGPSign(interp, env, body);
 	}
 	if (!body) {
 	    Tcl_SetResult(interp, "Failed to do PGP operation", TCL_STATIC);
@@ -697,6 +740,53 @@ RatCreateBody(Tcl_Interp *interp, char *handler, ENVELOPE *env,
     return body;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * RatParseParameter --
+ *
+ *	Splits a parameter list into elements
+ *
+ * Results:
+ *      number of encountered errors
+ *
+ * Side effects:
+ *      None.
+ *
+ *
+ *----------------------------------------------------------------------
+ */
+
+static int
+RatParseParameter(Tcl_Interp *interp, const char *src, PARAMETER **dstPtrPtr)
+{
+    int parmListArgc, parmArgc, i, errcount = 0;
+    char **parmListArgv, **parmArgv;
+    Tcl_Obj *oPtr;
+    char buf[1024];
+
+    Tcl_SplitList(interp, src, &parmListArgc, &parmListArgv);
+    for (i=0; i<parmListArgc; i++) {
+	if (TCL_ERROR == Tcl_SplitList(interp, parmListArgv[i], &parmArgc,
+		&parmArgv) || 2 != parmArgc) {
+	    snprintf(buf, sizeof(buf), "Illegal parameter: %s",
+		    parmListArgv[i]);
+	    Tcl_SetResult(interp, buf, TCL_VOLATILE);
+	    errcount++;
+	} else {
+	    *dstPtrPtr = mail_newbody_parameter();
+	    (*dstPtrPtr)->attribute = cpystr(parmArgv[0]);
+	    oPtr = Tcl_NewStringObj(parmArgv[1], -1);
+	    (*dstPtrPtr)->value = RatEncodeHeaderLine(interp, oPtr, 0);
+	    Tcl_DecrRefCount(oPtr);
+	    dstPtrPtr = &(*dstPtrPtr)->next;
+	}
+	ckfree(parmArgv);
+    }
+    ckfree(parmListArgv);
+    return errcount;
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -730,7 +820,7 @@ RatSendPGPCommand(char *cmd)
 	}
 	buf[strlen(buf)-1] = '\0';
 	if (strncmp("PGP ", buf, 4)) {
-	    *cmdPtrPtr = (CmdList*)malloc(sizeof(CmdList));
+	    *cmdPtrPtr = (CmdList*)ckalloc(sizeof(CmdList));
 	    (*cmdPtrPtr)->cmd = cpystr(buf);
 	    (*cmdPtrPtr)->next = NULL;
 	    cmdPtrPtr = &(*cmdPtrPtr)->next;
