@@ -3,8 +3,8 @@
  *
  *	This file contains code which implements standard c-client messages.
  *
- * TkRat software and its included text is Copyright 1996,1997,1998
- * by Martin Forssn
+ * TkRat software and its included text is Copyright 1996-2000 by
+ * Martin Forssn
  *
  * The full text of the legal notice is contained in the file called
  * COPYRIGHT, included with this distribution.
@@ -38,6 +38,10 @@ typedef struct StdBodyInfo {
  */
 static int numStdMessages = 0;
 
+#ifdef MEM_DEBUG
+static char *mem_header = NULL;
+#endif /* MEM_DEBUG */
+
 
 /*
  *----------------------------------------------------------------------
@@ -65,11 +69,11 @@ RatStdMessagesInit(MessageProcInfo *messageProcInfoPtr)
     messageProcInfoPtr->createBodyProc = Std_CreateBodyProc;
     messageProcInfoPtr->fetchTextProc = Std_FetchTextProc;
     messageProcInfoPtr->envelopeProc = Std_EnvelopeProc;
-    messageProcInfoPtr->setIndexProc = Std_SetIndexProc;
     messageProcInfoPtr->msgDeleteProc = Std_MsgDeleteProc;
     messageProcInfoPtr->makeChildrenProc = Std_MakeChildrenProc;
     messageProcInfoPtr->fetchBodyProc = Std_FetchBodyProc;
     messageProcInfoPtr->bodyDeleteProc = Std_BodyDeleteProc;
+    messageProcInfoPtr->getInternalDateProc = Std_GetInternalDateProc;
 }
 
 
@@ -96,34 +100,18 @@ RatStdMessageCreate(Tcl_Interp *interp, RatFolderInfoPtr folderInfoPtr,
 	MAILSTREAM *stream, int msgNo, RatStdFolderType type,
 	char *host, char *user)
 {
-    StdMessageInfo *msgStdPtr=(StdMessageInfo*)ckalloc(sizeof(StdMessageInfo));
-    MessageInfo *msgPtr=(MessageInfo*)ckalloc(sizeof(MessageInfo));
-    char *name = (char*)ckalloc(16);
+    MessageInfo *msgPtr = (MessageInfo*)folderInfoPtr->privatePtr[msgNo];
+    StdMessageInfo *stdMsgPtr = (StdMessageInfo*)msgPtr->clientData;
 
-    msgPtr->folderInfoPtr = folderInfoPtr;
-    msgPtr->name = name;
-    msgPtr->type = RAT_CCLIENT_MESSAGE;
-    msgPtr->bodyInfoPtr = NULL;
-    msgPtr->msgNo = msgNo;
-    msgPtr->fromMe = RAT_ISME_UNKOWN;
-    msgPtr->toMe = RAT_ISME_UNKOWN;
-    msgPtr->clientData = (ClientData)msgStdPtr;
-    msgStdPtr->stream = stream;
-    msgStdPtr->envPtr =
-	    mail_fetchstructure_full(stream, msgNo+1, &msgStdPtr->bodyPtr, NIL);
-    if (NULL == msgStdPtr->envPtr) {
-	RatLog(interp, RAT_FATAL, "Failed to fetch message structure", 0);
-	exit(1);
-    }
-    msgStdPtr->eltPtr = mail_elt(stream, msgNo+1);
-    msgStdPtr->eltPtr->lockcount++;
-    msgStdPtr->type = type;
-    msgStdPtr->host = host;
-    msgStdPtr->user = user;
-    folderInfoPtr->privatePtr[msgNo] = (ClientData*) msgPtr;
-    sprintf(name, "RatStdMsg%d", numStdMessages++);
-    Tcl_CreateCommand(interp, name, RatMessageCmd, (ClientData) msgPtr, NULL);
-    return name;
+    msgPtr->clientData = (ClientData)stdMsgPtr;
+    stdMsgPtr->envPtr =
+	    mail_fetchstructure_full(stream, msgNo+1, &stdMsgPtr->bodyPtr, NIL);
+    stdMsgPtr->eltPtr = mail_elt(stream, msgNo+1);
+    stdMsgPtr->eltPtr->lockcount++;
+    sprintf(msgPtr->name, "RatStdMsg%d", numStdMessages++);
+    Tcl_CreateObjCommand(interp, msgPtr->name, RatMessageCmd,
+	    (ClientData)msgPtr, NULL);
+    return msgPtr->name;
 }
 
 
@@ -152,7 +140,9 @@ RatStdEasyCopyingOK(MessageInfo *msgPtr, char *prot, char *destination,
     int nameLen;
 
     switch (stdMsgPtr->type) {
-	case RAT_BERKELY:
+	case RAT_MBX:
+	    return 0;
+	case RAT_UNIX:
 	    return !strcasecmp(prot, "file");
 	case RAT_MH:
 	    return !strcasecmp(prot, "mh");
@@ -176,7 +166,7 @@ RatStdEasyCopyingOK(MessageInfo *msgPtr, char *prot, char *destination,
  *
  * RatStdMessageCopy --
  *
- *      Copy a message to aother c-client folder.
+ *      Copy a message to another c-client folder.
  *
  * Results:
  *	A boolean which says if it went OK or not.
@@ -193,6 +183,7 @@ RatStdMessageCopy (Tcl_Interp *interp, MessageInfo *msgPtr, char *destination)
 {
     StdMessageInfo *stdMsgPtr = (StdMessageInfo*)msgPtr->clientData;
     int flagged = stdMsgPtr->eltPtr->flagged;
+    int deleted = stdMsgPtr->eltPtr->deleted;
     char *cPtr, seq[16];
     int r = TCL_ERROR;
 
@@ -200,8 +191,12 @@ RatStdMessageCopy (Tcl_Interp *interp, MessageInfo *msgPtr, char *destination)
     if (flagged) {
 	mail_clearflag(stdMsgPtr->stream, seq, RAT_FLAGGED_STR);
     }
+    if (deleted) {
+	mail_clearflag(stdMsgPtr->stream, seq, RAT_DELETED_STR);
+    }
     switch (stdMsgPtr->type) {
-	case RAT_BERKELY:	/* fallthrough */
+	case RAT_UNIX:	/* fallthrough */
+	case RAT_MBX:	/* fallthrough */
 	case RAT_MH:	/* fallthrough */
 	case RAT_POP:	/* fallthrough */
 	    if (T == mail_copy_full(stdMsgPtr->stream, seq, destination, 0)) {
@@ -218,6 +213,9 @@ RatStdMessageCopy (Tcl_Interp *interp, MessageInfo *msgPtr, char *destination)
     if (flagged) {
 	mail_setflag(stdMsgPtr->stream, seq, RAT_FLAGGED_STR);
     }
+    if (deleted) {
+	mail_setflag(stdMsgPtr->stream, seq, RAT_DELETED_STR);
+    }
     return r;
 }
 
@@ -246,11 +244,12 @@ Std_GetHeadersProc(Tcl_Interp *interp, MessageInfo *msgPtr)
 	length -= 2;
     }
 
-    if (length+32 > headerSize) {
-	headerSize = length+32;
-	header = (char*)REALLOC(header, headerSize);
+    if (length+64 > headerSize) {
+	headerSize = length+64;
+	header = (char*)ckrealloc(header, headerSize);
     }
     memmove(header, fetchedHeader, length);
+    header[length] = '\0';
     if (stdMsgPtr->eltPtr->seen) {
 	strcpy(&header[length], "Status: RO\r\n");
 	length += strlen(&header[length]);
@@ -259,8 +258,10 @@ Std_GetHeadersProc(Tcl_Interp *interp, MessageInfo *msgPtr)
 	strcpy(&header[length], "X-Status: A\r\n");
 	length += strlen(&header[length]);
     }
-    strcpy(&header[length], "\r\n");
 
+#ifdef MEM_DEBUG
+    mem_header = header;
+#endif /* MEM_DEBUG */
     return header;
 }
 
@@ -291,13 +292,14 @@ Std_GetEnvelopeProc(Tcl_Interp *interp, MessageInfo *msgPtr)
     } else {
 	adrPtr = stdMsgPtr->envPtr->from;
     }
-    if (!adrPtr) {
-	sprintf(buf, "From unkown");
-    } else if (!strcmp(currentHost, adrPtr->host)) {
-	sprintf(buf, "From %s", adrPtr->mailbox);
-    } else {
+    if (adrPtr && adrPtr->host && !strcmp(currentHost, adrPtr->host)
+	    && strlen(adrPtr->mailbox) < sizeof(buf)-6) {
+	snprintf(buf, sizeof(buf), "From %s", adrPtr->mailbox);
+    } else if (adrPtr && RatAddressSize(adrPtr, 0) < sizeof(buf)-6) {
 	sprintf(buf, "From ");
 	rfc822_address(buf+5, adrPtr);
+    } else {
+	sprintf(buf, "From unkown");
     }
     tm.tm_sec = stdMsgPtr->eltPtr->seconds;
     tm.tm_min = stdMsgPtr->eltPtr->minutes;
@@ -310,9 +312,9 @@ Std_GetEnvelopeProc(Tcl_Interp *interp, MessageInfo *msgPtr)
     tm.tm_isdst = -1;
     date = (int)mktime(&tm);
     tmPtr = gmtime(&date);
-    sprintf(buf + strlen(buf), " %s %s %2d %02d:%02d GMT 19%02d\n",
+    sprintf(buf + strlen(buf), " %s %s %2d %02d:%02d GMT %04d\n",
 	    dayName[tmPtr->tm_wday], monthName[tmPtr->tm_mon],
-	    tmPtr->tm_mday, tmPtr->tm_hour, tmPtr->tm_min, tmPtr->tm_year);
+	    tmPtr->tm_mday, tmPtr->tm_hour, tmPtr->tm_min, tmPtr->tm_year+1900);
     return buf;
 }
 
@@ -334,8 +336,7 @@ Std_CreateBodyProc(Tcl_Interp *interp, MessageInfo *msgPtr)
     StdBodyInfo *stdBodyInfoPtr = (StdBodyInfo*)ckalloc(sizeof(StdBodyInfo));
     msgPtr->bodyInfoPtr = CreateBodyInfo(msgPtr);
 
-    mail_fetchstructure_full(stdMsgPtr->stream, msgPtr->msgNo+1,
-	    &msgPtr->bodyInfoPtr->bodyPtr, NIL);
+    msgPtr->bodyInfoPtr->bodyPtr = stdMsgPtr->bodyPtr;
     msgPtr->bodyInfoPtr->clientData = (ClientData)stdBodyInfoPtr;
     if (TYPEMULTIPART == msgPtr->bodyInfoPtr->bodyPtr->type) {
         stdBodyInfoPtr->section = NULL;
@@ -379,24 +380,6 @@ Std_EnvelopeProc(MessageInfo *msgPtr)
 {
     return ((StdMessageInfo*)msgPtr->clientData)->envPtr;
 }
-
-
-/*
- *----------------------------------------------------------------------
- *
- * Std_SetIndexProc --
- *
- *      See ratFolder.h
- *
- *----------------------------------------------------------------------
- */
-
-void
-Std_SetIndexProc(MessageInfo *msgPtr, int index)
-{
-    msgPtr->msgNo = index;
-}
-
 
 /*
  *----------------------------------------------------------------------
@@ -411,7 +394,10 @@ Std_SetIndexProc(MessageInfo *msgPtr, int index)
 void
 Std_MsgDeleteProc(MessageInfo *msgPtr)
 {
+    RatFolderInfo *infoPtr = msgPtr->folderInfoPtr;
     StdMessageInfo *stdMsgPtr = (StdMessageInfo*)msgPtr->clientData;
+
+    infoPtr->privatePtr[msgPtr->msgNo] = NULL;
     stdMsgPtr->eltPtr->lockcount--;
     ckfree(stdMsgPtr);
 }
@@ -451,7 +437,8 @@ Std_MakeChildrenProc(Tcl_Interp *interp, BodyInfo *bodyInfoPtr)
 	    if (stdBodyInfoPtr->section) {
 		partStdInfoPtr->section = (char*)ckalloc(
 			strlen(stdBodyInfoPtr->section)+8);
-		sprintf(partStdInfoPtr->section, "%s.%d",
+		snprintf(partStdInfoPtr->section,
+			sizeof(partStdInfoPtr->section), "%s.%d",
 			stdBodyInfoPtr->section, index++);
 	    } else {
 		partStdInfoPtr->section = (char*)ckalloc(8);
@@ -505,6 +492,24 @@ Std_BodyDeleteProc(BodyInfo *bodyInfoPtr)
     }
     ckfree(bodyInfoPtr->clientData);
 }
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Std_GetInternalDateProc --
+ *
+ *      See ratFolder.h
+ *
+ *----------------------------------------------------------------------
+ */
+
+MESSAGECACHE*
+Std_GetInternalDateProc(Tcl_Interp *interp, MessageInfo *msgPtr)
+{
+    StdMessageInfo *stdMsgPtr = (StdMessageInfo*)msgPtr->clientData;
+    return stdMsgPtr->eltPtr;
+}
 
 /*
  *----------------------------------------------------------------------
@@ -516,18 +521,33 @@ Std_BodyDeleteProc(BodyInfo *bodyInfoPtr)
  *----------------------------------------------------------------------
  */
 
-char*
+Tcl_Obj*
 Std_GetInfoProc(Tcl_Interp *interp, ClientData clientData,
 	RatFolderInfoType type, int notused)
 {
-    static char buf[32];
+    Tcl_Obj *oPtr = NULL;
     MessageInfo *msgPtr = (MessageInfo*)clientData;
     StdMessageInfo *stdMsgPtr = (StdMessageInfo*)msgPtr->clientData;
     ADDRESS *addressPtr;
     int i, presIndex;
 
+    if (msgPtr->info[type]) {
+	if (type == RAT_FOLDER_INDEX && msgPtr->folderInfoPtr) {
+	    Tcl_GetIntFromObj(interp, msgPtr->info[type], &i);
+	    if (i < msgPtr->folderInfoPtr->visible
+		    && msgPtr->folderInfoPtr->privatePtr[
+		       msgPtr->folderInfoPtr->presentationOrder[i-1]] ==
+		    (ClientData)msgPtr) {
+		return msgPtr->info[type];
+	    }
+	} else {
+	    return msgPtr->info[type];
+	}
+    }
+
     switch (type) {
 	case RAT_FOLDER_SUBJECT:	/* fallthrough */
+	case RAT_FOLDER_CANONSUBJECT:	/* fallthrough */
 	case RAT_FOLDER_NAME:		/* fallthrough */
 	case RAT_FOLDER_MAIL_REAL:	/* fallthrough */
 	case RAT_FOLDER_MAIL:		/* fallthrough */
@@ -535,8 +555,6 @@ Std_GetInfoProc(Tcl_Interp *interp, ClientData clientData,
 	case RAT_FOLDER_MAIL_RECIPIENT:	/* fallthrough */
 	case RAT_FOLDER_SIZE:		/* fallthrough */
 	case RAT_FOLDER_SIZE_F:		/* fallthrough */
-	case RAT_FOLDER_TYPE:		/* fallthrough */
-	case RAT_FOLDER_PARAMETERS:	/* fallthrough */
 	case RAT_FOLDER_DATE_F:		/* fallthrough */
 	case RAT_FOLDER_DATE_N:		/* fallthrough */
 	case RAT_FOLDER_DATE_IMAP4:	/* fallthrough */
@@ -545,10 +563,43 @@ Std_GetInfoProc(Tcl_Interp *interp, ClientData clientData,
 	case RAT_FOLDER_SENDER:		/* fallthrough */
 	case RAT_FOLDER_CC:		/* fallthrough */
 	case RAT_FOLDER_FLAGS:		/* fallthrough */
+	case RAT_FOLDER_UNIXFLAGS:	/* fallthrough */
+	case RAT_FOLDER_MSGID:		/* fallthrough */
+	case RAT_FOLDER_REF:		/* fallthrough */
+	case RAT_FOLDER_THREADING:	/* fallthrough */
 	case RAT_FOLDER_REPLY_TO:
+	    return RatGetMsgInfo(interp, type, msgPtr, stdMsgPtr->envPtr,
+		    NULL, stdMsgPtr->eltPtr, stdMsgPtr->eltPtr->rfc822_size);
+
+	case RAT_FOLDER_PARAMETERS:
+	    if (!stdMsgPtr->bodyPtr) {
+		stdMsgPtr->envPtr = mail_fetchstructure_full(
+			stdMsgPtr->stream, msgPtr->msgNo+1,
+			&stdMsgPtr->bodyPtr, NIL);
+	    }
 	    return RatGetMsgInfo(interp, type, msgPtr, stdMsgPtr->envPtr,
 		    stdMsgPtr->bodyPtr, stdMsgPtr->eltPtr,
 		    stdMsgPtr->eltPtr->rfc822_size);
+	    
+	case RAT_FOLDER_TYPE:
+	    if (stdMsgPtr->envPtr->optional.subtype) {
+		oPtr = Tcl_NewStringObj(
+			body_types[stdMsgPtr->envPtr->optional.type], -1);
+		Tcl_AppendStringsToObj(oPtr, "/",
+				       stdMsgPtr->envPtr->optional.subtype,
+				       NULL);
+	    } else {
+		if (!stdMsgPtr->bodyPtr) {
+		    stdMsgPtr->envPtr = mail_fetchstructure_full(
+			    stdMsgPtr->stream, msgPtr->msgNo+1,
+			    &stdMsgPtr->bodyPtr, NIL);
+		}
+		oPtr =Tcl_NewStringObj(body_types[stdMsgPtr->bodyPtr->type],-1);
+		Tcl_AppendStringsToObj(oPtr, "/",
+				       stdMsgPtr->bodyPtr->subtype, NULL);
+	    }
+	    break;
+
 	case RAT_FOLDER_STATUS:
 	    if (RAT_ISME_UNKOWN == msgPtr->toMe) {
 		msgPtr->toMe = RAT_ISME_NO;
@@ -560,40 +611,100 @@ Std_GetInfoProc(Tcl_Interp *interp, ClientData clientData,
 		    }
 		}
 	    }
-	    i = 0;
+	    oPtr = Tcl_NewStringObj(NULL, 0);
 	    if (!stdMsgPtr->eltPtr->seen) {
-		buf[i++] = 'N';
+		Tcl_AppendToObj(oPtr, "N", 1);
 	    }
 	    if (stdMsgPtr->eltPtr->deleted) {
-		buf[i++] = 'D';
+		Tcl_AppendToObj(oPtr, "D", 1);
 	    }
 	    if (stdMsgPtr->eltPtr->flagged) {
-		buf[i++] = 'F';
+		Tcl_AppendToObj(oPtr, "F", 1);
 	    }
 	    if (stdMsgPtr->eltPtr->answered) {
-		buf[i++] = 'A';
+		Tcl_AppendToObj(oPtr, "A", 1);
 	    }
 	    if (RAT_ISME_YES == msgPtr->toMe) {
-		buf[i++] = '+';
+		Tcl_AppendToObj(oPtr, "+", 1);
 	    } else {
-		buf[i++] = ' ';
+		Tcl_AppendToObj(oPtr, " ", 1);
 	    }
-	    buf[i] = '\0';
-	    return buf;
+	    break;
 	case RAT_FOLDER_INDEX:
 	    if (msgPtr->folderInfoPtr) {
 		for (i=0; i< msgPtr->folderInfoPtr->number; i++) {
 		    presIndex = msgPtr->folderInfoPtr->presentationOrder[i];
-		    if (msgPtr->folderInfoPtr->msgCmdPtr[presIndex] ==
-			    msgPtr->name) {
-			sprintf(buf, "%d", i+1);
-			return buf;
+		    if (msgPtr->folderInfoPtr->privatePtr[presIndex] ==
+			    (ClientData)msgPtr){
+			oPtr = Tcl_NewIntObj(i+1);
+			break;
 		    }
 		}
 	    }
-	    return "1";
+	    break;
 	case RAT_FOLDER_END:
 	    break;
     }
-    return NULL;
+    if (!oPtr) {
+	oPtr = Tcl_NewObj();
+    }
+    msgPtr->info[type] = oPtr;
+    Tcl_IncrRefCount(oPtr);
+    return oPtr;
 }
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * RatStdMsgStructInit --
+ *
+ *      Initializes the client data part of the message info structures
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	More data is allocated
+ *
+ *
+ *----------------------------------------------------------------------
+ */
+void
+RatStdMsgStructInit(RatFolderInfoPtr infoPtr, Tcl_Interp *interp, int index,
+	MAILSTREAM *stream, RatStdFolderType type, char *host, char *user)
+{
+    StdMessageInfo *stdMsgPtr;
+    int i, start, end;
+    char seq[32];
+
+    if (-1 == index) {
+       start = 0;
+       end = infoPtr->number;
+       sprintf(seq, "%d:%d", 1, end);
+    } else {
+       start = index;
+       end = start+1;
+       sprintf(seq, "%d", end);
+    }
+    for (i=start; i<end; i++) {
+	stdMsgPtr = (StdMessageInfo*)ckalloc(sizeof(StdMessageInfo));
+	stdMsgPtr->stream = stream;
+	stdMsgPtr->eltPtr = mail_elt(stream, i+1);
+	stdMsgPtr->envPtr = mail_fetch_structure(stream, i+1, NIL, NIL);
+	stdMsgPtr->bodyPtr = NULL;
+	stdMsgPtr->type = type;
+	stdMsgPtr->host = host;
+	stdMsgPtr->user = user;
+	((MessageInfo*)infoPtr->privatePtr[i])->clientData =
+		(ClientData)stdMsgPtr;
+    }
+}
+#ifdef MEM_DEBUG
+void ratStdMessageCleanup()
+{
+    if (mem_header) {
+	ckfree(mem_header);
+    }
+}
+#endif /* MEM_DEBUG */
