@@ -2,7 +2,7 @@
 #
 # Code which handles aliases.
 #
-#  TkRat software and its included text is Copyright 1996,1997,1998 by
+#  TkRat software and its included text is Copyright 1996-2000 by
 #  Martin Forssn.
 #
 #  The full text of the legal notice is contained in the file called
@@ -24,7 +24,7 @@ set bookMod 0
 # Arguments:
 
 proc Aliases {} {
-    global idCnt t b option defaultFont aliasWindows
+    global idCnt t b option aliasWindows
 
     # Create identifier
     set id al[incr idCnt]
@@ -32,7 +32,7 @@ proc Aliases {} {
     upvar #0 $id hd
 
     # Create toplevel
-    toplevel $w
+    toplevel $w -class TkRat
     wm title $w $t(aliases)
     set hd(w) $w
     set hd(old,default_book) $option(default_book)
@@ -88,13 +88,14 @@ proc Aliases {} {
 	    -bd 1 \
 	    -exportselection false \
 	    -highlightthickness 0 \
-	    -font $defaultFont \
 	    -selectmode extended
     Size $w.l.list aliasList
     set hd(listbox) $w.l.list
     pack $w.l.scroll -side right -fill y
     pack $w.l.list -expand 1 -fill both
     bind $w.l.list <ButtonRelease-1> "SetAliasesState $id"
+    bind $w.l.list <KeyRelease> "SetAliasesState $id"
+    bind $w.l.list <Double-1> "AliasDetail $id edit \$${id}(current)"
     set b($w.l.list) alias_list
 
     # Buttons
@@ -135,7 +136,7 @@ proc Aliases {} {
     Place $w aliases
     lappend aliasWindows $id
 
-    bind $w <Destroy> AliasSave
+    wm protocol $w WM_DELETE_WINDOW "AliasClose $id"
 
     # Set up traces
     trace variable option(use_system_aliases) w "AliasesUpdateBooklist {}"
@@ -187,7 +188,7 @@ proc AliasesUpdateBooklist {handler args} {
 		    -command "AliasesPopulate $handler"
 	    if $aliasBook(writable,$name) {
 		$hd(movemenu) add command -label $name \
-			-command "AliasMoveTo move $handler $name"
+			-command [list AliasMoveTo move $handler $name]
 		$hd(defaultmenu) add radiobutton -label $name \
 			-variable option(default_book) -value $name \
 			-command {set bookMod 1}
@@ -220,7 +221,6 @@ proc AliasesPopulate {{handler {}}} {
 	    lappend old $sel
 	}
 	RatAlias list alias
-	set hd(aliasIds) [lsort [array names alias]]
 	set top [lindex [$hd(listbox) yview] 0]
 	$hd(listbox) delete 0 end
 	set hd(aliasIds) ""
@@ -256,11 +256,11 @@ proc SetAliasesState {handler} {
     upvar #0 $handler hd
     global aliasBook
 
-    set onlywr 1
+    set writable 1
     foreach selected [$hd(listbox) curselection] {
 	set a [RatAlias get [lindex $hd(aliasIds) $selected]]
 	if !$aliasBook(writable,[lindex $a 0]) {
-	    set onlywr 0
+	    set writable 0
 	    break
 	}
     }
@@ -268,16 +268,18 @@ proc SetAliasesState {handler} {
     set moveState disabled
     set deleteState disabled
     set l [llength [$hd(listbox) curselection]]
-    if {$l == 1 && $onlywr} {
-	set editState normal
-	set hd(current) [lindex $hd(aliasIds) [$hd(listbox) curselection]]
-    }
-
-    if {$l > 0} {
-	if $onlywr {
-	    set moveState normal
+    set hd(current) {}
+    if $writable {
+	if {$l == 1} {
+	    set hd(current) [lindex $hd(aliasIds) [$hd(listbox) curselection]]
+	    set editState normal
 	    set deleteState normal
+	    set moveState normal
+	} elseif {$l > 0} {
+	    set deleteState normal
+	    set moveState normal
 	}
+	
     }
     $hd(w).b.edit configure -state $editState
     $hd(w).b.move configure -state $moveState
@@ -331,6 +333,7 @@ proc AliasClose {handler} {
     RecordPos $hd(w) aliases
     foreach bn [array names b $hd(w)*] {unset b($bn)}
     destroy $hd(w)
+    AliasSave
 }           
 
 # AliasDetail --
@@ -346,6 +349,11 @@ proc AliasDetail {handler mode {template {}}} {
     global idCnt t b aliasBook option aliasDetail
     upvar #0 $handler ahd
 
+    # Sanity check
+    if {"edit" == $mode && "" == $template} {
+	return
+    }
+
     # Check if we have another window active first
     if {[string length $template] && [info exists aliasDetail($template)]} {
 	wm deiconify $aliasDetail($template)
@@ -360,7 +368,7 @@ proc AliasDetail {handler mode {template {}}} {
     set hd(handler) $handler
 
     # Create toplevel
-    toplevel $w
+    toplevel $w -class TkRat
     wm title $w $t(alias)
     set hd(w) $w
 
@@ -410,6 +418,7 @@ proc AliasDetail {handler mode {template {}}} {
     grid columnconfigure $w 1 -weight 1
     grid rowconfigure $w 3 -weight 1
 
+    bind $w.alias_entry <space> {bell; break}
     bindtags $w.content_text [list Text $w.content_text . all]
     bind $w.content_text <Key-Return> {break}
     bindtags $w.comment_text [list Text $w.comment_text . all]
@@ -432,6 +441,7 @@ proc AliasDetail {handler mode {template {}}} {
     }
     set hd(old,alias) $hd(alias)
 
+    set hd(oldfocus) [focus]
     focus $w.alias_entry
 
     Place $w aliasDetail
@@ -450,11 +460,15 @@ proc AliasDetailDone {handler action} {
     upvar #0 $handler hd
 
     if {1 == $action} {
+	if {[regexp " |\t" $hd(alias)]} {
+	    Popup $t(alias_may_only_contain_chars) $hd(w)
+	    return
+	}
 	set hd(content) [string trim [$hd(content_text) get 1.0 end]]
 	set hd(comment) [string trim [$hd(comment_text) get 1.0 end]]
 	if { 0 == [string length $hd(alias)] || \
 		0 == [string length $hd(content)]} {
-	    Popup $t(need_alias_and_content)
+	    Popup $t(need_alias_and_content) $hd(w)
 	    return
 	}
 	if {"edit" == $hd(mode)} {
@@ -462,7 +476,7 @@ proc AliasDetailDone {handler action} {
 	    set aliasBook(changed,$hd(old,book)) 1
 	}
 	RatAlias add $hd(book) $hd(alias) $hd(fullname) $hd(content) \
-		$hd(comment)
+		$hd(comment) {}
 	set aliasMod 1
 	set aliasBook(changed,$hd(book)) 1
 	AliasesPopulate
@@ -470,6 +484,7 @@ proc AliasDetailDone {handler action} {
     RecordPos $hd(w) aliasDetail
     RecordSize $hd(content_text) aliasText
     foreach bn [array names b $hd(w)*] {unset b($bn)}
+    catch {focus $hd(oldfocus)}
     destroy $hd(w)
     if [info exists hd(template)] {
 	unset aliasDetail($hd(template))
@@ -513,21 +528,26 @@ proc AddrbookImport {format} {
     upvar #0 $id hd
 
     # Create toplevel
-    toplevel $w
+    toplevel $w -bd 5 -class TkRat
     wm title $w $t(import)
     set hd(w) $w
     set hd(format) $format
 
-    label $w.label -text $t(name):
-    entry $w.entry -textvariable ${id}(name) -width 20
-    set b($w.entry) name_of_adrbook
-    grid $w.label -sticky e
-    grid $w.entry -row 0 -column 1 -sticky ew -pady 5
+    label $w.n_label -text $t(name): -anchor e
+    entry $w.n_entry -textvariable ${id}(name)
+    set b($w.n_entry) name_of_adrbook
+    grid $w.n_label $w.n_entry -sticky ew
 
-    frame $w.chooser
-    set hd(fh) [FileSelectorCreate $w.chooser ${id}(fname) \
-	    "AddrbookImportDone $id 1" 0 1]
-    grid $w.chooser - -sticky nsew -pady 5 -padx 5
+    label $w.f_label -text $t(filename): -anchor e
+    entry $w.f_entry -textvariable ${id}(file) -width 40
+    set b($w.f_entry) name_of_adrbook_file
+    grid $w.f_label $w.f_entry -sticky ew
+
+    button $w.browse -text $t(browse) -command \
+	    "set ${id}(file) \[rat_fbox::run -parent $w -title $t(import) \
+	     -ok $t(ok) -mode open\]"
+    set b($w.browse) file_browse
+    grid x $w.browse -sticky e
 
     grid columnconfigure $w 1 -weight 1
     grid rowconfigure $w 1 -weight 1
@@ -539,7 +559,8 @@ proc AddrbookImport {format} {
 
     Place $w addrbookImport
     update
-    focus $w.entry
+    set hd(oldfocus) [focus]
+    focus $w.n_entry
 }
 
 # AddrbookImportDone
@@ -556,32 +577,32 @@ proc AddrbookImportDone {handler action} {
 
     if $action {
 	if ![string length $hd(name)] {
-	    Popup $t(need_name)
+	    Popup $t(need_name) $hd(w)
 	    return
 	}
-	if [info exists aliasBook(writable,$hd(name)] {
-	    Popup $t(book_already_exists)
+	if [info exists aliasBook(writable,$hd(name))] {
+	    Popup $t(book_already_exists) $hd(w)
 	    return
 	}
-	FileSelectorCleanup $hd(fh)
-	if {![file isfile $hd(fname)] || ![file readable $hd(fname)]} {
-	    Popup "$t(illegal_file_spec): $hd(fname)"
+	if {![file isfile $hd(file)] || ![file readable $hd(file)]} {
+	    Popup "$t(illegal_file_spec): $hd(file)" $hd(w)
 	    return
 	}
 	set aliasBook(writable,$hd(name)) 0
 	set aliasBook(changed,$hd(name)) 0
 	set bookMod 1
-	lappend option(addrbooks) [list $hd(name) $hd(format) $hd(fname)]
+	lappend option(addrbooks) [list $hd(name) $hd(format) $hd(file)]
 	switch $hd(format) {
-	    mail {ReadMailAliases $hd(fname) $hd(name)}
-	    elm  {ReadElmAliases $hd(fname) $hd(name)}
-	    pine {ReadPineAliases $hd(fname) $hd(name)}
+	    mail {ReadMailAliases $hd(file) $hd(name)}
+	    elm  {ReadElmAliases $hd(file) $hd(name)}
+	    pine {ReadPineAliases $hd(file) $hd(name)}
 	}
 	AliasesPopulate
     }
 
     RecordPos $hd(w) addrbookAdd
     foreach bn [array names b $hd(w)*] {unset b($bn)}
+    catch {focus $hd(oldfocus)}
     destroy $hd(w)
     unset hd
 }
@@ -601,20 +622,25 @@ proc AddrbookAdd {} {
     upvar #0 $id hd
 
     # Create toplevel
-    toplevel $w
+    toplevel $w -bd 5 -class TkRat
     wm title $w $t(new)
     set hd(w) $w
 
-    label $w.label -text $t(name):
-    entry $w.entry -textvariable ${id}(name) -width 20
-    set b($w.entry) name_of_adrbook
-    grid $w.label -sticky e
-    grid $w.entry -row 0 -column 1 -sticky ew -pady 5
+    label $w.n_label -text $t(name): -anchor e
+    entry $w.n_entry -textvariable ${id}(name)
+    set b($w.n_entry) name_of_adrbook
+    grid $w.n_label $w.n_entry -sticky ew
 
-    frame $w.chooser
-    set hd(fh) [FileSelectorCreate $w.chooser ${id}(fname) \
-	    "AddrbookAddDone $id 1" 0 0]
-    grid $w.chooser - -sticky nsew -pady 5 -padx 5
+    label $w.f_label -text $t(filename): -anchor e
+    entry $w.f_entry -textvariable ${id}(file) -width 40
+    set b($w.f_entry) name_of_adrbook_file
+    grid $w.f_label $w.f_entry -sticky ew
+
+    button $w.browse -text $t(browse) -command \
+	    "set ${id}(file) \[rat_fbox::run -parent $w -title $t(new) \
+	     -ok $t(save) -mode save\]"
+    set b($w.browse) file_browse
+    grid x $w.browse -sticky e
 
     grid columnconfigure $w 1 -weight 1
     grid rowconfigure $w 1 -weight 1
@@ -626,7 +652,8 @@ proc AddrbookAdd {} {
 
     Place $w addrbookAdd
     update
-    focus $w.entry
+    set hd(oldfocus) [focus]
+    focus $w.n_entry
 }
 
 # AddrbookAddDone
@@ -643,23 +670,22 @@ proc AddrbookAddDone {handler action} {
 
     if $action {
 	if ![string length $hd(name)] {
-	    Popup $t(need_name)
+	    Popup $t(need_name) $hd(w)
 	    return
 	}
-	if [info exists aliasBook(writable,$hd(name)] {
-	    Popup $t(book_already_exists)
+	if [info exists aliasBook(writable,$hd(name))] {
+	    Popup $t(book_already_exists) $hd(w)
 	    return
 	}
-	FileSelectorCleanup $hd(fh)
-	set dir [file dirname $hd(fname)]
-	if {([file exists $hd(fname)] && ![file readable $hd(fname)])
-		|| [file isdirectory $hd(fname)]
-		|| (![file exists $hd(fname)] && ![file writable $dir])} {
-	    Popup "$t(illegal_file_spec): $hd(fname)"
+	set dir [file dirname $hd(file)]
+	if {([file exists $hd(file)] && ![file readable $hd(file)])
+		|| [file isdirectory $hd(file)]
+		|| (![file exists $hd(file)] && ![file writable $dir])} {
+	    Popup "$t(illegal_file_spec): $hd(file)" $hd(w)
 	    return
 	}
-	if {([file isfile $hd(fname)] && [file writable $hd(fname)])
-		|| (![file exists $hd(fname)] && [file isdirectory $dir]
+	if {([file isfile $hd(file)] && [file writable $hd(file)])
+		|| (![file exists $hd(file)] && [file isdirectory $dir]
 		    && [file writable $dir])} {
 	    set aliasBook(writable,$hd(name)) 1
 	} else {
@@ -667,14 +693,15 @@ proc AddrbookAddDone {handler action} {
 	}
 	set aliasBook(changed,$hd(name)) 0
 	set bookMod 1
-	lappend option(addrbooks) [list $hd(name) tkrat $hd(fname)]
-	if [file exists $hd(fname)] {
-	    RatAlias read $hd(fname)
+	lappend option(addrbooks) [list $hd(name) tkrat $hd(file)]
+	if [file exists $hd(file)] {
+	    RatAlias read $hd(file)
 	}
     }
 
     RecordPos $hd(w) addrbookAdd
     foreach bn [array names b $hd(w)*] {unset b($bn)}
+    catch {focus $hd(oldfocus)}
     destroy $hd(w)
     unset hd
 }
@@ -686,7 +713,7 @@ proc AddrbookAddDone {handler action} {
 # Arguments:
 
 proc AddrbookDelete {} {
-    global idCnt t b option defaultFont
+    global idCnt t b option
 
     # Create identifier
     set id al[incr idCnt]
@@ -694,7 +721,7 @@ proc AddrbookDelete {} {
     upvar #0 $id hd
 
     # Create toplevel
-    toplevel $w
+    toplevel $w -class TkRat
     wm title $w $t(delete)
     set hd(w) $w
 
@@ -709,7 +736,6 @@ proc AddrbookDelete {} {
 	    -bd 1 \
 	    -exportselection false \
 	    -highlightthickness 0 \
-	    -font $defaultFont \
 	    -selectmode extended
     Size $w.l.list bookList
     set hd(listbox) $w.l.list
@@ -770,7 +796,7 @@ proc AddrbookDeleteDone {handler action} {
 		}
 	    }
 	    if ![string length $newDefault] {
-		Popup $t(need_writable_book)
+		Popup $t(need_writable_book) $hd(w)
 		return
 	    }
 	    set option(default_book) $newDefault
@@ -852,7 +878,7 @@ proc AliasExtract {handler} {
 
     # Check that we found something
     if ![llength $adrlist] {
-	Popup $t(could_not_find_adr)
+	Popup $t(could_not_find_adr) $fh(toplevel)
 	return
     }
 
@@ -869,7 +895,7 @@ proc AliasExtract {handler} {
     upvar #0 $id hd
 
     # Create toplevel
-    toplevel $w
+    toplevel $w -class TkRat
     wm title $w $t(extract_adr)
     set hd(w) $w
 
@@ -896,11 +922,14 @@ proc AliasExtract {handler} {
     # Create frame with aliases to add
     frame $w.f
     label $w.f.use -text $t(use)
-    label $w.f.name -text $t(alias)
-    label $w.f.fname -text $t(fullname)
-    label $w.f.content -text $t(content)
-    label $w.f.comment -text $t(comment)
+    label $w.f.name -text $t(alias) -width 8
+    label $w.f.fname -text $t(fullname) -width 20
+    label $w.f.content -text $t(content) -width 35
+    label $w.f.comment -text $t(comment) -width 30
     grid $w.f.use $w.f.name $w.f.fname $w.f.content $w.f.comment -sticky w
+    canvas $w.f.c
+    frame $w.f.c.f
+    set f $w.f.c.f
     set totlist ""
     foreach a $adrlist {
 	if [string length $totlist] {
@@ -930,61 +959,86 @@ proc AliasExtract {handler} {
 	set hd($idCnt,name) $name
 	set hd($idCnt,fname) [$a get name]
 	set hd($idCnt,content) [$a get mail]
-	checkbutton $w.f.c$idCnt -variable ${id}($idCnt,use)
-	entry $w.f.en$idCnt -textvariable ${id}($idCnt,name) -width 8
-	entry $w.f.ef$idCnt -textvariable ${id}($idCnt,fname) -width 20
-	entry $w.f.ec$idCnt -textvariable ${id}($idCnt,content) -width 35
-	entry $w.f.ek$idCnt -textvariable ${id}($idCnt,comment) -width 30
-	set b($w.f.c$idCnt) aliasadd_use
-	set b($w.f.en$idCnt) alias_alias
-	set b($w.f.ef$idCnt) alias_fullname
-	set b($w.f.ec$idCnt) alias_content
-	set b($w.f.ek$idCnt) alias_comment
-	grid $w.f.c$idCnt \
-	     $w.f.en$idCnt \
-	     $w.f.ef$idCnt \
-	     $w.f.ec$idCnt \
-	     $w.f.ek$idCnt -sticky we
+	checkbutton $f.c$idCnt -variable ${id}($idCnt,use)
+	entry $f.en$idCnt -textvariable ${id}($idCnt,name) -width 8
+	entry $f.ef$idCnt -textvariable ${id}($idCnt,fname) -width 20
+	entry $f.ec$idCnt -textvariable ${id}($idCnt,content) -width 35
+	entry $f.ek$idCnt -textvariable ${id}($idCnt,comment) -width 30
+	set b($f.c$idCnt) aliasadd_use
+	set b($f.en$idCnt) alias_alias
+	set b($f.ef$idCnt) alias_fullname
+	set b($f.ec$idCnt) alias_content
+	set b($f.ek$idCnt) alias_comment
+	grid $f.c$idCnt \
+	     $f.en$idCnt \
+	     $f.ef$idCnt \
+	     $f.ec$idCnt \
+	     $f.ek$idCnt -sticky we
+	set idw $idCnt
     }
-    set height [llength $adrlist]
-    if {$height > 1} {
+    if ![info exists idw] {
+	Popup $t(could_not_find_adr) $fh(toplevel)
+	destroy $w
+	unset hd
+	return
+    }
+    set num [llength $adrlist]
+    if {$num > 1} {
 	incr idCnt
 	set hd($idCnt,use) 0
 	set hd($idCnt,content) $totlist
-	checkbutton $w.f.c$idCnt -variable ${id}($idCnt,use)
-	entry $w.f.en$idCnt -textvariable ${id}($idCnt,name) -width 8
-	entry $w.f.ef$idCnt -textvariable ${id}($idCnt,fname) -width 20
-	if {$height > 10} {
-	    frame $w.f.ec$idCnt
-	    scrollbar $w.f.ec$idCnt.scroll -relief sunken \
-		    -command "$w.f.ec$idCnt.text yview" -highlightthickness 0
-	    text $w.f.ec$idCnt.text -width 35 -height 10 -wrap none \
-		    -yscroll "$w.f.ec$idCnt.scroll set"
-	    pack $w.f.ec$idCnt.scroll -side right -fill y
-	    pack $w.f.ec$idCnt.text -expand yes -fill both
-	    set hd(listcmd) $w.f.ec$idCnt.text
+	checkbutton $f.c$idCnt -variable ${id}($idCnt,use)
+	entry $f.en$idCnt -textvariable ${id}($idCnt,name) -width 8
+	entry $f.ef$idCnt -textvariable ${id}($idCnt,fname) -width 20
+	if {$num > 10} {
+	    frame $f.ec$idCnt
+	    scrollbar $f.ec$idCnt.scroll -relief sunken \
+		    -command "$f.ec$idCnt.text yview" -highlightthickness 0
+	    text $f.ec$idCnt.text -width 35 -height 10 -wrap none \
+		    -yscroll "$f.ec$idCnt.scroll set"
+	    pack $f.ec$idCnt.scroll -side right -fill y 
+	    pack $f.ec$idCnt.text -expand yes -fill both
+	    set hd(listcmd) $f.ec$idCnt.text
 	} else {
-	    text $w.f.ec$idCnt -width 35 -height $height -wrap none
-	    set hd(listcmd) $w.f.ec$idCnt
+	    text $f.ec$idCnt -width 35 -height $num -wrap none
+	    set hd(listcmd) $f.ec$idCnt
 	}
 	$hd(listcmd) insert 1.0 $totlist
 	set hd(listvar) $idCnt,content
-	entry $w.f.ek$idCnt -textvariable ${id}($idCnt,comment) -width 30
-	set b($w.f.c$idCnt) aliasadd_use
-	set b($w.f.en$idCnt) alias_alias
-	set b($w.f.ef$idCnt) alias_fullname
-	set b($w.f.ec$idCnt) alias_content
-	set b($w.f.ek$idCnt) alias_comment
-	grid $w.f.c$idCnt \
-	     $w.f.en$idCnt \
-	     $w.f.ef$idCnt \
-	     $w.f.ec$idCnt \
-	     $w.f.ek$idCnt -sticky wen
+	entry $f.ek$idCnt -textvariable ${id}($idCnt,comment) -width 30
+	set b($f.c$idCnt) aliasadd_use
+	set b($f.en$idCnt) alias_alias
+	set b($f.ef$idCnt) alias_fullname
+	set b($f.ec$idCnt) alias_content
+	set b($f.ek$idCnt) alias_comment
+	grid $f.c$idCnt \
+	     $f.en$idCnt \
+	     $f.ef$idCnt \
+	     $f.ec$idCnt \
+	     $f.ek$idCnt -sticky wen
     }
-    grid columnconfigure $w.f 1 -weight 1
-    grid columnconfigure $w.f 2 -weight 1
-    grid columnconfigure $w.f 3 -weight 1
-    grid columnconfigure $w.f 4 -weight 1
+    grid columnconfigure $f 1 -weight 1
+    grid columnconfigure $f 2 -weight 1
+    grid columnconfigure $f 3 -weight 1
+    grid columnconfigure $f 4 -weight 1
+    set wid [$w.f.c create window 0 0 -anchor nw -window $f]
+    update idletasks
+    set bbox [$w.f.c bbox $wid]
+    $w.f.c configure -scrollregion $bbox
+    set height [lindex $bbox 3]
+    set width [lindex $bbox 2]
+    set maxheight [expr [winfo screenheight $w] - 200]
+    if {$height > $maxheight} {
+	scrollbar $w.f.s -command "$w.f.c yview" \
+		  -highlightthickness 0 -bd 1
+	$w.f.c configure -yscrollcommand "$w.f.s set"
+	set height $maxheight
+	set scroll $w.f.s
+    } else {
+	set scroll -
+    }
+    grid $w.f.c - - - - $scroll -sticky ns
+    $w.f.c configure -width $width -height $height
     pack $w.f -side top -fill both
 
     # Create buttons
@@ -1004,7 +1058,7 @@ proc AliasExtract {handler} {
 
 proc AliasExtractDone {handler action} {
     upvar #0 $handler hd
-    global aliasMod t b
+    global aliasMod t b aliasBook
 
     # Find which entries we should use
     set ids {}
@@ -1021,12 +1075,13 @@ proc AliasExtractDone {handler action} {
 	# Add the aliases
 	foreach id $ids {
 	    if ![string length $hd($id,name)] {
-		Popup $t(missing_alias_name)
+		Popup $t(missing_alias_name) $fh(toplevel)
 		continue
 	    }
 	    RatAlias add $hd(book) $hd($id,name) $hd($id,fname) \
-		    $hd($id,content) $hd($id,comment)
+		    $hd($id,content) $hd($id,comment) {}
 	    set aliasMod 1
+	    set aliasBook(changed,$hd(book)) 1
 	}
 	if { 1 == $aliasMod } {
 	    AliasesPopulate
@@ -1049,7 +1104,7 @@ proc AliasExtractDone {handler action} {
 # master    - The text widget that is to be master for this window
 
 proc AliasChooser {master} {
-    global idCnt t b defaultFont
+    global idCnt t b
 
     # Create identifier
     set id al[incr idCnt]
@@ -1057,9 +1112,9 @@ proc AliasChooser {master} {
     upvar #0 $id hd
 
     # Create toplevel
-    toplevel $w
-    wm transient $w [winfo toplevel $master]
+    toplevel $w -class TkRat
     wm title $w $t(alias_chooser)
+    wm transient $w [winfo toplevel $master]
     set hd(w) $w
     set hd(search) ""
 
@@ -1081,7 +1136,6 @@ proc AliasChooser {master} {
 	    -bd 1 \
 	    -exportselection false \
 	    -highlightthickness 0 \
-	    -font $defaultFont \
 	    -selectmode single
     set b($w.list) alias_chooser
     Size $w.list aliasChooser
@@ -1098,6 +1152,7 @@ proc AliasChooser {master} {
     bind $w <Key-Up> "AliasChooserMoveSel $id up"
     bind $w <Key-Down> "AliasChooserMoveSel $id down"
     bind $w <Key> "AliasChooserSearch $id %A"
+    wm protocol $w WM_DELETE_WINDOW "set ${id}(done) 0"
 
     # Populate list
     RatAlias list alias
@@ -1107,10 +1162,7 @@ proc AliasChooser {master} {
     }
     $hd(list) selection set 0
 
-    # Set focus and grab
-    set hd(oldFocus) [focus]
-    grab $w
-    focus $w
+    ModalGrab $w
 
     # Wait for action
     tkwait variable ${id}(done)
@@ -1122,7 +1174,6 @@ proc AliasChooser {master} {
     }
     unset b($w.list)
     destroy $w
-    focus $hd(oldFocus)
     unset hd
     return $ret
 }
@@ -1142,7 +1193,7 @@ proc AliasChooserMoveSel {handler direction} {
     $hd(list) selection clear $cur
 
     if [string compare up $direction] {
-	if {[incr cur] > [$hd(list) size]} {
+	if {[incr cur] >= [$hd(list) size]} {
 	    incr cur -1
 	}
 	if {[expr $cur/[$hd(list) size].0] >= [lindex [$hd(list) yview] 1]} {
@@ -1235,7 +1286,7 @@ proc ReadElmAliases {file book} {
 	    RatAlias add $book \
 			 [string trim [lindex $a 0]] \
 			 [string trim [lindex $a 1]] \
-			 [string trim [lindex $a 2]] {}
+			 [string trim [lindex $a 2]] {} {}
 	    incr n
 	}
     }
@@ -1256,18 +1307,16 @@ proc ReadMailAliases {file book} {
     set n 0
     set fh [open $file r]
     while { -1 != [gets $fh line]} {
-	if [string match {alias *} $line] {
-	    regsub -all {[ 	][ 	]} $line { } line
-	    set a [split $line]
-	    set content ""
-	    foreach e [lrange $a 2 end] {
-		set content "$content [string trim $e]"
+	while { 1 == [regexp {\\$} $line]} {
+	    if {-1 == [gets $fh cont]} {
+		break
 	    }
-	    if [string length [string trim [lindex $a 1]]] {
-		RatAlias add $book \
-			     [string trim [lindex $a 1]] \
-			     [string trim [lindex $a 1]] \
-			     [string trim $content] {}
+	    set line [join [list [string trimright $line \\] $cont] ""]
+	}
+	if [string match {alias *} $line] {
+	    if {[regexp "^alias\[ \t\]+(\[a-zA-Z0-9_-\]+)\[ \t\]+(.+)$" $line \
+		    {} name content]} {
+		RatAlias add $book $name $name $content {} {}
 		incr n
 	    }
 	}
@@ -1285,10 +1334,13 @@ proc ReadMailAliases {file book} {
 # book -	Address book to insert them into
  
 proc ReadPineAliases {file book} {
-    set fh [open $file r]
+    if [catch {open $file r} fh] {
+	Popup $fh
+	return
+    }
     set aliases {}
     while { -1 != [gets $fh line]} {
-        if [regsub {^ } $line "" cont] {
+        if {[regsub {^ } $line "" cont]} {
             set aliases [lreplace $aliases end end \
 		    "[lindex $aliases end] $cont"]
         } else {
@@ -1299,7 +1351,7 @@ proc ReadPineAliases {file book} {
 
     set n 0
     foreach a $aliases {
-        if [regexp {^#DELETED} $a] {
+        if {[regexp {^#DELETED} $a]} {
             continue
         }
         set sa [split $a "\t"]
@@ -1307,8 +1359,22 @@ proc ReadPineAliases {file book} {
 	    incr n
 	    set content [lindex $sa 2]
 	    regexp {^\((.+)\)$} $content notUsed content
-	    RatAlias add $book [lindex $sa 0] [lindex $sa 1] $content {}
+	    RatAlias add $book [lindex $sa 0] [lindex $sa 1] $content {} {}
 	}
     }
     return $n
 }
+
+# AliasWeAreQuitting --
+#
+# Save aliases if needed since tkrat is quitting
+#
+# Arguments:
+
+proc AliasWeAreQuitting {} {
+    global aliasWindows
+
+    if [llength $aliasWindows] {
+	AliasSave
+    }
+}
