@@ -3,8 +3,8 @@
 # This file contains code which shows the DSN window etc.
 #
 #
-#  TkRat software and its included text is Copyright 1996,1997,1998
-#  by Martin Forssn
+#  TkRat software and its included text is Copyright 1996-2000 by
+#  Martin Forssn
 #
 #  The full text of the legal notice is contained in the file called
 #  COPYRIGHT, included with this distribution.
@@ -27,7 +27,7 @@ proc ShowDSNList {} {
     set w .$id
 
     # Create toplevel
-    toplevel $w
+    toplevel $w -class TkRat
     wm title $w $t(notifications)
 
     # Populate window
@@ -55,6 +55,7 @@ proc ShowDSNList {} {
     grid columnconfigure $w 0 -weight 1
     grid rowconfigure $w 0 -weight 1
     set b($w.text) notification_window
+    wm protocol $w WM_DELETE_WINDOW "DestroyDSN $w.text"
 
     UpdateDSNText $w.text
     $w.text see end
@@ -87,16 +88,16 @@ proc DestroyDSN {textCmd} {
 # text -	The text widget to update
 
 proc UpdateDSNText {text} {
-    global t option langCharset
+    global t option fixedNormFont fixedBoldFont
     set top [lindex [$text yview] 0]
     set id 0
 
     $text configure -state normal
     $text delete 0.0 end
     $text tag delete [$text tag names]
-    $text tag configure Normal -font [GetFont $langCharset 0 {}]
-    $text tag configure Bold -font [GetFont $langCharset 0 bold]
-    $text tag configure Failed -font [GetFont $langCharset 0 bold]
+    $text tag configure Normal -font $fixedNormFont
+    $text tag configure Bold -font $fixedBoldFont
+    $text tag configure Failed -font $fixedBoldFont
     if { 4 < [winfo cells $text]} {
 	$text tag configure Failed -foreground red
 	set mark "-background #43ce80 -relief raised -borderwidth 1"
@@ -111,9 +112,7 @@ proc UpdateDSNText {text} {
 	set args [clock format [lindex $d 1] -format "%Y %m %e %H %M %S"]
 	regsub -all { 0}  $args { } args
 	$text insert end " [eval "RatFormatDate $args"]  " Normal
-	foreach part [lindex $d 2] {
-	    $text insert end [lindex $part 1] Normal
-	}
+	$text insert end [lindex $d 2] Normal
 	$text insert end "\n"
 	foreach r [lindex $d 3] {
 	    regsub -all {\([^)]+\)} [lindex $r 0] {} action
@@ -158,7 +157,6 @@ proc UpdateDSNText {text} {
 proc RatDSNRecieve {subject action recipient id} {
     global dsnCmds option t
 
-
     regsub -all {\([^)]+\)} $action {} action
     set action [string tolower [string trim $action]]
     foreach v $option(dsn_verbose) {
@@ -188,11 +186,9 @@ proc RatDSNRecieve {subject action recipient id} {
 # recipient - Recipient we are interestred in
 
 proc DSNShow {dsnID recipient} {
-    global idCnt t b option langCharset
+    global idCnt t b option fixedNormFont fixedBoldFont
 
     # Get info
-    set boldFont [GetFont $langCharset 0 bold]
-    set normalFont [GetFont $langCharset 0 {}]
     set msg [RatDSNGet msg $dsnID]
     set report [RatDSNGet report $dsnID $recipient]
 
@@ -201,7 +197,7 @@ proc DSNShow {dsnID recipient} {
     set w .$id
 
     # Create toplevel
-    toplevel $w
+    toplevel $w -class TkRat
     wm title $w $t(notification)
 
     # The human readable part
@@ -214,8 +210,7 @@ proc DSNShow {dsnID recipient} {
 	    -relief sunken \
 	    -wrap none \
 	    -yscroll "$w.h.scroll set" \
-	    -setgrid true \
-	    -font [GetFont $langCharset 0 {}]
+	    -setgrid true
     scrollbar $w.h.scroll \
 	    -relief sunken \
 	    -command "$w.h.text yview" \
@@ -253,8 +248,8 @@ proc DSNShow {dsnID recipient} {
 	} else {
 	    set key [lindex $r 0]
 	}
-	label $w.i.lk$row -text $key: -font $boldFont
-	label $w.i.lv$row -text [lindex $r 1] -font $normalFont
+	label $w.i.lk$row -text $key: -font $fixedBoldFont
+	label $w.i.lv$row -text [lindex $r 1] -font $fixedNormFont
 	grid $w.i.lk$row -row $row -column 0 -sticky e
 	grid $w.i.lv$row -row $row -column 1 -sticky w
 	incr row
@@ -278,6 +273,11 @@ proc DSNShow {dsnID recipient} {
     pack $w.but -fill x -pady 5 -padx 5
     set b($w.but.more) dsn_more_info
     set b($w.but.dismiss) dismiss
+    wm protocol $w WM_DELETE_WINDOW "\
+		      unset b($w.h.text); \
+		      unset b($w.but.more); \
+		      unset b($w.but.dismiss); \
+		      destroy $w"
 
     Place $w dSNShow
 }
