@@ -3,15 +3,12 @@
 # This file contains code for handling message selection expressions
 #
 #
-#  TkRat software and its included text is Copyright 1996,1997,1998
-#  by Martin Forssn
+#  TkRat software and its included text is Copyright 1996-2000 by
+#  Martin Forssn
 #
 #  The full text of my legal notices is contained in the file called
 #  COPYRIGHT, included with this distribution.
 
-# Counter used to generate uniqe expression indexes for saved expressions
-set expArrayId 0
-
 # ExpCreate --
 #
 # Create the expression creation window
@@ -20,8 +17,8 @@ set expArrayId 0
 # handler	- The handler which identifies the folder window that
 #		  the selection should be done in.
 
-proc ExpCreate {handler} {
-    global t b option idCnt defaultFont
+proc ExpCreate {handler {addproc {}}} {
+    global t b option idCnt
 
     # Create identifier
     set id expWin[incr idCnt]
@@ -31,9 +28,12 @@ proc ExpCreate {handler} {
     set hd(w) $w
     set hd(handler) $handler
     set hd(op) and
+    set hd(exp) {}
+    set hd(oldfocus) [focus]
+    set hd(addproc) $addproc
 
     # Create toplevel
-    toplevel $w
+    toplevel $w -class TkRat
     wm title $w $t(create_exp)
 
     # Populate window
@@ -45,8 +45,7 @@ proc ExpCreate {handler} {
     $w.top.mode.m add command -label $t(advanced_mode) -command "ExpModeAdv $id"
     frame $w.top.c
     checkbutton $w.top.c.but -text $t(save_as): -variable ${id}(doSave)
-    entry $w.top.c.entry -width 20 -textvariable ${id}(saveAs) \
-	    -font $defaultFont
+    entry $w.top.c.entry -width 20 -textvariable ${id}(saveAs)
     bind $w.top.c.entry <KeyRelease> \
 	    "if {0 < \[string length ${id}(saveAs)\]} { \
 		 set ${id}(doSave) 1 \
@@ -65,23 +64,23 @@ proc ExpCreate {handler} {
     set hd(frame) $w.f
 
     frame $w.buttons
-    frame $w.buttons.def -relief sunken -bd 1
-    button $w.buttons.def.ok -text $t(ok) -command "ExpDone $id ok"
-    pack $w.buttons.def.ok -padx 4 -pady 4
+    button $w.buttons.ok -text $t(ok) -default active -command "ExpDone $id 1"
     button $w.buttons.clear -text $t(clear) -command "ExpClear $id"
-    button $w.buttons.cancel -text $t(cancel) -command "ExpDone $id cancel"
-    pack $w.buttons.def \
+    button $w.buttons.cancel -text $t(cancel) -command "ExpDone $id 0"
+    pack $w.buttons.ok \
 	 $w.buttons.clear \
 	 $w.buttons.cancel -side left -expand 1
-    bind $w <Return> "ExpDone $id ok"
+    bind $w <Return> "ExpDone $id 1"
     pack $w.top -side top -fill x -pady 5
     pack $w.f -side top -fill both -pady 5 -expand 1
     pack $w.buttons -side top -fill x -pady 5
-    set b($w.buttons.def.ok) exp_ok
+    set b($w.buttons.ok) exp_ok
     set b($w.buttons.clear) clear
     set b($w.buttons.cancel) dismiss
+    wm protocol $w WM_DELETE_WINDOW "ExpDone $id 0"
 
     set hd(mode) {}
+    set hd(action) create
     if [string compare advanced $option(expression_mode)] {
 	ExpModeBasic $id
     } else {
@@ -90,6 +89,43 @@ proc ExpCreate {handler} {
     Place $w ratExpression
 }
 
+# ExpEdit --
+#
+# Edit an expression
+#
+# Arguments:
+# name	- Name of expression to edit
+
+proc ExpEdit {name namechange} {
+    global t b idCnt expExp
+
+    # Create identifier
+    set id expWin[incr idCnt]
+    set w .$id
+    upvar #0 $id hd
+    set hd(w) $w
+    set hd(handler) $id
+    set hd(mode) {}
+    set hd(frame) $w.f
+    set hd(action) edit
+    set hd(exp) "$expExp($name) "
+    set hd(saveAs) $name
+    set hd(oldName) $name
+
+    # Create toplevel
+    toplevel $w -class TkRat
+    wm title $w $t(edit_exp)
+    frame $w.f
+
+    ExpModeAdv $id
+    OkButtons $w $t(ok) $t(cancel) "ExpDone $id"
+
+    pack $w.buttons -side bottom -pady 5 -fill x
+    pack $w.f
+
+    Place $w ratExpEdit
+}
+
 # ExpModeBasic --
 #
 # Configure window for basic mode
@@ -99,7 +135,7 @@ proc ExpCreate {handler} {
 
 proc ExpModeBasic {handler} {
     upvar #0 $handler hd
-    global t b defaultFont
+    global t b
 
     # Check that this is really neccessary
     if ![string compare $hd(mode) basic] {
@@ -125,9 +161,9 @@ proc ExpModeBasic {handler} {
     set b($w.f.and) exp_basic_and
     set b($w.f.or) exp_basic_or
 
-    foreach f {subject from reply_to sender to cc} {
+    foreach f {subject from reply-to sender to cc} {
 	label $w.l$f -text $t($f):
-	entry $w.e$f -textvariable ${handler}($f) -width 50 -font $defaultFont
+	entry $w.e$f -textvariable ${handler}($f) -width 50
 	grid $w.l$f $w.e$f -sticky e
 	set b($w.l$f) exp_basic_field
 	set b($w.e$f) exp_basic_field
@@ -144,7 +180,7 @@ proc ExpModeBasic {handler} {
 
 proc ExpModeAdv {handler} {
     upvar #0 $handler hd
-    global t b defaultFont
+    global t b
 
     # Check that this is really neccessary
     if ![string compare $hd(mode) advanced] {
@@ -164,7 +200,7 @@ proc ExpModeAdv {handler} {
     # Pack windows
     frame $w.f
     text $w.f.text -relief sunken -bd 1 -width 80 -height 4 -wrap word \
-	    -yscroll "$w.f.scroll set" -setgrid 1 -font $defaultFont
+	    -yscroll "$w.f.scroll set" -setgrid 1
     $w.f.text tag configure error -underline 1
     $w.f.text tag bind error <KeyPress> "$w.f.text tag remove error 1.0 end"
     scrollbar $w.f.scroll -relief sunken -bd 1 -command "$w.f.text yview"
@@ -177,7 +213,7 @@ proc ExpModeAdv {handler} {
     label $w.f1.l -text $t(fields)
     grid $w.f1.l -sticky we -columnspan 2
     set i 1
-    foreach n {to from subject sender cc reply_to size} {
+    foreach n {to from subject sender cc reply-to size} {
 	button $w.f1.f$i -text $t($n) -width 10 \
 		-command "$w.f.text insert insert \"$n \""
 	set b($w.f1.f$i) exp_adv_fields
@@ -230,7 +266,8 @@ proc ExpModeAdv {handler} {
 	 $w.g1 -sticky n
 
     focus $w.f.text
-    bind $w.f.text <Return> "ExpDone $handler ok; break"
+    $w.f.text insert end $hd(exp)
+    bind $w.f.text <Return> "ExpDone $handler 1; break"
     set hd(text) $w.f.text
 }
 
@@ -245,7 +282,7 @@ proc ExpClear {handler} {
     upvar #0 $handler hd
 
     if ![string compare basic $hd(mode)] {
-	foreach f {subject from reply_to sender to cc} {
+	foreach f {subject from reply-to sender to cc} {
 	    set hd($f) ""
 	}
     } else {
@@ -264,18 +301,21 @@ proc ExpClear {handler} {
 proc ExpDone {handler action} {
     upvar #0 $handler hd
     upvar #0 $hd(handler) fHd
-    global t b option expArrayId expList expName expExp
+    global t b option expList expExp
 
-    if ![string compare ok $action] {
+    if ![info exist expList] {
+	set expList {}
+    }
+    if {1 == $action} {
 	# Build expression
 	if ![string compare basic $hd(mode)] {
 	    set exp ""
-	    foreach f {subject from reply_to sender to cc} {
+	    foreach f {subject from reply-to sender to cc} {
 		if [string length $hd($f)] {
 		    if [string length $exp] {
 			set exp "$exp $hd(op) "
 		    }
-		    if [string compare reply_to $f] {
+		    if [string compare reply-to $f] {
 			set exp "${exp}$f has [list $hd($f)]"
 		    } else {
 			set exp "${exp}reply-to has [list $hd($f)]"
@@ -283,38 +323,62 @@ proc ExpDone {handler action} {
 		}
 	    }
 	    if [catch {RatParseExp $exp} expId] {
-		Popup $t(syntax_error_exp)
+		Popup $t(syntax_error_exp) $hd(w)
 		return
 	    }
 	} else {
-	    set exp [$hd(text) get 1.0 end]
+	    set exp [string trim [$hd(text) get 1.0 end]]
 	    if [catch {RatParseExp $exp} expId] {
 	        set i [lindex $expId 0]
 	        $hd(text) tag add error "1.$i wordstart" "1.$i wordend"
-		Popup "$t(error_underlined): [lindex $expId 1]"
+		Popup "$t(error_underlined): [lindex $expId 1]" $hd(w)
 		return
 	   }
 	}
-	# Apply expression
-	set ids [$fHd(folder_handler) match $expId]
-	if [string length $ids] {
-	    SetFlag $hd(handler) flagged 1 $ids
-	}
-
-	if $hd(doSave) {
-	    if [string length $hd(saveAs)] {
-		lappend expList $expArrayId
-		set expName($expArrayId) $hd(saveAs)
-		set expExp($expArrayId) $exp
-		incr expArrayId
-		ExpWrite
-	    } else {
-		Popup $t(need_name)
+	if {"create" == $hd(action)} {
+	    if $hd(doSave) {
+		if [string length $hd(saveAs)] {
+		    if {-1 != [lsearch -exact $expList $hd(saveAs)]} {
+			Popup $t(name_occupied) $hd(w)
+			return
+		    }
+		    if {"" != $hd(addproc)} {
+			eval "$hd(addproc) [list $hd(saveAs)]"
+		    } else {
+			lappend expList $hd(saveAs)
+		    }
+		    set expExp($hd(saveAs)) $exp
+		    ExpWrite
+		} else {
+		    Popup $t(need_name) $hd(w)
+		}
 	    }
+	    # Apply expression
+	    set ids [$fHd(folder_handler) match $expId]
+	    if [string length $ids] {
+		SetFlag $hd(handler) flagged 1 $ids
+	    }
+
+	} else {
+	    if {$hd(saveAs) != $hd(oldName)} {
+		if {-1 != [lsearch -exact $expList $hd(saveAs)]} {
+		    Popup $t(name_occupied) $hd(w)
+		    return
+		}
+		ExpDelete $hd(oldName)
+		lappend expList $hd(saveAs)
+	    }
+	    set expExp($hd(saveAs)) $exp
+	    ExpWrite
 	}
 	RatFreeExp $expId
     }
-    RecordPos $hd(w) ratExpression
+    if {"create" == $hd(action)} {
+	RecordPos $hd(w) ratExpression
+    } else {
+	RecordPos $hd(w) ratExpEdit
+    }
+    catch {focus $hd(oldfocus)}
     destroy $hd(w)
     foreach bn [array names b $hd(w).*] {unset b($bn)}
     if [string compare $hd(mode) $option(expression_mode)] {
@@ -331,14 +395,12 @@ proc ExpDone {handler action} {
 # Arguments:
 
 proc ExpWrite {} {
-    global option expArrayId expList expName expExp
+    global option expList expExp
 
     set f [open $option(ratatosk_dir)/expressions w]
-    puts $f "set expArrayId $expArrayId"
     puts $f "set expList [list $expList]"
     foreach e $expList {
-	puts $f "set expName($e) [list $expName($e)]"
-	puts $f "set expExp($e) [list $expExp($e)]"
+	puts $f [list set expExp($e) $expExp($e)]
     }
     close $f
 }
@@ -350,7 +412,7 @@ proc ExpWrite {} {
 # Arguments:
 
 proc ExpRead {} {
-    global option expArrayId expList expName expExp
+    global option expList expExp
 
     if [file readable $option(ratatosk_dir)/expressions] {
 	source $option(ratatosk_dir)/expressions
@@ -367,7 +429,7 @@ proc ExpRead {} {
 #		  the selection should be done in.
 
 proc ExpBuildMenu {m handler} {
-    global expArrayId expList expName expExp
+    global expList expExp
 
     $m delete 0 end
 
@@ -375,7 +437,7 @@ proc ExpBuildMenu {m handler} {
 	return
     }
     foreach i $expList {
-	$m add command -label $expName($i) -command "ExpMenuApply $i $handler"
+	$m add command -label $i -command [list ExpMenuApply $i $handler]
     }
 }
 
@@ -406,62 +468,11 @@ proc ExpMenuApply {id handler} {
 #
 # Arguments:
 
-proc ExpHandleSaved {} {
-    global t b expArrayId expList expName idCnt defaultFont
+proc ExpHandleSaved {handler} {
+    global t
 
-    # Create identifier
-    set id expWin[incr idCnt]
-    set w .$id
-    upvar #0 $id hd
-
-    # Create toplevel
-    toplevel $w
-    wm title $w $t(saved_expr)
-
-    # List of expressions
-    frame $w.list
-    listbox $w.list.list -yscroll "$w.list.scroll set" -exportselection 0 \
-	    -font $defaultFont -highlightthickness 0 -selectmode single -bd 1 \
-	    -setgrid 1
-    scrollbar $w.list.scroll -command "$w.list.list yview" -bd 1
-    Size $w.list.list expList
-    pack $w.list.scroll -side right -fill y
-    pack $w.list.list -expand 1 -fill both
-    set b($w.list.list) exp_list
-
-    text $w.text -font $defaultFont -width 30 -height 5 -bd 1
-    set b($w.text) exp_text
-
-    button $w.delete -text $t(delete) -command "ExpDelete $w.list.list $w.text"
-    set b($w.delete) exp_delete
-
-    button $w.dismiss -text $t(dismiss) -command \
-	    "RecordPos $w savedExp; \
-	     RecordSize $w.list.list expList; \
-	     unset b($w.list.list); \
-	     unset b($w.text); \
-	     unset b($w.delete); \
-	     unset b($w.dismiss); \
-	     destroy $w"
-    set b($w.dismiss) dismiss
-
-    pack $w.list -side top -padx 5 -pady 5 -expand 1 -fill both
-    pack $w.text \
-	 $w.delete -side top -fill x -padx 5 -pady 2
-    pack $w.dismiss -pady 5 -padx 5
-    Place $w savedExp
-
-    if [info exists expList] {
-	foreach i $expList {
-	    $w.list.list insert end $expName($i)
-	}
-    }
-
-    bind $w.list.list <1> " \
-	    $w.text delete 1.0 end; \
-	    $w.text insert end \
-		    \$expExp(\[lindex \$expList \[%W index @%x,%y\]\]) \
-	"
+    rat_list::create expList expList "ExpCreate $handler" ExpEdit ExpDelete \
+	    ExpWrite $t(saved_expr) $t(create) $t(edit) $t(delete) $t(dismiss)
 }
 
 # ExpDelete --
@@ -469,24 +480,12 @@ proc ExpHandleSaved {} {
 # Delete the selected expression
 #
 # Arguments:
-# lw - The listbox to delete from
-# tw - The text widget to clear
+# name - Name of expression to delete
 
-proc ExpDelete {lw tw} {
-    global t expArrayId expList expName expExp
+proc ExpDelete {name} {
+    global t expList expExp
 
-    set i [$lw curselection]
-
-    if { 1 != [llength $i]} {
-	Popup $t(need_one_selected_exp)
-	return
-    }
-
-    set arIndex [lindex $expList $i]
+    set i [lsearch -exact $expList $name]
     set expList [lreplace $expList $i $i]
-    unset expName($arIndex)
-    unset expExp($arIndex)
-    $lw delete $i
-    $tw delete 1.0 end
-    ExpWrite
+    unset expExp($name)
 }
