@@ -3,8 +3,8 @@
 # This file contains code which handles help windows
 #
 #
-#  TkRat software and its included text is Copyright 1996,1997,1998
-#  by Martin Forssn
+#  TkRat software and its included text is Copyright 1996-2000 by
+#  Martin Forssn
 #
 #  The full text of the legal notice is contained in the file called
 #  COPYRIGHT, included with this distribution.
@@ -29,7 +29,7 @@ set helporder { intro
 # section  - The section to show (may be empty)
 
 proc Help {{subject intro}} {
-    global idCnt t b help option defaultFont helporder
+    global idCnt t b help option helporder
 
     # Initialize help texts (if needed)
     if ![info exists help(intro)] {
@@ -43,7 +43,7 @@ proc Help {{subject intro}} {
     set hd(w) $w
 
     # Create toplevel
-    toplevel $w
+    toplevel $w -class TkRat
     wm title $w $t(help_window)
 
     # Populate window
@@ -58,8 +58,7 @@ proc Help {{subject intro}} {
 	-bd 1 \
 	-exportselection false \
 	-highlightthickness 0 \
-	-selectmode single \
-	-font $defaultFont 
+	-selectmode single
     set hd(list) $w.subjlist
     Size $hd(list) subjlist
     set b($hd(list)) help_subjlist
@@ -75,8 +74,7 @@ proc Help {{subject intro}} {
 	-wrap word \
 	-relief sunken \
 	-bd 1 \
-	-highlightthickness 0 \
-	-font $defaultFont 
+	-highlightthickness 0
     set hd(text) $w.texttext
     Size $hd(text) helptext
     set b($hd(text)) help_text
@@ -93,6 +91,7 @@ proc Help {{subject intro}} {
     bind $w <Key-space> "$hd(text) yview scroll 1 pages"
     bind $w <Key-BackSpace> "$hd(text) yview scroll -1 pages"
     bind $hd(list) <ButtonRelease-1> "SelectHelp $id"
+    wm protocol $w WM_DELETE_WINDOW "DismissHelp $id"
 
     # Populate list
     foreach topic $helporder {
