@@ -4,12 +4,18 @@
 # about this program.
 #
 #
-#  TkRat software and its included text is Copyright 1996,1997,1998
-#  by Martin Forssn
+#  TkRat software and its included text is Copyright 1996-2000 by
+#  Martin Forssn
 #
 #  The full text of the legal notice is contained in the file called
 #  COPYRIGHT, included with this distribution.
 
+# The version history
+set ratHistory {0.50 0.51 0.52 0.53 0.54 0.55 0.56 0.57 0.58 0.59
+		0.60 0.61 0.62 0.63 0.64 0.65 0.66 0.67 0.68 0.69
+		0.70 0.71 0.72 0.73 0.74 0.75 1.0 1.0.1 1.0.2 1.0.3
+		1.0.4 1.0.5 1.1a 1.2 2.0}
+
 # Version --
 #
 # Opens a window and prints the version information in it.
@@ -17,7 +23,7 @@
 # Arguments:
 
 proc Version {} {
-    global t tkrat_version tkrat_version_date
+    global t tkrat_version tkrat_version_date tcl_patchLevel tk_patchLevel
 
     set w .about
 
@@ -26,21 +32,19 @@ proc Version {} {
 	raise $w
 	return
     }
-    toplevel $w -relief raised
+    toplevel $w -relief raised -class TkRat
     wm title $w $t(about)
     wm minsize $w 1 1
 
-    label $w.tkrat -text "TkRat v$tkrat_version" \
-	    -font "-Adobe-Helvetica-Bold-R-Normal--24-*"
-    label $w.date -text $tkrat_version_date \
-	    -font "-Adobe-Helvetica-Bold-R-Normal--12-*"
-    label $w.version -text "TkRat is copyright 1995,1996,1997 by" \
-	    -font "-Adobe-Helvetica-Bold-R-Normal--12-*"
-    label $w.author -text "Martin Forssn (maf@math.chalmers.se)" \
-	    -font "-Adobe-Helvetica-Bold-R-Normal--12-*"
+    catch {font create big -family Helvetica -size -24 -weight bold}
+    catch {font create norm -family Helvetica -size -12 -weight bold}
+    label $w.tkrat -text "TkRat v$tkrat_version" -font big
+    label $w.date -text "$t(date): $tkrat_version_date" -font norm
+    label $w.tcltk -text "Tcl-$tcl_patchLevel/Tk-$tk_patchLevel" -font norm
+    label $w.copyright -text "TkRat is copyright 1995-2000 by" -font norm
+    label $w.author -text "Martin Forssn (maf@dtek.chalmers.se)" -font norm
     message $w.more -text $t(send_bugs_etc) -aspect 700
-    text $w.text -relief flat -width 40 -height 8 \
-	    -font "-Adobe-Helvetica-Bold-R-Normal--12-*"
+    text $w.text -relief flat -width 40 -height 8 -font norm
     $w.text insert 1.0 $t(send_postcards)
 
     button $w.ok \
@@ -50,7 +54,8 @@ proc Version {} {
 
     pack $w.tkrat \
 	 $w.date \
-	 $w.version \
+	 $w.tcltk \
+	 $w.copyright \
 	 $w.author \
 	 $w.more \
 	 $w.text \
@@ -74,7 +79,7 @@ proc Ratatosk {} {
 	raise $w
 	return
     }
-    toplevel $w -relief raised
+    toplevel $w -relief raised -class TkRat
     wm title $w Ratatosk
     wm minsize $w 1 1
 
@@ -103,7 +108,7 @@ proc InfoWelcome {} {
     InitMessages $option(language) changes
 
     # Create window
-    toplevel .welcome
+    toplevel .welcome -class TkRat
     wm title .welcome $changes(welcome_title)
 
     # Populate window
@@ -151,17 +156,9 @@ proc InfoWelcome {} {
 
     set i 0
     foreach l [GetLanguages] {
-	if ![string length [GetFont [lindex $l 2] 0 {}]] {
-	    continue
-	}
 	.welcome.b.lang.menu.m add command -label [lindex $l 1] \
 		-command [list WelcomeLanguage $l]
-	if [RatEncodingCompat [lindex $l 2] $option(charset)] {
-	    set font [.welcome.b.lang.label cget -font]
-	} else {
-	    set font [GetFont [lindex $l 2] 0 bold]
-	}
-	.welcome.b.lang.menu.m entryconfigure $i -font $font
+	.welcome.b.lang.menu.m entryconfigure $i
 	incr i
 	if ![string compare [lindex $l 0] $option(language)] {
 	    WelcomeLanguage $l
@@ -214,33 +211,17 @@ proc WelcomeShowMenu {{font {}}} {
 # lang -	The language information
 
 proc WelcomeLanguage {lang} {
-    global welcomeLanguage option changes defaultFont
+    global welcomeLanguage option changes propNormFont
 
-    set font [GetFont [lindex $lang 2] 0 bold]
     set welcomeLanguage [lindex $lang 1]
     set option(language) [lindex $lang 0]
     InitMessages $option(language) changes
     wm title .welcome $changes(welcome_title)
-    .welcome.message.text configure -state normal \
-	    -font [GetFont [lindex $lang 2] 0 {}]
+    .welcome.message.text configure -state normal -font $propNormFont
     .welcome.message.text delete 0.0 end
     .welcome.message.text insert 0.0 $changes(welcome)
     .welcome.message.text configure -state disabled
-    if [RatEncodingCompat [lindex $lang 2] $option(charset)] {
-	foreach w {.welcome.b.lang.label .welcome.b.lang.menu 
-		   .welcome.b.shutup.label .welcome.b.shutup.menu
-		   .welcome.b.cont} {
-	    $w configure -font [$w cget -font]
-	}
-	WelcomeShowMenu
-    } else {
-	foreach w {.welcome.b.lang.label .welcome.b.lang.menu 
-		   .welcome.b.shutup.label .welcome.b.shutup.menu
-		   .welcome.b.cont} {
-	    $w configure -font $font
-	}
-	WelcomeShowMenu $font
-    }
+    WelcomeShowMenu
 }
 
 # InfoChanges --
@@ -258,7 +239,7 @@ proc InfoChanges {} {
     InitMessages $option(language) changes
 
     # Create window
-    toplevel .changes
+    toplevel .changes -class TkRat
     wm title .changes $changes(changes_title)
 
     # Message part
@@ -333,7 +314,7 @@ proc SeeLog {} {
     set w .$id
 
     # Create toplevel
-    toplevel $w
+    toplevel $w -class TkRat
     wm title $w $t(seelog_title)
 
     # Message part
@@ -364,18 +345,73 @@ proc SeeLog {} {
 #		which looks like {name data}
 
 proc SendBugReport {{attachments {}}} {
+    global idCnt t
+
+    # Create identifier
+    set id sb[incr idCnt]
+    set w .$id
+    upvar #0 $id hd
+    set hd(oldfocus) [focus]
+
+    # Create the toplevel
+    toplevel $w -bd 5 -class TkRat
+    wm title $w $t(send_bug)
+    set hd(w) $w
+    set hd(attachments) $attachments
+
+    # The contents
+    label $w.slabel -text $t(bug_shortdesc): -anchor e
+    entry $w.sentry -textvariable ${id}(subject)
+    grid $w.slabel $w.sentry - -sticky we
+
+    label $w.dlabel -text $t(bug_description) -justify left 
+    grid $w.dlabel - - -sticky w -pady 5
+
+    text $w.text -relief sunken -bd 1 -setgrid true \
+	    -yscrollcommand "$w.scroll set" -wrap none
+    scrollbar $w.scroll -relief sunken -bd 1 -takefocus 0 \
+	    -command "$w.text yview" -highlightthickness 0
+    Size $w.text bugText
+    grid $w.text - $w.scroll -sticky nsew
+    set hd(text) $w.text
+
+    button $w.continue -text $t(continue) -command "DoSendBugReport $id"
+    grid $w.continue - -
+
+    grid columnconfigure $w 1 -weight 1
+    grid rowconfigure $w 1 -weight 1
+
+    # Binding and focus
+    rat_edit::create $w.text
+    focus $w.sentry
+
+    # Place window
+    Place $w sendBug
+    wm protocol $w WM_DELETE_WINDOW "unset $id; destroy $w"
+}
+
+
+# DoSendBugReport --
+#
+# Construct a skeleton bug report
+#
+# Arguments:
+# handler - name of global array holding data for the report
+
+proc DoSendBugReport {handler} {
+    upvar #0 $handler hd
     global idCnt option t  tkrat_version tkrat_version_date \
-	   tcl_version tk_version tcl_patchLevel tk_patchLevel
+	   tcl_version tk_version tcl_patchLevel tk_patchLevel rat_lib
 
     set mhandler composeM[incr idCnt]
     upvar #0 $mhandler mh
 
     set mh(to) maf@dtek.chalmers.se
-    set mh(subject) "TkRat $tkrat_version bug: \[$t(short_description)\]"
+    set mh(subject) $hd(subject)
     set mh(description) "Bug report"
-    set mh(data) $t(bug_report_template)
+    set mh(data) [$hd(text) get 1.0 end]
 
-    foreach a $attachments {
+    foreach a $hd(attachments) {
 	set ahandler composeB[incr idCnt]
 	upvar #0 $ahandler ah
 
@@ -402,6 +438,11 @@ proc SendBugReport {{attachments {}}} {
     catch {exec uname -a} uname
     puts $fh "uname -a: '$uname'"
     puts $fh "Version: $tkrat_version ($tkrat_version_date)"
+    if [info exists rat_lib(version)] {
+	puts $fh "Libversion: $rat_lib(version) ($rat_lib(date))"
+    } else {
+	puts $fh "Libversion: older"
+    }
     puts $fh "Tcl/Tk: $tcl_version/$tk_version ($tcl_patchLevel/$tk_patchLevel)"
     foreach n [lsort [array names tcl_platform]] {
 	puts $fh "tcl_platform($n): '$tcl_platform($n)'"
@@ -411,6 +452,10 @@ proc SendBugReport {{attachments {}}} {
     }
     close $fh
 
+    catch {focus $hd(oldfocus)}
+    destroy $hd(w)
+    unset hd
+
     DoCompose $mhandler 0 1
 }
 
@@ -430,7 +475,7 @@ proc Warn {tag} {
     set option($tag) 0
 
     # Create toplevel
-    toplevel $w
+    toplevel $w -class TkRat
     wm title $w $t(warning)
 
     # Message part
@@ -448,3 +493,38 @@ proc Warn {tag} {
 
     tkwait window $w
 }
+
+
+# StartupInfo --
+#
+# Give information when starting a new version for for the first time
+#
+# Arguments:
+
+proc StartupInfo {} {
+    global option tkrat_version tkrat_version_date currentLanguage_t ratHistory
+
+    # Check which version the user last used
+    if ![string length $option(last_version)] {
+	InfoWelcome
+
+	# Reinitialize language (if needed)
+	if [string compare $option(language) $currentLanguage_t] {
+	    InitMessages $option(language) t
+	}
+	set option(last_version) $tkrat_version
+	set option(last_version_date) $tkrat_version_date
+	SaveOptions
+    } else {
+	set io [lsearch -exact $ratHistory $option(last_version)]
+	set ic [lsearch -exact $ratHistory $tkrat_version]
+	if {$io < $ic && -1 != $io && $option(info_changes)} {
+	    InfoChanges
+	}
+	if {$option(last_version_date) < $tkrat_version_date} {
+	    set option(last_version_date) $tkrat_version_date
+	    set option(last_version) $tkrat_version
+	    SaveOptions
+	}
+    }
+}
