@@ -3,8 +3,8 @@
 # This file contains code which handles the key definitions window
 #
 #
-#  TkRat software and its included text is Copyright 1996,1997,1998
-#  by Martin Forssn
+#  TkRat software and its included text is Copyright 1996-2000 by
+#  Martin Forssn
 #
 #  The full text of the legal notice is contained in the file called
 #  COPYRIGHT, included with this distribution.
@@ -12,14 +12,15 @@
 # The order of the definitions
 set keyDefOrder(folder) {folder_key_find folder_key_compose folder_key_replya
 	folder_key_replys folder_key_forward_i folder_key_forward_a
-	folder_key_sync folder_key_update folder_key_delete folder_key_undelete
-	folder_key_flag folder_key_nextu folder_key_next folder_key_prev
-	folder_key_home folder_key_bottom folder_key_pagedown folder_key_pageup
+	folder_key_bounce folder_key_sync folder_key_update folder_key_delete
+	folder_key_undelete folder_key_markunread folder_key_flag
+	folder_key_nextu folder_key_next folder_key_prev folder_key_home
+	folder_key_bottom folder_key_pagedown folder_key_pageup
 	folder_key_linedown folder_key_lineup folder_key_cycle_header
-	folder_key_quit}
+	folder_key_print folder_key_close folder_key_quit}
 set keyDefOrder(compose) {compose_key_send compose_key_abort 
 	compose_key_editor compose_key_undo compose_key_cut
-	compose_key_cut_all compose_key_paste}
+	compose_key_cut_all compose_key_copy compose_key_paste}
 
 # KeyDef --
 #
@@ -44,7 +45,7 @@ proc KeyDef {area} {
     set hd(w) $w
 
     # Create toplevel
-    toplevel $w
+    toplevel $w -class TkRat
     wm title $w $t(define_keys)
 
     # Buttons
@@ -103,6 +104,7 @@ proc KeyDef {area} {
 	}
     }
     grid columnconfigure $w.f.canvas 1 -weight 1
+    wm protocol $w WM_DELETE_WINDOW "unset $id; destroy $w"
 
     Place $w keydef
 
@@ -165,7 +167,6 @@ proc AddKey {w name handler} {
     bind $w.label <KeyPress> "KeyEvent p %K $w $name $handler; break"
     bind $w.label <KeyRelease> "KeyEvent r %K $w $name $handler; break"
     focus $w.label
-    grab $w.label
 }
 
 # KeyEvent --
@@ -183,19 +184,19 @@ proc KeyEvent {e key w name handler} {
     upvar #0 $handler hd
     global t
 
-    if [regexp {(Shift|Control|Alt|Mod[1-5]|Meta)(_[LR])?} $key tot mod] {
+    if {[regexp {(Shift|Control|Alt|Mod[1-5]|Meta)(_[LR])?} $key tot mod]} {
 	if [string compare p $e] {
 	    regsub "$mod-" $hd(mod) {} hd(mod)
 	} else {
 	    set hd(mod) "$mod-$hd(mod)"
 	}
-    } else {
+    } elseif [string compare r $e] {
 	set event "<$hd(mod)Key-$key>"
 	set hd(state) ""
 	if [info exists hd($event)] {
 	    # The key already exists
 	    set oname $hd($event)
-	    if { 0 == [RatDialog $t(add_key) \
+	    if { 0 == [RatDialog [winfo toplevel $w] $t(add_key) \
 			   "$t(key_defined) $t($oname)" {} 0 \
 			   $t(replace_key) $t(cancel)]} {
 	        # Remove old definition and update it
@@ -272,7 +273,7 @@ proc KeyDefApply {area handler} {
 	folder {
 		global folderWindowList
 
-		foreach f $folderWindowList {
+		foreach f [array names folderWindowList] {
 		    upvar #0 $f fh
 		    foreach e $remove {
 			bind $fh(w) $e {}
