@@ -4,8 +4,8 @@
 # built in defaults.
 #
 #
-#  TkRat software and its included text is Copyright 1996,1997,1998
-#  by Martin Forssn
+#  TkRat software and its included text is Copyright 1996-2000 by
+#  Martin Forssn
 #
 #  The full text of the legal notice is contained in the file called
 #  COPYRIGHT, included with this distribution.
@@ -18,7 +18,7 @@
 # Arguments:
 
 proc OptionsInit {} {
-    global env option tkrat_version
+    global env option tkrat_version ratCurrent
 
     # Last run version
     set option(last_version) ""
@@ -36,16 +36,18 @@ proc OptionsInit {} {
     set option(global_config_path) $env(CONFIG_DIR)
 
     # Personal config directory
-    set option(ratatosk_dir) ~/.ratatosk
+    if ![info exists option(ratatosk_dir)] {
+	set option(ratatosk_dir) ~/.ratatosk
+    }
 
     # Database directory
-    set option(dbase_dir) ~/.ratatosk/db
+    set option(dbase_dir) $option(ratatosk_dir)/db
 
     # Directory to backup database messages to
-    set option(dbase_backup) ~/.ratatosk/backup
+    set option(dbase_backup) $option(ratatosk_dir)/backup
 
     # Directory to store outgoing messages
-    set option(send_cache) ~/.ratatosk/send
+    set option(send_cache) $option(ratatosk_dir)/send
 
     # Delivery mode
     set option(delivery_mode) direct
@@ -54,47 +56,50 @@ proc OptionsInit {} {
     set option(expire_interval) 7
 
     # Directory for message hold
-    set option(hold_dir) ~/.ratatosk/hold
+    set option(hold_dir) $option(ratatosk_dir)/hold
 
     # Userprocedures file
-    set option(userproc) ~/.ratatosk/userproc
+    set option(userproc) $option(ratatosk_dir)/userproc
+
+    # WIndow size and positions file
+    set option(placement) $option(ratatosk_dir)/placement
 
     # Main window name
-    set option(main_window_name) "TkRat v$tkrat_version"
+    set option(main_window_name) "TkRat v$tkrat_version: %f"
 
     # Main window geometry
     set option(main_geometry) +0+50
 
     # Icon name
-    set option(icon_name) "TkRat v$tkrat_version"
+    set option(icon_name) "TkRat v$tkrat_version: %f"
 
-    # Command to get default folder
-    set option(default_folder) "INBOX file {} /var/spool/mail/$env(USER)"
+    # Default folder specification
+    set option(default_folder) "INBOX file {} INBOX"
 
     # Format of list of messages
-    set option(list_format) "%4S %6d  %-24n %4B %s"
+    set option(list_format) "%4S %6d  %-24n %4B %t%s"
 
-    # How many headers we should show
+    # Which headers we should show
     set option(show_header) selected
 
     # Which the selected headers are:
     set option(show_header_selection) {From Subject Date To CC Reply-To}
 
-    # How many old directories we should remember
-    set option(prev_dir) 10
-
     # Default permissions mask
     set option(permissions) 0600
 
     # Geometry of compose window
     set option(compose_geometry) +0+50
 
-    # Which headers to compose
+    # Which headers to compose by default
     set option(compose_headers) {To Subject Cc}
 
     # Which editor to use (%s will be expanded to a filename)
     set option(editor) "emacs %s"
 
+    # True if we always want to use the external editor
+    set option(always_editor) 0
+
     # Which domain we should pretend we are from
     set option(masquerade_as) {}
 
@@ -111,16 +116,13 @@ proc OptionsInit {} {
     set option(sendprog) /usr/lib/sendmail
 
     # Can the sending program handle eightbit data
-    set option(sendprog_8bit) false
+    set option(sendprog_8bit) true
 
     # Default character set for tcl
     set option(charset) iso-8859-1
 
     # Leader string for replies
-    set option(reply_lead) {>}
-
-    # Width of listboxes in vfolderdef window
-    set option(vfoldername_width) 15
+    set option(reply_lead) {> }
 
     # True (1) if we should show the watcher
     set option(watcher_enable) 1
@@ -137,10 +139,7 @@ proc OptionsInit {} {
     # Watcher max height
     set option(watcher_max_height) 10
 
-    # Font in watcher
-    set option(watcher_font) 5x7
-
-    # How many messages the watcher shall show
+    # Which messages the watcher shall show ('new' or 'all')
     set option(watcher_show) new
 
     # How many times the bell should be run when new messages arive
@@ -150,7 +149,7 @@ proc OptionsInit {} {
     set option(watcher_format) "%4S %-24n %s"
 
     # Print command
-    set option(print_command) "lpr %s"
+    set option(print_command) "lpr -P %p %s"
 
     # Headers to print
     set option(print_header) selected
@@ -165,7 +164,7 @@ proc OptionsInit {} {
     set option(signature) ~/.signature
 
     # Default folder sort method
-    set option(folder_sort) folder
+    set option(folder_sort) threaded
 
     # Message attribution
     set option(attribution) "On %d, %n wrote:"
@@ -174,12 +173,12 @@ proc OptionsInit {} {
     set option(forwarded_message) "------ Forwarded message ------"
 
     # File typing
-    set option(typetable) { {*text* text/plain}
-			    {*GIF* image/gif}
+    set option(typetable) { {*GIF* image/gif}
 			    {*JPEG* image/jpeg}
 			    {*JPG* image/jpeg}
 			    {"*8-bit u-law*" audio/basic}
 			    {*PostScript* application/postscript}
+			    {*text* text/plain}
 			    {*data* application/octet-stream}}
 
     # True if we want to see ALL messages from c-client (including babble)
@@ -192,7 +191,7 @@ proc OptionsInit {} {
     set option(num_messages) 10
 
     # True if we should lookup usernames in the local passwd-list
-    set option(lookup_name) true
+    set option(lookup_name) 1
 
     # Default database expiration type
     set option(def_extype) remove
@@ -204,13 +203,13 @@ proc OptionsInit {} {
     set option(chunksize) 100
 
     # Where we should store dsn files
-    set option(dsn_directory) ~/.ratatosk/DSN
+    set option(dsn_directory) $option(ratatosk_dir)/DSN
 
     # If we should default to request DSN
     set option(dsn_request) 0
 
     # If we should remove delivery reports from folders
-    set option(dsn_snarf_reports) 0
+    set option(dsn_snarf_reports) 1
 
     # How many days each DSN entry should be kept in the list
     set option(dsn_expiration) 7
@@ -227,12 +226,16 @@ proc OptionsInit {} {
     # The default reply_to address
     set option(reply_to) ""
 
+    # THe defailt bcc address
+    set option(bcc) ""
+
     # The font size we user
     set option(fontsize) 12
 
     # Folder window key combination
     set option(folder_key_compose) <Key-m>
-    set option(folder_key_quit) <Control-Key-c>
+    set option(folder_key_close) <Control-Key-c>
+    set option(folder_key_quit) <Control-Key-q>
     set option(folder_key_nextu) <Key-Tab>
     set option(folder_key_sync) <Control-Key-s>
     set option(folder_key_update) <Control-Key-u>
@@ -253,6 +256,9 @@ proc OptionsInit {} {
     set option(folder_key_lineup) {<Key-Up>}
     set option(folder_key_cycle_header) <Key-h>
     set option(folder_key_find) <Key-l>
+    set option(folder_key_bounce) <Key-b>
+    set option(folder_key_markunread) <Key-U>
+    set option(folder_key_print) <Key-P>
 
     # Compose window key combinations
     set option(compose_key_send) <Control-s>
@@ -260,8 +266,10 @@ proc OptionsInit {} {
     set option(compose_key_editor) <Control-o>
     set option(compose_key_undo) <Control-u>
     set option(compose_key_cut) <Control-w>
-    set option(compose_key_cut_all) <Control-Shift-w>
+    set option(compose_key_copy) <Meta-w>
+    set option(compose_key_cut_all) <Control-x>
     set option(compose_key_paste) <Control-y>
+    set option(compose_key_wrap) <Control-j>
 
     # If we should check for stolen mail
     set option(mail_steal) 1
@@ -285,6 +293,9 @@ proc OptionsInit {} {
     # If we should try to send multiple letters though one channel
     set option(smtp_reuse) 1
 
+    # Override color resources
+    set option(override_color) 1
+
     # The color set
     set option(color_set) {gray85 black}
 
@@ -300,9 +311,6 @@ proc OptionsInit {} {
     # If the compose editor should warn about cutting all text etc
     set option(compose_warn) 1
 
-    # Length of message list
-    set option(message_llength) 14
-
     # Mailcap path
     set option(mailcap_path) {~/.mailcap:/etc/mailcap:/usr/etc/mailcap:/usr/local/etc/mailcap}
 
@@ -312,6 +320,15 @@ proc OptionsInit {} {
     # Imap port
     set option(imap_port) 143
 
+    # Pop3 port
+    set option(pop3_port) 110
+
+    # Default remote user
+    set option(remote_user) $env(USER)
+
+    # Default remote host
+    set option(remote_host) ""
+
     # SMTP timeout
     set option(smtp_timeout) 120
 
@@ -319,28 +336,20 @@ proc OptionsInit {} {
     set option(force_send) 0
 
     # Should we skip the signature of the message we are replying to
-    set option(skip_sig) true
+    set option(skip_sig) 1
 
-    # Are we PGP capable?
-    if [info exists env(PGPPATH)] {
-	set pgpdir $env(PGPPATH)
-    } else {
-	set pgpdir ~/.pgp
-    }
-    if {0 < [string length "$env(PGP)"] &&
-	    1 == [file readable $pgpdir/pubring.pgp] &&
-	    1 == [file readable $pgpdir/secring.pgp]} {
-	set option(pgp_enable) 1
-    } else {
-	set option(pgp_enable) 0
-    }
+    # PGP operations
+    set option(pgp_version) auto
 
     # Path to pgp program
-    set option(pgp_prog) $env(PGP)
+    set option(pgp_path) {}
 
     # Extra pgp options
     set option(pgp_args) {}
 
+    # Name of pgp keyring
+    set option(pgp_keyring) {}
+
     # If we should make a copy of attached files
     set option(copy_attached) 1
 
@@ -353,11 +362,14 @@ proc OptionsInit {} {
     # Default url viewer
     set option(url_viewer) netscape
 
-    # Name (and path) of netscape executable
-    set option(netscape) netscape
+    # Name (and path) of netscape command
+    set option(netscape) {netscape -install}
 
-    # Extra arguments for netscape
-    set option(netscape_args) -install
+    # Name (and path) of lynx command
+    set option(lynx) {xterm -T "Lynx:%u" +sb -e lynx "%u"}
+
+    # Name (and path) of other command
+    set option(other_browser) {other_browser %u}
 
     # Color of URL
     set option(url_color) blue
@@ -367,7 +379,8 @@ proc OptionsInit {} {
     set option(use_system_aliases) 1
 
     # Personal alias lists
-    set option(addrbooks) {{Personal tkrat ~/.ratatosk/aliases}}
+    set option(addrbooks) \
+	    [list [list Personal tkrat $option(ratatosk_dir)/aliases]]
 
     # Default alias book
     set option(default_book) Personal
@@ -384,7 +397,7 @@ proc OptionsInit {} {
     set option(cache_conn_timeout) 10
 
     # URL protocols
-    set option(urlprot) {http shttp ftp news telnet}
+    set option(urlprot) {http https ftp news telnet}
 
     # Balloon help
     set option(show_balhelp) 1
@@ -395,11 +408,134 @@ proc OptionsInit {} {
     # Message finding fields
     set option(msgfind_format) "%s%n%b%D"
 
-    # Should we warn about tk version 8.0
-    set option(warn_tk80) 1
-
     # Automatically expunge on folder close
     set option(expunge_on_close) 1
+
+    # Checkpoint on window unmap
+    set option(checkpoint_on_unmap) 1
+
+    # How often should it checkpoint the mailbox (when deiconfied) (seconds)
+    set option(checkpoint_interval) 600
+
+    # List of known character sets
+    set option(charsets) {
+	us-ascii
+	iso-8859-1
+	iso-8859-2
+	iso-8859-3
+	iso-8859-4
+	iso-8859-5
+	iso-8859-6
+	iso-8859-7
+	iso-8859-8
+	iso-8859-9
+	iso2022-jp
+	iso2022-kr
+    }
+
+    # Default envelope from address
+    set option(return_path) {}
+
+    # Automatically create sender field
+    set option(create_sender) 1
+
+    # Unused option which must be here
+    set option(tip) {}
+
+    # Alias expansion level 
+    set option(alias_expand) 1
+
+    # Protocol for simple mail folders (file,mh,imap)
+    set option(simple_prot) file
+
+    # Data for simple mail folders
+    set option(simple_data) $env(HOME)/Mail
+    #set option(simple_data) host:port
+
+    # Dynamic folder behaviour (expanded | closed)
+    set option(dynamic_behaviour) expanded
+
+    # If submenus should have a tearoff entry
+    set option(tearoff) 0
+
+    # How long to delay certain menus (in milliseconds)
+    set option(menu_delay) 200
+
+    # Where to store cached passwords
+    set option(pwcache_file) $option(ratatosk_dir)/pwcache
+
+    # If we should add the signature delimiter
+    set option(sigdelimit) 1
+
+    # Place where lines wrap
+    set option(wrap_length) 72
+
+    # Regexp for finding citation marks
+    set option(citexp) {^[ 	]*(([a-zA-Z0-9]+> *)|(>+ *)+)?}
+
+    # Should we wrap cited text automatically
+    set option(wrap_cited) 0
+
+    # Directory to store local copies of disconnected folders
+    set option(disconnected_dir) $option(ratatosk_dir)/disconnected
+
+    # What to synchronize when doing a network synchronization
+    # deferred_messages disconnected_mailboxes run_cmd cmd_to_run
+    set option(network_sync) {1 1 0 {}}
+
+    # Name of busy cursor
+    set option(busy_cursor) watch
+
+    # Regular expression which identifies the Re: part of subjects
+    # will be applied with -nocase
+    set option(re_regexp) {re:|sv:}
+
+    # Printing defaults
+    set option(print_pretty) 1
+    set option(print_dest) printer
+    if [info exists env(PRINTER)] {
+	set option(print_printer) $env(PRINTER)
+    } else {
+	set option(print_printer) ps
+    }
+    set option(print_file) {tkrat.ps}
+    set option(print_papersize) A4
+    set option(print_papersizes) {{A4 {596 842}} {A3 {842 1191}}
+				  {Letter {612 792}} {Legal {612 1008}}}
+    set option(print_orientation) portrait
+    set option(print_fontsize) 12
+    set option(print_resolution) 72
+    set option(print_fontfamily) Times
+
+    # additional Compose/Replies options
+    set option(append_sig) 1
+    set option(reply_bottom) 1
+
+    # Font options
+    set option(override_fonts) 1
+    set option(prop_norm) {components Helvetica 12 bold roman 0 0}
+    set option(prop_light) {components Helvetica 12 normal roman 0 0}
+    set option(fixed_norm) {components Courier 12 normal roman 0 0}
+    set option(fixed_bold) {components Courier 12 bold roman 0 0}
+    set option(watcher_font) {name 5x7}
+
+    # Debug output dir
+    set option(debug_file) $option(ratatosk_dir)/log
+
+    # Debugging status
+    set option(debug_cclient) 0
+
+    # Wrap mode for shown messages
+    set option(wrap_mode) word
+
+    # Use input methodes?
+    set option(useinputmethods) 0
+
+    # path to ispell
+    set option(ispell_path) ispell
+
+    # ispell dictionary
+    set option(ispell_dictionary) ~/.ispell_words
 }
 
 # OptionsRead --
@@ -425,6 +561,10 @@ proc OptionsRead {} {
     if [file readable $option(ratatosk_dir)/ratatoskrc] {
 	source $option(ratatosk_dir)/ratatoskrc
     }
+    # Read local overrides
+    if [file readable $option(ratatosk_dir)/ratatoskrc.tcl] {
+	source $option(ratatosk_dir)/ratatoskrc.tcl
+    }
 }
 
 # SaveOptions --
@@ -446,7 +586,7 @@ proc SaveOptions {} {
 
     set fh [open $option(ratatosk_dir)/ratatoskrc w]
     puts $fh $message
-    foreach name [array names globalOption] {
+    foreach name [lsort [array names globalOption]] {
 	if [string compare $option($name) $globalOption($name)] {
 	    puts $fh "set option($name) [list $option($name)]"
 	}
@@ -469,41 +609,6 @@ proc ReadUserproc {} {
     }
 }
 
-# InitFonts --
-#
-# Initializes the default fonts
-#
-# These initializations can be overridden in the local configuration files.
-# See the ../CONFIGURATION file for details.
-#
-# Arguments:
-
-proc InitFonts {} {
-    AddFont iso-8859-1 8 {} *-courier-medium-r-normal-*-8-*-iso8859-1
-    AddFont iso-8859-1 8 {bold} *-courier-bold-r-normal-*-8-*-iso8859-1
-    AddFont iso-8859-1 8 {italic} *-courier-medium-o-normal-*-8-*-iso8859-1
-    AddFont iso-8859-1 8 {bold italic} *-courier-bold-o-normal-*-8-*-iso8859-1
-    AddFont iso-8859-1 10 {} *-courier-medium-r-normal-*-10-*-iso8859-1
-    AddFont iso-8859-1 10 {bold} *-courier-bold-r-normal-*-10-*-iso8859-1
-    AddFont iso-8859-1 10 {italic} *-courier-medium-o-normal-*-10-*-iso8859-1
-    AddFont iso-8859-1 10 {bold italic} *-courier-bold-o-normal-*-10-*-iso8859-1
-    AddFont iso-8859-1 12 {} *-courier-medium-r-normal-*-12-*-iso8859-1
-    AddFont iso-8859-1 12 {bold} *-courier-bold-r-normal-*-12-*-iso8859-1
-    AddFont iso-8859-1 12 {italic} *-courier-medium-o-normal-*-12-*-iso8859-1
-    AddFont iso-8859-1 12 {bold italic} *-courier-bold-o-normal-*-12-*-iso8859-1
-    AddFont iso-8859-1 14 {} *-courier-medium-r-normal-*-14-*-iso8859-1
-    AddFont iso-8859-1 14 {bold} *-courier-bold-r-normal-*-14-*-iso8859-1
-    AddFont iso-8859-1 14 {italic} *-courier-medium-o-normal-*-14-*-iso8859-1
-    AddFont iso-8859-1 14 {bold italic} *-courier-bold-o-normal-*-14-*-iso8859-1
-    AddFont iso-8859-1 18 {} *-courier-medium-r-normal-*-18-*-iso8859-1
-    AddFont iso-8859-1 18 {bold} *-courier-bold-r-normal-*-18-*-iso8859-1
-    AddFont iso-8859-1 18 {italic} *-courier-medium-o-normal-*-18-*-iso8859-1
-    AddFont iso-8859-1 18 {bold italic} *-courier-bold-o-normal-*-18-*-iso8859-1
-    AddFont iso-8859-1 24 {} *-courier-medium-r-normal-*-24-*-iso8859-1
-    AddFont iso-8859-1 24 {bold} *-courier-bold-r-normal-*-24-*-iso8859-1
-    AddFont iso-8859-1 24 {italic} *-courier-medium-o-normal-*-24-*-iso8859-1
-    AddFont iso-8859-1 24 {bold italic} *-courier-bold-o-normal-*-24-*-iso8859-1
-}
 
 # InitCharsetAliases
 #
@@ -512,10 +617,57 @@ proc InitFonts {} {
 # Arguments:
 
 proc InitCharsetAliases {} {
-    global charsetAlias option
+    global charsetAlias option charsetName charsetMapping charsetReverseMapping
+
+    # Mapping to tcl names
+    set charsetMapping(us-ascii) ascii
+    set charsetMapping(utf-8) utf-8
+    set charsetMapping(iso-8859-1) iso8859-1
+    set charsetMapping(iso-8859-2) iso8859-2
+    set charsetMapping(iso-8859-3) iso8859-3
+    set charsetMapping(iso-8859-4) iso8859-4
+    set charsetMapping(iso-8859-5) iso8859-5
+    set charsetMapping(iso-8859-6) iso8859-6
+    set charsetMapping(iso-8859-7) iso8859-7
+    set charsetMapping(iso-8859-8) iso8859-8
+    set charsetMapping(iso-8859-9) iso8859-9
+    set charsetMapping(iso-2022-jp) iso2022-jp
+    set charsetMapping(iso-2022-kr) iso2022-kr
+    set charsetMapping(windows-1250) cp1250
+    set charsetMapping(windows-1251) cp1251
+    set charsetMapping(windows-1252) cp1252
+    set charsetMapping(windows-1253) cp1253
+    set charsetMapping(windows-1254) cp1254
+    set charsetMapping(windows-1255) cp1255
+    set charsetMapping(windows-1256) cp1256
+    set charsetMapping(windows-1257) cp1257
+    set charsetMapping(windows-1258) cp1258
+    set charsetMapping(windows-437) cp437
+    set charsetMapping(windows-737) cp737
+    set charsetMapping(windows-775) cp775
+    set charsetMapping(windows-850) cp850
+    set charsetMapping(windows-852) cp852
+    set charsetMapping(windows-855) cp855
+    set charsetMapping(windows-857) cp857
+    set charsetMapping(windows-860) cp860
+    set charsetMapping(windows-861) cp861
+    set charsetMapping(windows-862) cp862
+    set charsetMapping(windows-863) cp863
+    set charsetMapping(windows-864) cp864
+    set charsetMapping(windows-865) cp865
+    set charsetMapping(windows-866) cp866
+    set charsetMapping(windows-869) cp869
+    set charsetMapping(windows-874) cp874
+    set charsetMapping(windows-932) cp932
+    set charsetMapping(windows-936) cp936
+    set charsetMapping(windows-949) cp949
+    set charsetMapping(windows-950) cp950
+    set charsetMapping(gb2312) ascii
+    set charsetMapping(gb1988) ascii
+    set charsetMapping(gb12345) ascii
 
     # These are predefined (remember to only use lowercase letters)
-    set charsetAlias(bar) iso-8859-1
+    #set charsetAlias(bar) iso-8859-1
 
     # Read global files
     foreach dir $option(global_config_path) {
@@ -527,4 +679,60 @@ proc InitCharsetAliases {} {
     if [file readable $option(ratatosk_dir)/charsetAliases] {
 	source $option(ratatosk_dir)/charsetAliases
     }
+
+    # Build structure
+    catch {unset charsetName}
+    foreach c $option(charsets) {
+	set charsetName($c) ""
+    }
+    foreach a [array names charsetAlias] {
+	if [info exists charsetName($charsetAlias($a))] {
+	    lappend charsetName($charsetAlias($a)) $a
+	}
+    }
+
+    foreach c [array names charsetMapping] {
+	set charsetReverseMapping($charsetMapping($c)) $c
+    }
+}
+
+# InitPgp --
+#
+# Initializes the pgp_version and pgp_prog variables if they are
+# no already set.
+#
+# Arguments:
+
+proc InitPgp {} {
+    global option env
+
+    if {"auto" != $option(pgp_version)} {
+	return
+    }
+
+    set option(pgp_version) 0
+    foreach d [split $env(PATH) :] {
+	if [file executable $d/pgpk] {
+	    set option(pgp_version) 5
+	    set option(pgp_path) $d
+	    set option(pgp_keyring) "~/.pgp/pubring.pkr"
+	    return
+	}
+	if [file executable $d/gpg] {
+	    set option(pgp_version) gpg-1
+	    set option(pgp_path) $d
+	    set option(pgp_keyring) "~/.pgp/pubring.pgp"
+	}
+	if [file executable $d/pgp] {
+	    catch {exec $d/pgp -v} out
+	    set version [lindex [lindex [split $out \n] 0] end]
+	    if {[regexp {^6\.} $version]} {
+		set option(pgp_version) 6
+	    } else {
+		set option(pgp_version) 2
+	    }
+	    set option(pgp_path) $d
+	    set option(pgp_keyring) "~/.pgp/pubring.pgp"
+	}
+    }
 }
