@@ -3,8 +3,8 @@
 # This file contains code which handles pgp interaction
 #
 #
-#  TkRat software and its included text is Copyright 1996,1997,1998
-#  by Martin Forssn
+#  TkRat software and its included text is Copyright 1996-2000 by
+#  Martin Forssn
 #
 #  The full text of the legal notices is contained in the file called
 #  COPYRIGHT, included with this distribution.
@@ -23,11 +23,9 @@ proc RatGetPGPPassPhrase {} {
     set id pgpPass[incr idCnt]
     set w .$id
     upvar #0 $id hd
-    set oldFocus [focus]
 
     # Create toplevel
-    toplevel $w
-    wm transient $w .
+    toplevel $w -class TkRat
     wm title $w $t(pgp_pass_phrase) 
 
     # Populate window
@@ -38,15 +36,14 @@ proc RatGetPGPPassPhrase {} {
     grid $w.label $w.entry -pady 5 -padx 5
     grid $w.button_ok $w.button_cancel
     bind $w.entry <Return> "set ${id}(done) ok"
+    wm protocol $w WM_DELETE_WINDOW "set ${id}(done) abort"
 
     Place $w pgpPhrase
-    focus $w.entry
-    grab $w
+    ModalGrab $w $w.entry
 
     tkwait variable ${id}(done)
 
     RecordPos $w pgpPhrase
-    catch {focus $oldFocus}
     destroy $w
     set ret [list $hd(done) $hd(phrase)]
     unset hd
@@ -70,7 +67,7 @@ proc RatPGPError {error} {
     upvar #0 $id hd
 
     # Create toplevel
-    toplevel $w
+    toplevel $w -class TkRat
     wm transient $w .
     wm title $w $t(pgp_problem) 
 
@@ -105,9 +102,11 @@ proc RatPGPError {error} {
     button $w.f.b.abort -text $t(abort) -command "set ${id}(done) ABORT"
     pack $w.f.b.retry $w.f.b.abort -side left -expand 1 -pady 5
     pack $w.f.b -side bottom -fill x
+    wm protocol $w WM_DELETE_WINDOW "set ${id}(done) ABORT"
 
     Place $w pgpError
 
+    ModalGrab $w
     tkwait variable ${id}(done)
 
     RecordSize $w.f.t.text pgpError
@@ -127,74 +126,58 @@ proc RatPGPError {error} {
 # arg	- argument to procedure (before list of ids)
 
 proc RatPGPGetIds {proc arg} {
-    global idCnt t defaultFont
+    global idCnt t option
+
+    # List keys
+    if [catch {RatPGP listkeys} keylist] {
+	Popup $keylist
+	return
+    }
 
     # Create identifier
     set id pgpGet[incr idCnt]
     set w .$id
     upvar #0 $id hd
-    set oldFocus [focus]
 
     # Create toplevel
-    toplevel $w
+    toplevel $w -class TkRat
     wm title $w $t(select_keys) 
 
-    # Add label
-    set form "%4s %5s %8s  %10s   %s"
-    set l [format $form $t(type) $t(bits) $t(keyid) $t(date) $t(user_id)]
-    label $w.label -text $l -font $defaultFont
-    grid $w.label -sticky w
-
-    # Add list
-    scrollbar $w.scroll \
-        -relief sunken \
-        -command "$w.list yview" \
-	-highlightthickness 0 \
-	-bd 1
-    listbox $w.list \
-        -yscroll "$w.scroll set" \
-        -font $defaultFont \
-        -exportselection false \
-	-highlightthickness 0 \
-	-selectmode multiple \
-	-setgrid true \
-	-bd 1
-    Size $w.list pgpGet
-    grid $w.list $w.scroll -sticky nsew
+    # Add text
+    set hd(list) $w.l
+    rat_textlist::create $hd(list) [lindex $keylist 0]
+    Size [rat_textlist::textwidget $hd(list)] pgpGet
+    pack $w.l -side top -fill both -expand 1
 
     # Buttons
     frame $w.buttons
     button $w.buttons.ok -text $t(ok) \
 	    -command "RatPGPGetIdsDone $w $id 1 [list $proc $arg]"
     button $w.buttons.sel -text $t(select_all) \
-	    -command "$w.list selection set 0 end"
+	    -command "rat_textlist::selection $hd(list) set 0 end"
     button $w.buttons.unsel -text $t(deselect_all) \
-	    -command "$w.list selection clear 0 end"
+	    -command "rat_textlist::selection $hd(list) clear"
     button $w.buttons.cancel -text $t(cancel) \
 	    -command "RatPGPGetIdsDone $w $id 0 [list $proc $arg]"
     pack $w.buttons.ok \
 	 $w.buttons.sel \
 	 $w.buttons.unsel \
 	 $w.buttons.cancel -side left -expand 1
-    grid $w.buttons -columnspan 2 -sticky ew -pady 5
+    pack $w.buttons -side bottom -pady 5 -fill x
 
-    grid columnconfigure $w 0 -weight 1
-    grid rowconfigure $w 1 -weight 1
-
-    foreach key [RatPGP listkeys] {
-	set l [format $form [lindex $key 0] \
-			    [lindex $key 1] \
-			    [lindex $key 2] \
-			    [lindex $key 3] \
-			    [lindex $key 4]]
-	$w.list insert end $l
-	lappend hd(ids) [lindex $key 4]
+    # Populate text widget
+    set hd(keys) {}
+    foreach e [lindex $keylist 1] {
+	lappend hd(keys) [lrange $e 0 1]
+	rat_textlist::insert $hd(list) end [lindex $e 2]
     }
 
+    wm protocol $w WM_DELETE_WINDOW \
+	    "RatPGPGetIdsDone $w $id 0 [list $proc $arg]"
+
     Place $w pgpGet
 }
 
-
 # RatPGPGetIdsDone --
 #
 # Calls when the selection is done
@@ -203,21 +186,23 @@ proc RatPGPGetIds {proc arg} {
 # w	  -	The id selection window
 # handler -	The handler which identifies the session window
 # done    -	The users selection (1=ok, 0=cancel)
-# proc	- procedure to call when done
-# arg	- argument to procedure (before list of ids)
+# proc	  -	procedure to call when done
+# arg	  -	argument to procedure (before list of ids)
 
 proc RatPGPGetIdsDone {w handler done proc arg} {
     upvar #0 $handler hd
     global option
 
     if $done {
-	foreach s [$w.list curselection] {
-	    lappend ids [lindex $hd(ids) $s]
+	set ids {}
+	foreach s [rat_textlist::selection $hd(list) get] {
+	    lappend ids [lrange [lindex $hd(keys) $s] 0 1]
 	}
 	$proc $arg $ids
     }
     RecordPos $w pgpGet
-    RecordSize $w.list pgpGet
+    RecordSize [rat_textlist::textwidget $hd(list)] pgpGet
+    catch {focus $hd(oldfocus)}
     destroy $w
     unset hd
 }
@@ -244,7 +229,20 @@ proc RatPGPAddKeys {keys {keyring ""}} {
     close $f
 
     # Create command and run it
-    set cmd "$option(pgp_prog) $option(pgp_args) -ka $hd(fileName) $keyring"
+    if {"" != $option(pgp_path)} {
+	set dir $option(pgp_path)/
+    } else {
+	set dir ""
+    }
+    if {$option(pgp_version) == 2} {
+        set cmd "${dir}pgp -ka $hd(fileName) $keyring"
+    } elseif {$option(pgp_version) == 5} {
+        set cmd "${dir}pgpk -ka $hd(fileName) $keyring"
+    } elseif {$option(pgp_version) == "gpg-1"} {
+        set cmd "${dir}gpg --no-secmem-warning -q --import $hd(fileName)"
+    } elseif {$option(pgp_version) == 6} {
+        set cmd "${dir}pgp -ka $hd(fileName) $keyring"
+    }
     set cmd "$cmd; echo '$t(press_return_to_dismiss)'; read FOO"
 
     RatBgExec ${id}(existStatus) "$option(terminal) \"$cmd\""
