@@ -3,20 +3,105 @@
 # This file contains code which handles the actual displaying of a message
 #
 #
-#  TkRat software and its included text is Copyright 1996,1997,1998
-#  by Martin Forssn
+#  TkRat software and its included text is Copyright 1996-2000 by
+#  Martin Forssn
 #
 #  The full text of the legal notice is contained in the file called
 #  COPYRIGHT, included with this distribution.
 
 
 # A list of types we can show
-set GoodTypes {text/plain message/rfc822 image/gif}
+set GoodTypes [list text/plain message/rfc822 image/gif image/ppm image/pgm]
+set ImageTypes [list gif ppm pgm]
 
 # A list of parameters to show for different types
 set showParams(text/plain) {charset}
 set showParams(message/external-body) {access-type site server name directory}
 
+# A list of address headers
+set showAddrHdr {from to cc bcc reply_to Return_Path}
+
+# handlers for different bodyparts
+set mimeHandler(text/plain)		{ ShowTextPlain $handler $body $msg}
+set mimeHandler(text)			{ ShowTextOther $handler $body $msg}
+set mimeHandler(multipart/alternative)	{ ShowMultiAlt $handler $body $msg }
+set mimeHandler(multipart/encrypted)	{ ShowMultiEnc $handler $body $msg }
+set mimeHandler(multipart)		{ ShowMultiMixed $handler $body $msg }
+set mimeHandler(message/rfc822)		{ ShowMessage 0 $handler [$body message] }
+set mimeHandler(message/delivery-status) {ShowDSN $handler $body }
+set mimeHandler(image)			{ ShowImage $handler $body [lindex $type 1]}
+set mimeHandler(application/pgp-keys)	{ ShowPGPKeys $handler $body }
+set mimeHandler(application/pgp)	{ ShowTextPlain $handler $body $msg}
+
+if {![catch {package require Img} err]} {
+    set extensions(Img) 1
+    lappend GoodTypes image/jpeg image/tiff image/bmp image/xpm image/png \
+        image/pjpeg
+    lappend ImageTypes jpeg tiff bmp xpm png pjpeg
+}
+if {![catch {package require Tkhtml} err]} {
+    set extensions(Tkhtml) 1
+    lappend GoodTypes text/html
+    set mimeHandler(text/html) {ShowTextHtml $handler $body $msg}
+
+    bind HtmlClip <Motion> {
+        global htmlWinCursor
+        set parent [winfo parent %W]
+        set url [$parent href %x %y] 
+        if {![info exists htmlWinCursor($parent)]} {
+            set htmlWinCursor($parent) [lindex [$parent configure -cursor] end]
+        }
+        if {[string length $url] > 0} {
+            if {[string length $htmlWinCursor($parent)] == 0} {
+                set htmlWinCursor($parent) "hand2"
+                $parent configure -cursor $htmlWinCursor($parent)
+            }
+        } else {
+            if {[string length $htmlWinCursor($parent)] > 0} {
+                set htmlWinCursor($parent) ""
+                $parent configure -cursor {}
+            }
+        }
+    }
+    bind HtmlClip <Button-1> {
+        set ::htmlWinClick [[winfo parent %W] href %x %y]
+    }
+    bind HtmlClip <ButtonRelease-1> {
+        if {![string compare $::htmlWinClick [[winfo parent %W] href %x %y]]} {
+	    set url $::htmlWinClick
+	    # Start viewer
+	    switch $option(url_viewer) {
+		RatUP {
+		    if [catch {RatUP_ShowURL $url} error] {
+			Popup $error %W
+		    }
+		}
+		netscape {
+		    trace variable urlstatus($url) w \
+			    [list RatMaybeStartNetscape $url]
+		    regsub -all , $url %2c url
+		    regsub -all {\$} $url %24 netscape_url
+		    RatBgExec urlstatus($url) "$option(netscape) -remote \
+			    \"openURL($netscape_url)\" >&/dev/null"
+                }
+		lynx {
+		    regsub -all %u $option(lynx) $url cmd
+		    if [catch {eval exec $cmd &} error] {
+			Popup $error %W
+		    }
+		}
+		other {
+		    regsub -all %u $option(other_browser) $url cmd
+		    if [catch {eval exec $cmd &} error] {
+			Popup $error %W
+		    }
+		}
+	    }
+        }
+    }
+}
+
+
 # ShowNothing --
 #
 # Clear the show window
@@ -40,7 +125,7 @@ proc ShowNothing handler {
 # browse  -	True if we should use browse mode showing
 
 proc Show {handler msg browse} {
-    global option b langCharset
+    global option b fixedBoldFont
     upvar #0 $handler fh \
     	     msgInfo_$msg msgInfo
 
@@ -49,6 +134,7 @@ proc Show {handler msg browse} {
     set fh(sigstatus) pgp_none
     set fh(pgpOutput) ""
     set fh(browse) $browse
+    set fh(toplevel) [winfo toplevel $handler]
 
     # Enable updates & clear data
     $handler configure -state normal
@@ -56,8 +142,9 @@ proc Show {handler msg browse} {
     eval "$handler tag delete [$handler tag names]"
 
     # Reset the tags to a known status
-    $handler tag configure HeaderName -font [GetFont $langCharset 0 bold]
+    $handler tag configure HeaderName -font $fixedBoldFont
     $handler tag configure Center -justify center -spacing1 5m -spacing3 5m
+    $handler tag configure CenterNoSpacing -justify center
     if { 4 < [winfo cells $handler]} {
 	$handler tag configure URL -foreground $option(url_color) -underline 1
 	$handler tag configure Found -background #ffff80
@@ -110,7 +197,7 @@ proc Show {handler msg browse} {
 # header - List of header rows
 
 proc InsertHeader {gtag w header {width 1}} {
-    global t
+    global t showAddrHdr
     upvar #0 $w fh
 
     set hn [lindex $header 0]
@@ -119,27 +206,19 @@ proc InsertHeader {gtag w header {width 1}} {
 	set hn $t($hni)
     }
     $w insert insert [format "%${width}s: " $hn] "HeaderName $gtag"
-
-    foreach h [lindex $header 1] {
-	set charset [lindex $h 0]
-	if [info exists fh(charset_$charset)] {
-	    set tag $fh(charset_$charset)
-	} else {
-	    set font [GetFont $charset 0 {}]
-	    if [string length $font] {
-		set tag t[incr fh(id)]
-		set fh(charset_$charset) $tag
-		$w tag configure $tag -font $font
+    if {-1 == [lsearch -exact $showAddrHdr $hni]} {
+	$w insert insert "[lindex $header 1]\n" $gtag
+    } else {
+	set first 1
+	foreach a [RatSplitAdr [lindex $header 1]] {
+	    if {0 == $first} {
+		$w insert insert ",\n[format "%${width}s  " ""]" $gtag
 	    }
+	    $w insert insert "[string trim $a]" $gtag
+	    set first 0
 	}
-	if [info exists tag] {
-	    $w insert insert [lindex $h 1] "$tag $gtag"
-	    unset tag
-	} else {
-	    $w insert insert " ??? "
-	}
+	$w insert insert "\n" $gtag
     }
-    $w insert insert "\n" $gtag
 }
 
 # ShowMessage --
@@ -181,13 +260,13 @@ proc ShowMessage {first handler msg} {
 
     switch $fh(show_header) {
     all	{
-	    foreach h [$msg headers split] {
+	    foreach h [$msg headers] {
 		InsertHeader $tag $handler $h
 	    }
 	}
     selected {
-	    foreach h [$msg headers split] {
-		set header([string tolower [lindex $h 0]]) [lindex $h 1]
+	    foreach h [$msg headers] {
+		lappend header([string tolower [lindex $h 0]]) [lindex $h 1]
 	    }
 	    set length 5
 	    foreach f $option(show_header_selection) {
@@ -198,7 +277,9 @@ proc ShowMessage {first handler msg} {
 	    foreach f $option(show_header_selection) {
 		set n [string tolower $f]
 		if [info exists header($n)] {
-		    InsertHeader $tag $handler [list $f $header($n)] $length
+		    foreach h $header($n) {
+			InsertHeader $tag $handler [list $f $h] $length
+		    }
 		}
 	    }
 	}
@@ -257,7 +338,7 @@ proc ShowMessage {first handler msg} {
 # msg     -	The message name
 
 proc ShowBody {handler body msg} {
-    global option
+    global option mimeHandler
     upvar #0 $handler fh \
     	     msgInfo_$msg msgInfo
 
@@ -293,18 +374,15 @@ proc ShowBody {handler body msg} {
     }
 
     # Switch for subroutines which does the actual drawing
-    switch -glob [lindex $type 0]/[lindex $type 1] {
-    text/plain			{ ShowTextPlain $handler $body $msg}
-    text/*			{ ShowTextOther $handler $body $msg}
-    multipart/alternative	{ ShowMultiAlt $handler $body $msg }
-    multipart/*			{ ShowMultiMixed $handler $body $msg }
-    message/rfc822		{ ShowMessage 0 $handler [$body message] }
-    message/delivery-status	{ ShowDSN $handler $body }
-    image/gif           	{ ShowImage $handler $body gif }
-    application/pgp-keys	{ ShowPGPKeys $handler $body }
-    application/pgp		{ ShowTextPlain $handler $body $msg}
-    default			{ ShowDefault $handler $body }
+    set tp [join $type /]
+    if [info exists mimeHandler($tp)] {
+	set mh $mimeHandler($tp)
+    } elseif [info exists mimeHandler([lindex $type 0])] {
+	set mh $mimeHandler([lindex $type 0])
+    } else {
+	set mh { ShowDefault $handler $body }
     }
+    eval $mh
 }
 
 # ShowTextPlain --
@@ -317,7 +395,7 @@ proc ShowBody {handler body msg} {
 # msg     -	The message name
 
 proc ShowTextPlain {handler body msg} {
-    global option b defaultFont
+    global option b charsetName propNormFont
     upvar #0 $handler fh \
     	     msgInfo_$msg msgInfo
 
@@ -325,45 +403,48 @@ proc ShowTextPlain {handler body msg} {
     $handler tag bind $tag <3> "tk_popup $fh(struct_menu) %X %Y \
 			        \[lsearch \[set ${handler}(struct_list)\] \
 			        $body\]"
-    set action [$body getShowCharset]
-    if ![string compare good [lindex $action 0]] {
-	$handler tag configure $tag -font [GetFont [lindex $action 1] 0 {}]
-	$handler insert insert [$body data false [lindex $action 1]] $tag
+
+    if [$body isGoodCharset] {
+	$handler insert insert [$body data false] $tag
     } else {
 	global t
 	set w $handler.w[incr fh(id)]
 	frame $w -relief raised -bd 2 -cursor top_left_arrow
 	if [string length [$body description]] {
 	    label $w.desc -text "$t(description): [$body description]" \
-		    -font $defaultFont
+		    -font $propNormFont
 	    pack $w.desc -side top -anchor nw
 	}
 	set charset [$body parameter charset]
 	label $w.type \
 	    	-text "$t(here_is_text) '$charset' $t(which_cant_be_shown)"
-	label $w.modlab -text $t(mode):
+	label $w.modlab -text $t(interpret_as):
 	menubutton $w.mode -menu $w.mode.m -indicatoron 1 -relief raised \
 	    	-textvariable ${handler}(mode,$body)
 	set b($w.mode) howto_display
-	button $w.save -text $t(save_to_file) -command "SaveBody $body"
+	button $w.save -text $t(save_to_file) \
+		-command "SaveBody $body $fh(toplevel)"
 	set b($w.save) save_bodypart
 	pack $w.type -side top -anchor w
 	pack $w.modlab $w.mode -side left
 	pack $w.save -side left -padx 20
 	set l 0
 	menu $w.mode.m -tearoff 0
-	foreach s {dont_show show_sevenbit show_all} {
-	    $w.mode.m add command -label $t($s) \
-		    -command "ShowTextCharset $body $msg $handler $handler \
-					      $tag $s"
-	    if {[string length $t($s)] > $l} {
-		set l [string length $t($s)]
-		$w.mode configure -width $l
+	foreach c $option(charsets) {
+	    if {0 < [string length $charsetName($c)]} {
+		set name "$c ($charsetName($c))"
+	    } else {
+		set name $c
 	    }
+	    $w.mode.m add command \
+		    -label $name \
+		    -command "ShowTextCharset $body $msg $handler $handler \
+					      $tag $c"
 	}
 	if ![info exists msgInfo(show,$body,how)] {
-	    set msgInfo(show,$body,how) show_sevenbit
+	    set msgInfo(show,$body,how) $option(charset)
 	}
+	set fh(mode,$body) $msgInfo(show,$body,how)
 	$handler window create insert -window $w -pady 5
 	foreach win [concat $w [pack slaves $w]] {
 	    bind $win <3> "tk_popup $fh(struct_menu) %X %Y \
@@ -380,6 +461,54 @@ proc ShowTextPlain {handler body msg} {
     }
 }
 
+# ShowTextHtml --
+#
+# Show text/html entities, should handle different fonts...
+#
+# Arguments:
+# handler -	The handler which identifies the show text widget
+# body    -	The bodypart to show
+# msg     -	The message name
+proc ShowTextHtml {handler body msg} {
+    upvar #0 $handler fh \
+    	     msgInfo_$msg msgInfo
+
+    set tag t[incr fh(id)]
+    set frame [frame $handler.f[incr fh(id)] -width [winfo width $handler]\
+		-height [winfo height $handler] -cursor left_ptr]
+    set htmlwin [html $frame.html \
+		-xscrollcommand [list $frame.xscroll set] \
+		-yscrollcommand [list $frame.yscroll set]]
+    $htmlwin parse [$body data false]
+    set yscroll [scrollbar $frame.yscroll -command [list $htmlwin yview]]
+    set xscroll [scrollbar $frame.xscroll -command [list $htmlwin xview] \
+		-orient horizontal]
+    bind $frame <Destroy> {
+        bind [winfo parent %W] <Configure> {}
+    }
+    grid $htmlwin -row 0 -col 0 -sticky news
+    grid $yscroll -row 0 -col 1 -sticky ns
+    grid $xscroll -row 1 -col 0 -sticky ew
+    grid columnconfigure $frame 0 -weight 1
+    grid rowconfigure $frame 0 -weight 1
+    grid propagate $frame 0
+    $handler insert insert " " "Center $tag"
+    $handler window create insert -window $frame
+    $handler insert insert "\n" $tag
+    # lappend fh(show_cleanup_cmds) "image delete $img"
+    $handler tag bind $tag <3> "tk_popup $fh(struct_menu) %X %Y \
+				 \[lsearch \[set ${handler}(struct_list)\] \
+				 $body\]"
+    set binding [list ResizeFrame $frame $handler -1 -1 \
+           $xscroll $yscroll]
+    if {[string first $binding [bind $handler <Configure>]] == -1} {
+        bind $handler <Configure> +$binding
+	}
+    bind $htmlwin.x <3> "tk_popup $fh(struct_menu) %X %Y \
+			  \[lsearch \[set ${handler}(struct_list)\] \
+			  $body\]"
+}
+
 # ShowTextCharset --
 #
 # Shows text in a nonstandard character set
@@ -390,29 +519,24 @@ proc ShowTextPlain {handler body msg} {
 # hd	  -	The array to use for global variables
 # w 	  -	The handler which identifies the show text widget
 # tag	  -	The tag this entity should have
-# mode	  -	What to show
+# charset -	What to assume
 
-proc ShowTextCharset {body msg hd w tag mode} {
+proc ShowTextCharset {body msg hd w tag charset} {
     upvar #0 $hd fh \
     	     msgInfo_$msg msgInfo
     global t
 
+    set oldState [$w cget -state]
     $w configure -state normal
+    $w mark set oldInsert insert
     $w delete ${body}_s ${body}_e
     $w mark set insert ${body}_s
 
-    set fh(mode,$body) $t($mode)
-    set msgInfo(show,$body,how) $mode
-
-    switch $mode {
-    show_sevenbit {
-	    $w insert insert [$body data false 7bit] $tag
-	}
-    show_all {
-	    $w insert insert [$body data false [$body parameter charset]] $tag
-	}
-    }
-    $w configure -state disabled
+    set fh(mode,$body) $charset
+    set msgInfo(show,$body,how) $charset
+    $w insert insert [$body data false $charset] $tag
+    $w mark set insert oldInsert
+    $w configure -state $oldState
 }
 
 # ShowTextOther --
@@ -425,7 +549,7 @@ proc ShowTextCharset {body msg hd w tag mode} {
 # msg     -	The message name
 
 proc ShowTextOther {handler body msg} {
-    global t b defaultFont langCharset
+    global t b fixedBoldFont
     upvar #0 $handler fh \
     	     msgInfo_$msg msgInfo
 
@@ -443,7 +567,7 @@ proc ShowTextOther {handler body msg} {
 	}
     }
     text $w.text -relief flat -cursor top_left_arrow
-    $w.text tag configure Bold -font [GetFont $langCharset 0 bold]
+    $w.text tag configure Bold -font $fixedBoldFont
     if [string length [$body description]] {
 	set width [string length "$t(description): [$body description]"]
 	$w.text insert insert "$t(description): " Bold [$body description]\n
@@ -486,7 +610,8 @@ proc ShowTextOther {handler body msg} {
 	    -variable msgInfo_${msg}(show,$body,tp) \
 	    -command "ShowTextOtherDo $handler $body $msg" \
 	    -padx 4 -pady 4
-    button $w.save -text $t(save_to_file) -command "SaveBody $body"
+    button $w.save -text $t(save_to_file) \
+	    -command "SaveBody $body $fh(toplevel)"
     button $w.view_int -text $t(view_source) -command "ShowSource $body"
     pack $w.tp $w.save $w.view_int -side left -padx 5
     set b($w.tp) view_as_text
@@ -521,13 +646,14 @@ proc ShowTextOtherDo {handler body msg} {
     upvar #0 $handler fh \
     	     msgInfo_$msg msgInfo
     
+    set oldState [$handler cget -state]
     $handler configure -state normal
     $handler delete ${body}_s ${body}_e
     $handler mark set insert ${body}_s
     if $msgInfo(show,$body,tp) {
 	ShowTextPlain $handler $body $msg
     }
-    $handler configure -state disabled
+    $handler configure -state $oldState
 }
 
 # ShowMultiAlt --
@@ -543,39 +669,67 @@ proc ShowMultiAlt {handler body msg} {
     global GoodTypes
     upvar #0 $handler fh \
     	     msgInfo_$msg msgInfo
-    set found 0
 
     if ![info exists msgInfo(alternated,$body)] {
-        set msgInfo(alternated,$body) 1
-        set children [$body children]
-        foreach c $children {
-            set msgInfo(show,$c) 0
+	set found 0
+	set msgInfo(alternated,$body) 1
+	set children [$body children]
+	foreach c $children {
+	    set msgInfo(show,$c) 0
 	}
-        for {set i [expr [llength $children]-1]} {$i >= 0} {incr i -1} {
-            set child [lindex $children $i]
+	for {set i [expr [llength $children]-1]} {$i >= 0} {incr i -1} {
+	    set child [lindex $children $i]
 	    set tc $child
-            set typelist [$child type]
+	    set typelist [$child type]
 	    while {1 == [regexp -nocase multipart [lindex $typelist 0]]} {
 		set tc [lindex [$tc children] 0]
 		set typelist [$tc type]
 	    }
-            set type [string tolower [lindex $typelist 0]/[lindex $typelist 1]]
-            if { -1 != [lsearch $GoodTypes $type] } {
+	    set type [string tolower [lindex $typelist 0]/[lindex $typelist 1]]
+	    if { -1 != [lsearch $GoodTypes $type] } {
 		set msgInfo(show,$child) 1
 		set found 1
-                break
-            }
-        }
+	        break
+	    }
+	}
+	if !$found {
+	    foreach child [$body children] {
+		set msgInfo(show,$child) 1
+	    }
+	}
     }
 
     foreach child [$body children] {
-	if !$found {
-            set msgInfo(show,$child) 1
-	}
 	ShowBody $handler $child $msg
     }
 }
 
+# ShowMultiEnc --
+#
+# Show a multipart/encrypted object (which we failed to decrypt)
+#
+# Arguments:
+# handler -	The handler which identifies the show text widget
+# body    -	The bodypart to show
+# msg     -	The message name
+
+proc ShowMultiEnc {handler body msg} {
+    upvar #0 $handler fh
+
+    set children [$body children]
+    if {[llength $children] != 2} {
+	return
+    }
+    set b [lindex $children 1]
+    set tag t[incr fh(id)]
+    $handler tag bind $tag <3> "tk_popup $fh(struct_menu) %X %Y \
+			        \[lsearch \[set ${handler}(struct_list)\] \
+			        $body\]"
+
+    regsub -all "\r" [$b data false] "" data
+    $handler insert insert $data $tag
+}
+
 # ShowMultiMixed --
 #
 # Show a multipart/mixed object
@@ -598,9 +752,9 @@ proc ShowMultiMixed {handler body msg} {
     }
 }
 
-# ShowImageGif --
+# ShowImage --
 #
-# Show image/gif entities
+# Show image/* entities
 #
 # Arguments:
 # handler -	The handler which identifies the show text widget
@@ -609,36 +763,73 @@ proc ShowMultiMixed {handler body msg} {
 
 proc ShowImage {handler body subtype} {
     global option
+    global ImageTypes
     upvar #0 $handler fh
 
-    set tag t[incr fh(id)]
-    set filename $option(tmp)/rat.[RatGenId]
-    set fid [open $filename w 0600]
-    $body saveData $fid 0 0
-    close $fid
-    switch $subtype {
-    gif    {
-	    if [catch {image create photo -format gif -file $filename} img] {
-		Popup $img
-		ShowDefault $handler $body
-		return
-	    }
+    if {[lsearch $ImageTypes $subtype] != "-1"} {
+	set tag t[incr fh(id)]
+	set filename $option(tmp)/rat.[RatGenId]
+	set fid [open $filename w 0600]
+	fconfigure $fid -encoding binary
+	$body saveData $fid 0 0
+	close $fid
+	if [catch {image create photo -file $filename} img] {
+	    Popup $img $fh(toplevel)
+	    file delete -force -- $filename
+	    ShowDefault $handler $body
+	    return
 	}
+	file delete -force -- $filename
+	set imgheight [image height $img]
+	set imgwidth [image width $img]
+    set frame [frame $handler.f[incr fh(id)] -cursor left_ptr]
+    set c [canvas $frame.canvas -width $imgwidth \
+		    -height $imgheight \
+		    -xscrollcommand [list $frame.xscroll set] \
+		    -yscrollcommand [list $frame.yscroll set] \
+		    -scrollregion [list 0 0 $imgwidth $imgheight]]
+	set yscroll [scrollbar $frame.yscroll -command [list $c yview]]
+	set xscroll [scrollbar $frame.xscroll -command [list $c xview] \
+		    -orient horizontal]
+    grid $c -row 0 -col 0 -sticky news
+    if {[expr $imgheight>[winfo height $handler]]} {
+	$frame configure -height [winfo height $handler]
+	grid $yscroll -row 0 -col 1 -sticky ns
+    } else {
+        $frame configure -height $imgheight
+    }
+    if {[expr $imgwidth>[winfo width $handler]]} {
+        $frame configure -width [winfo width $handler]
+	grid $xscroll -row 1 -col 0 -sticky ew
+    } else {
+        $frame configure -width $imgwidth
+    }
+	grid columnconfigure $frame 0 -weight 1
+	grid rowconfigure $frame 0 -weight 1
+	grid propagate $frame 0
+	$c create image 0 0 -image $img -anchor nw
+	$handler insert insert "\n" $tag	;# virtual spacing1
+	$handler insert insert " " "CenterNoSpacing $tag"
+	$handler window create insert -window $frame
+	$handler insert insert "\n\n" $tag	;# NL plus virtual spacing3
+	lappend fh(show_cleanup_cmds) "image delete $img"
+	$handler tag bind $tag <3> "tk_popup $fh(struct_menu) %X %Y \
+				     \[lsearch \[set ${handler}(struct_list)\] \
+				     $body\]"
+    set binding [list ResizeFrame $frame $handler \
+          $imgheight $imgwidth $xscroll $yscroll]
+    if {[string first $binding [bind $handler <Configure>]] == -1} {
+        bind $handler <Configure> +$binding 
+	}
+	bind $c <3> "tk_popup $fh(struct_menu) %X %Y \
+			      \[lsearch \[set ${handler}(struct_list)\] \
+			      $body\]"
+	update
+	ResizeFrame $frame $handler \
+		$imgheight $imgwidth $xscroll $yscroll
+    } else {
+        ShowDefault $handler $body
     }
-    file delete -force -- $filename
-    set c [canvas $handler.c[incr fh(id)] -width [image width $img] \
-	    -height [image height $img]]
-    $c create image 0 0 -image $img -anchor nw
-    $handler insert insert " " "Center $tag"
-    $handler window create insert -window $c
-    $handler insert insert "\n" $tag
-    lappend fh(show_cleanup_cmds) "image delete $img"
-    $handler tag bind $tag <3> "tk_popup $fh(struct_menu) %X %Y \
-				 \[lsearch \[set ${handler}(struct_list)\] \
-				 $body\]"
-    bind $c <3> "tk_popup $fh(struct_menu) %X %Y \
-			  \[lsearch \[set ${handler}(struct_list)\] \
-			  $body\]"
 }
 
 
@@ -709,7 +900,8 @@ proc ShowPGPKeys {handler body} {
 
     button $w.add -text $t(add_to_keyring) \
 	    -command "RatPGP add \[$body data 0\]"
-    button $w.save -text $t(save_to_file) -command "SaveBody $body"
+    button $w.save -text $t(save_to_file) \
+	    -command "SaveBody $body $fh(toplevel)"
     button $w.view_int -text $t(view_source) -command "ShowSource $body"
     set b($w.add) add_to_keyring
     set b($w.save) save_bodypart
@@ -734,7 +926,7 @@ proc ShowPGPKeys {handler body} {
 # body    -	The bodypart to show
 
 proc ShowDefault {handler body} {
-    global t defaultFont langCharset
+    global t fixedBoldFont
     upvar #0 $handler fh
 
     set type [$body type]
@@ -751,7 +943,7 @@ proc ShowDefault {handler body} {
 	}
     }
     text $w.text -relief flat -cursor top_left_arrow
-    $w.text tag configure Bold -font [GetFont $langCharset 0 bold]
+    $w.text tag configure Bold -font $fixedBoldFont
     if [string length [$body description]] {
 	set width [string length "$t(description): [$body description]"]
 	$w.text insert insert "$t(description): " Bold [$body description]\n
@@ -781,7 +973,8 @@ proc ShowDefault {handler body} {
 	set b($w.view_ext) run_mailcap
 	pack $w.view_ext -side left -padx 10
     }
-    button $w.save -text $t(save_to_file) -command "SaveBody $body"
+    button $w.save -text $t(save_to_file) \
+	    -command "SaveBody $body $fh(toplevel)"
     button $w.view_int -text $t(view_source) -command "ShowSource $body"
     set b($w.save) save_bodypart
     set b($w.view_int) view_source
@@ -867,81 +1060,25 @@ proc ShowLineUp {handler} {
 #
 # Arguments:
 # body -	The bodypart to save
+# parent -	Parent of window
 
-proc SaveBody {body} {
+proc SaveBody {body parent} {
     global idCnt t option
 
-    # Create identifier
-    set id showSave[incr idCnt]
-    set w .$id
-    upvar #0 $id hd
-    set hd(done) 0
-    set oldFocus [focus]
+    set convertNL [regexp -nocase text [lindex [$body type] 0]]
 
-    if [regexp -nocase text [lindex [$body type] 0]] {
-	set hd(convertNL) 1
+    set filename [rat_fbox::run -ok $t(save) -mode save -parent $parent \
+	    -title $t(save_to_file) -initialfile [$body filename]]
+    if {"" == $filename} {
+	return
+    }
+
+    if { 0 == [catch [list open $filename w] fh]} {
+	$body saveData $fh false $convertNL
+	close $fh
     } else {
-	set hd(convertNL) 0
+	RatLog 4 "$t(save_failed): $fh"
     }
-
-    # Create toplevel
-    toplevel $w
-    wm title $w $t(save_to_file)
-
-    # Filechooser
-    set hd(name) [$body filename]
-    frame $w.chooser
-    set fh [FileSelectorCreate $w.chooser ${id}(name) "set ${id}(done) 1" 0 0]
-
-    # Newline conversion
-    checkbutton $w.localnl -text $t(convert_to_local_nl) \
-	    -variable ${id}(convertNL)
-
-    # OK and cancel buttons
-    frame $w.buttons
-    button $w.buttons.ok -text $t(ok) -command "FileSelectorUpdate $fh 1"
-    CreateModeMenu $w.buttons.mode $id $option(permissions)
-    button $w.buttons.cancel -text $t(cancel) -command "set ${id}(done) 0"
-    pack $w.buttons.ok \
-	 $w.buttons.mode \
-	 $w.buttons.cancel -side left -expand 1
-
-    # Pack it
-    pack $w.buttons -side bottom -fill x
-    pack $w.localnl -side bottom -anchor w -pady 5 -padx 5
-    pack $w.chooser -fill both -expand 1
-
-    Place $w saveBody
-    grab $w
-    tkwait variable ${id}(done)
-
-    # Do save
-    if { 1 == $hd(done) } {
-	FileSelectorDone $fh
-	if [file exists $hd(name)] {
-	    set action [RatDialog $t(file_exists) \
-				  "$hd(name) $t(already_exists)." \
-				  {} 1 $t(overwrite) $t(cancel)]
-	} else {
-	    set action 0
-	}
-	if {0 == $action} {
-	    set mode [GetMode $id]
-	    if { 0 == [catch [list open $hd(name) w $mode] fh]} {
-		$body saveData $fh false $hd(convertNL)
-		close $fh
-		# Defeat the umask
-		catch {exec chmod [format %#o $perm] $hd(name)}
-	    } else {
-		RatLog 4 "$t(save_failed): $fh"
-	    }
-	}
-    }
-
-    RecordPos $w saveBody
-    destroy $w
-    unset hd
-    focus $oldFocus
 }
 
 # BuildStructMenu
@@ -969,6 +1106,7 @@ proc BuildStructMenu {handler msg} {
     if $fh(browse,$msg) { return }
 
     BuildStructEntry $handler $fh(struct_menu) $msg [$msg body] ""
+    FixMenu $fh(struct_menu)
 }
 
 # BuildStructEntry
@@ -1004,7 +1142,9 @@ proc BuildStructEntry {handler m msg body preamble} {
     switch -glob $type {
     message/rfc822
         {
-	    set message [$body message]
+	    if [catch {$body message} message] {
+		return
+	    }
 	    set fh(struct_list) [lreplace $fh(struct_list) end end $message]
 	    if [string length $message] {
 		BuildStructEntry $handler $m $message [$message body] \
@@ -1071,7 +1211,8 @@ proc PopulateStructEntry {handler m msg body type} {
     	    -variable msgInfo_${msg}(show,$body) \
     	    -command "Show $handler $fh(current) 0"
     set b($m,[$m index end]) bodypart_show
-    $m add command -label $t(save_to_file)... -command "SaveBody $body"
+    $m add command -label $t(save_to_file)... \
+	    -command "SaveBody $body $fh(toplevel)"
     set b($m,[$m index end]) bodypart_save
     $m add command -label $t(view_source)... -command "ShowSource $body"
     set b($m,[$m index end]) view_source
@@ -1086,7 +1227,7 @@ proc PopulateStructEntry {handler m msg body type} {
 # mailcap -	The mailcap entry to use
 
 proc RunMailcap {body mailcap} {
-    global option t idCnt defaultFont
+    global option t idCnt
 
     set id id[incr idCnt]
     upvar #0 $id rm
@@ -1094,7 +1235,7 @@ proc RunMailcap {body mailcap} {
 
     # Fix files
     set rm(fileName) $option(tmp)/rat.[RatGenId]
-    if [regexp %s $cmd] {
+    if {[regexp %s $cmd]} {
 	regsub -all %s $cmd $rm(fileName) cmd
     } else {
 	if [lindex $mailcap 1] {
@@ -1113,13 +1254,12 @@ proc RunMailcap {body mailcap} {
     } elseif [lindex $mailcap 2] {
 	# This command produces lots of output
 	set w .$id
-	toplevel $w
+	toplevel $w -class TkRat
 	wm title $w $t(external_viewer)
 	text $w.text \
 		-yscroll "$w.scroll set" \
 		-relief sunken -bd 1 \
 		-setgrid 1 \
-		-font $defaultFont \
 		-highlightthickness 0
 	Size $w.text extView
 	scrollbar $w.scroll \
@@ -1172,7 +1312,7 @@ proc RunMailcapDone {name1 name2 op} {
 # x, y	  -	Mouse coordinates
 
 proc RatShowURL {url win tag x y} {
-    global option
+    global option urlstatus
 
     # Flash URL for feedback
     $win tag configure $tag -underline 1
@@ -1187,18 +1327,55 @@ proc RatShowURL {url win tag x y} {
     switch $option(url_viewer) {
 	RatUP {
 		if [catch {RatUP_ShowURL $url} error] {
-		    Popup error
+		    Popup $error [winfo toplevel $win]
 		}
 	    }
-	netscape {
-		if [catch {eval {exec $option(netscape)} $option(netscape_args)\
-			{-remote "openURL($url)"} >&/dev/null}] {
-		    if [catch {eval {exec $option(netscape)} \
-			    $option(netscape_args) {$url} &} error] {
-			Popup $error
-		    }
+        netscape {
+		trace variable urlstatus($url) w \
+			[list RatMaybeStartNetscape $url]
+		regsub -all , $url %2c url
+		regsub -all {\$} $url %24 netscape_url
+		RatBgExec urlstatus($url) "$option(netscape) -remote \
+			\"openURL($netscape_url)\" >&/dev/null"
+            }
+        lynx {
+		regsub -all %u $option(lynx) $url cmd
+		if [catch {eval exec $cmd &} error] {
+		    Popup $error [winfo toplevel $win]
 		}
 	    }
+        other {
+		regsub -all %u $option(other_browser) $url cmd
+		if [catch {eval exec $cmd &} error] {
+		    Popup $error [winfo toplevel $win]
+		}
+	    }
+    }
+}
+
+# RatMaybeStartNetscape --
+#
+# Start netscape, if the first invokation failed.
+#
+# Arguments:
+# url		   - URL to show
+# name1, name2, op - trace information
+
+proc RatMaybeStartNetscape {url name1 name2 op} {
+    upvar #0 ${name1}($name2) status
+
+    trace vdelete ${name1}($name2) w [list RatMaybeStartNetscape $url]
+    if {0 == $status} {
+	unset status
+	return
+    }
+
+    regsub -all , $url %2c url
+    regsub -all {\$} $url %24 url
+    unset status
+    global option
+    if [catch {eval exec $option(netscape) "$url" &} error] {
+	Popup $error
     }
 }
 
@@ -1214,7 +1391,7 @@ proc RatShowURL {url win tag x y} {
 proc RatFindURL {t start end} {
     global option
 
-    set exp "([join $option(urlprot) |]):/+\[^ \n\t\"\>]+\[^ \n\t,\.\)>\'\"\]"
+    set exp "([join $option(urlprot) |])://(\[a-zA-Z0-9\\-\\.@:\]+)+(:(\[0-9\]+))?(/\[a-zA-Z0-9/%#\\.\\-_;\\?:@&=\\+\\$!,~]*)?\[a-zA-Z0-9/%#\]"
     set found [$t search -nocase -regexp -count len $exp $start $end]
     if [string length $found] {
 	return [list $found $found+${len}c]
@@ -1252,7 +1429,7 @@ proc RatScrollShow {t scrollw sfrac efrac} {
 	set e [lindex $result 1]
 	$t tag add URL $s $e
 	$t tag add $tag $s $e
-	regsub % [$t get $s $e] %% url
+	regsub -all % [$t get $s $e] %% url
 	$t tag bind $tag <ButtonRelease-1> [list RatShowURL $url %W $tag %x %y]
 	$t tag bind $tag <1> \
 		"$t tag configure $tag -underline 0; update idletasks"
@@ -1260,3 +1437,64 @@ proc RatScrollShow {t scrollw sfrac efrac} {
     }
     $t mark set searched $end
 }
+
+# ResizeFrame --
+#
+# Resize frames containing scrolled information
+#
+# Arguments:
+# frame   - The frame to resize
+# parent  - The frame's parent
+# reqHeight - required height to see the scrollable item completely
+# reqWidth  - required width to see the scrollable item completely
+# xscroll   - The X axis scrollbar
+# yscroll   - The Y axis scrollbar
+#
+# Note:
+# A value of -1 for reqWidth or reqHeight will show both scrollbars all the
+# time. The frame will then take the entire available space of the parent.
+
+proc ResizeFrame {frame parent reqHeight reqWidth xscroll yscroll} {
+    set parWidth [winfo width $parent]
+    set parHeight [winfo height $parent]
+    set totalHeight $reqHeight
+    set totalWidth $reqWidth
+
+    if {![winfo exists $frame]} {
+        return
+    }
+
+    # Take care of the easy stuff first. If reqWeight or reqHeight is -1 then
+    # we *always* want scrollbars
+    
+    if {$reqHeight == -1 || $reqHeight == -1} {
+        $frame configure -height $parHeight -width $parWidth
+        return
+    }
+
+    # Determine if we need scrollbars. The deal is that if reqWidth or
+    # reqHeight is -1 then always put the scrollbars
+    if {[expr {$reqHeight > $parHeight}]} {
+        incr totalWidth [winfo width $yscroll]
+    }
+    if {[expr {$reqWidth > $parWidth}]} {
+        incr totalHeight [winfo height $xscroll]
+    }
+
+    if {[expr {$totalHeight > $parHeight}]} {
+        $frame configure -height $parHeight -bg blue
+        grid $yscroll -row 0 -col 1 -sticky ns
+    } else {
+        $frame configure -height $totalHeight
+        grid forget $yscroll
+    }
+
+    if {[expr {$totalWidth > $parWidth}]} {
+        $frame configure -width $parWidth
+        grid $xscroll -row 1 -col 0 -sticky ew
+    } else {
+        $frame configure -width $totalWidth
+        grid forget $xscroll
+    }
+
+}
